/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

/* Standard includes. */
#include <stdio.h>
#include <string.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Hardware library includes. */
#include "compiled_in_motor.h"
#include "motor_control.h"
#include "board.h"
#include "load_statistics.h"
#include "protocol.h"
#include "stall_detect.h"
#include "debug.h"
#include "dso.h"
#include "turn_control.h"
#include "motion_control.h"
#include "peripherals_control.h"
#include "config_storage.h"

#include "config.h"
#include BOARD_CONFIG_HEADER_FILE
#include "hal_wdt.h"

#include "led.h"
#include "rgb_led.h"
#include "ui.h"
#include "can.h"

extern void StandaloneApplication(void* pvParameters);

static xTaskHandle xSystemInit;                                                 /*!< Handle of the init task - for suicide */

/*! \brief  Application Stack Overflow
  *
  * Generic Function needed by FreeRTOS
  * Will be called in case a Task has exceeded its Stack Size
  *
  * If you come here - please check your stack sizes!!
  *
  * @param  pxTask: Handle of Task
  * @param  pxTask: Name  of Task
  * @retval None
*/
void vApplicationStackOverflowHook(xTaskHandle* pxTask, signed portCHAR* pcTaskName)
{

  /* This function will get called if a task overflows its stack. */
  dprintf("pxTask : %p\n",pxTask);
  dprintf("pcTaskName:%s\n",pcTaskName);

  for(;;);
}


/*! \brief  System Init
  *
  * @param  pvParameters: None
  * @retval None
*/
void system_init( void *pvParameters )
{
#ifdef USE_CONFIG_STORAGE
  int ret;
#endif /*  USE_CONFIG_STORAGE */
  
#ifdef MOTOR_CHANNEL_0  
  ChannelUsageConfiguration[0] = CHANNEL_IS_USED;
#endif /* MOTOR_CHANNEL_0 */
  
#ifdef MOTOR_CHANNEL_1  
  ChannelUsageConfiguration[1] = CHANNEL_IS_USED;
#endif /* MOTOR_CHANNEL_1 */

#ifdef MOTOR_CHANNEL_2    
  ChannelUsageConfiguration[2] = CHANNEL_IS_USED;
#endif /* MOTOR_CHANNEL_2 */  
  
  BOARD_SetupHW();                                                              /* Setup HW components on the Board (internal/external) */

  /* Clear all settings */
  memset(&MotorParameterValues[0], 0,sizeof(MotorParameterValues));
  memset(&PIControl[0],            0,sizeof(PIControl));
  memset(&SystemValues[0],         0,sizeof(SystemValues));

  /* Make some usefull basic setup - otherwise system will crash */
  SystemValues[0].PwmFrequency     = 16000;
  SystemValues[1].PwmFrequency     = 16000;
  SystemValues[2].PwmFrequency     = 16000;

  /* Read back parameters stored in EEProm */
#ifdef USE_CONFIG_STORAGE
  ret = config_storage_load_config();
  if (ret)
  {
#ifdef USE_INTERNAL_MOTOR_PARAMS
    SetupInternalMotorParams();                                                 /* Use compiled in Parameter setting on error */
#endif /* USE_INTERNAL_MOTOR_PARAMS */
#ifdef USE_RGB_LED
    RGB_LED_SetValue(0);
    RGB_LED_StoreValue();
#endif /* USE_RGB_LED */
  }
  else
  {
#ifdef USE_LED
    LED_SetState(LED_SIGNAL_CONFIG_READ,LED_ON);                                /* Signal successful read-back */
#endif /* USE_LED */
#ifdef USE_RGB_LED
    RGB_LED_SetValue(LED_RGB_BLUE);
    RGB_LED_StoreValue();
#endif /* USE_RGB_LED */
  }

#else
  SetupInternalMotorParams();                                                   /* When no EEProm - use compiled in parameters */
#endif /* USE_CONFIG_STORAGE */
  
  
  BOARD_SetupHW2();                                                             /* Setup rest of the HW components on the Board */
  BOARD_SetupWDT();
   
  printf("Init done\n");

  /* Kill myself to free stack memory */
  vTaskDelete(xSystemInit);
}

int main(void)
{
  IRQn_Type nr;

  SCB->SHCSR = SCB_SHCSR_MEMFAULTENA_Msk                                        /* enable Fault Interrupts */
              |SCB_SHCSR_BUSFAULTENA_Msk
              |SCB_SHCSR_USGFAULTENA_Msk;

  BOARD_SetupClocks();
  BOARD_SetupOFD();

  /* INTF_IRQn is the "last" interrupt for all
  supported platforms at the moment. */
#if (defined __TMPM_370__) || (defined __TMPM_372__) || (defined __TMPM_373__) || (defined __TMPM_374__)
  for (nr = (IRQn_Type)0; nr < INTF_IRQn; nr++)
    NVIC_SetPriority(nr, ~0);
#endif

#if (defined __TMPM_375__) || (defined __TMPM_376__)
  for (nr = (IRQn_Type)0; nr < INTSBI0_IRQn; nr++)
    NVIC_SetPriority(nr, ~0);
#endif

#if (defined __TMPM_475__)
  for (nr = (IRQn_Type)0; nr < INTENC11_IRQn  ; nr++)
    NVIC_SetPriority(nr, ~0);
#endif

#if (defined __TMPM_4KN__)
  for (nr = (IRQn_Type)0; nr < INTFLDRDY_IRQn  ; nr++)
    NVIC_SetPriority(nr, ~0);
#endif


  if ( xTaskCreate(system_init,                                                 /* Create an init task - needed as some of the inits use a task delay */
                   (char const*) "SystemInit",
                   INIT_TASK_STACK_SIZE,
                   NULL,
                   INIT_TASK_PRIORITY,
                   &xSystemInit) != pdPASS )
    dprintf("Can't create System Init Task\n");

#ifdef USE_LOAD_STATISTICS
  if ( xTaskCreate(SystemLoadTaks,                                              /* Create System Load Task */
                   (char const*) "SysLoad",
                   SYSTEM_LOAD_TASK_STACK_SIZE,
                   NULL,
                   SYSTEM_LOAD_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create System Load Task\n");
#endif /* USE_LOAD_STATISTICS */

#ifdef USE_UI
  if ( xTaskCreate(UITask,                                                      /* Create User Interface */
                   (char const*) "UI",
                   UI_TASK_STACK_SIZE,
                   NULL,
                   UI_TASK_PRIORITY,
                   NULL) != pdPASS )
    dprintf("Can't create UI Task\n");
#endif /* USE_UI */

#ifdef USE_SERIAL_COMMUNICATION
  if ( xTaskCreate(ProtocolTask,                                                /* Create serial communication protocol task */
                   (char const*) "Comm",
                   PROTOCOL_TASK_STACK_SIZE,
                   NULL,
                   PROTOCOL_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create Protocol Task\n");
#endif /* USE_SERIAL_COMMUNICATION */

#ifdef USE_CAN
  if ( xTaskCreate(CanTask,                                                     /* Create CAN Communication Protocol Task */
                   (char const*) "Can",
                   CAN_TASK_STACK_SIZE,
                   NULL,
                   CAN_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create CAN Task\n");
#endif /* USE_CAN */

#ifdef USE_STALL_DETECT
  if ( xTaskCreate(StallTask,                                                   /* Create Stall Detection Task */
                   (char const*) "Stall",
                   STALL_TASK_STACK_SIZE,
                   NULL,
                   STALL_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create Stall Task\n");
#endif /* USE_STALL_DETECT */

#ifdef USE_TURN_CONTROL
#if defined __TMPM_370__ || defined __TMPM_4KN__
  if ( xTaskCreate(TurnTask,                                                    /* Create Turn Task Channel 0*/
                   (char const*) "Turn0",
                   TURN_TASK_STACK_SIZE,
                   (void*)0,
                   TURN_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create Turn Task\n");
#endif /* USE_TURN_CONTROL */
  if ( xTaskCreate(TurnTask,                                                    /* Create Turn Task Channel 1*/
                   (char const*) "Turn1",
                   TURN_TASK_STACK_SIZE,
                   (void*)1,
                   TURN_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create Turn Task\n");

#ifdef __TMPM_4KN__
  if ( xTaskCreate(TurnTask,                                                    /* Create Turn Task Channel 0*/
                   (char const*) "Turn2",
                   TURN_TASK_STACK_SIZE,
                   (void*)2,
                   TURN_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create Turn Task\n");
#endif /* USE_TURN_CONTROL */


#endif /* USE_MOTION_CONTROL */


#ifdef USE_MOTION_CONTROL
  if ( xTaskCreate(MotionTask,                                                    /* Create Turn Task Channel 0*/
                   (char const*) "Motion0",
                   MOTION_TASK_STACK_SIZE,
                   (void*)0,
                   MOTION_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create motion Task\n");
  if ( xTaskCreate(MotionTask,                                                    /* Create Turn Task Channel 1*/
                   (char const*) "Motion1",
                   MOTION_TASK_STACK_SIZE,
                   (void*)1,
                   MOTION_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create motion Task\n");

  if ( xTaskCreate(MotionTask,                                                    /* Create Turn Task Channel 0*/
                   (char const*) "Motion2",
                   MOTION_TASK_STACK_SIZE,
                   (void*)2,
                   MOTION_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create motion Task\n");

#endif /* USE_MOTION_CONTROL */

#ifdef STANDALONE_APP
  if ( xTaskCreate(StandaloneApplication,
                   (char const*) "StandaloneApplication",
                   APP_TASK_STACK_SIZE,
                   (void*)0,
                   STANDALONE_APP_TASK_PRIORITY,
                   NULL) != pdPASS)
    dprintf("Can't create Standalone Apllication Task\n");
#endif


  vTaskStartScheduler();                                                        /* Start the scheduler. */

  /*
   * Will only get here if there was insufficient memory to create the idle task.
   * The idle task is created within vTaskStartScheduler().
   */
  for(;;);                                                                      /* Error */
}
