/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <string.h>

#include "config.h"

#include "FreeRTOS.h"
#include "task.h"

#include "drv8323.h"
#include "hal_spi.h"
#include BOARD_CONFIG_HEADER_FILE
#include BOARD_SPI_HEADER_FILE


#define   DRV8323_SPI_CHANNEL           (BOARD_SPI_CHANNEL)
#define   DRV8323_SPI_CS                (0)
#define   DRV8323_SPI_TRANSFER_MODE     (1)                                     /*!< This member configures the SPI mode: 1= master, 0 = slave */
#define   DRV8323_SPI_SPEED             (HAL_SPI_1_MHZ)                         /*!< speed for SPI port */
#define   DRV8323_SPI_DATA_LENGTH       (HAL_SPI_DATA_LENGTH_16)
#define   DRV8323_SPI_CK_EDGE           (HAL_SPI_CK_2ND_EDGE)
#define   DRV8323_SPI_CK_IDLE           (HAL_SPI_CK_IDLE_LOW)

#define   DRV8323_W0_READ_CMD           (0x8000)
#define   DRV8323_REG_ADDR_MASK         (0x0F)                                  /* only 4 bits used for the register address */
#define   DRV8323_REG_ADDR_SHIFT        (11)
#define   DRV8323_DATA_MASK             (0x3FF)                                 /* 11 bits used for the register value */

typedef enum
{
  FAULT_STATUS_1            = 0x00,
  FAULT_STATUS_2            = 0x01,
  DRV_CTRL                  = 0x02,
  GATE_DRV_HS               = 0x03,
  GATE_DRV_LS               = 0x04,
  OCP_CTRL                  = 0x05,
  CSA_CTRL                  = 0x06,
}DRV8323_REG_ADDR;

typedef enum
{
    DRV8323_READ_ONLY,
    DRV8323_READ_WRITE,
}DRV8323_Access;

const HAL_SPI_CfgData drv8323SpiCfgData = {
  DRV8323_SPI_CHANNEL      ,
  DRV8323_SPI_CS           ,
  DRV8323_SPI_TRANSFER_MODE,
  DRV8323_SPI_SPEED        ,
  DRV8323_SPI_DATA_LENGTH  ,
  DRV8323_SPI_CK_EDGE      ,
  DRV8323_SPI_CK_IDLE      ,
  (void*) 0 ,
  (void*) 0 ,
};

static DRV8323_RETURN DRV8323_ReadWriteRegister(DRV8323_REG_ADDR regAddr, uint16_t *data, DRV8323_Access access)
{
    HAL_SPI_TransRec_t transferReceive;                                           /* value of register */

    if (HAL_SPI_NO_ERR != HAL_SPI_Init(&drv8323SpiCfgData))
    {
      return DRV8323_SPI_ACCESS_ERROR;
    }

    (*data) = (((uint16_t)regAddr & DRV8323_REG_ADDR_MASK) << DRV8323_REG_ADDR_SHIFT)  | (*data & DRV8323_DATA_MASK);

    if (DRV8323_READ_ONLY == access)
    {
      /* set read flag */
      (*data) |= DRV8323_W0_READ_CMD;
    }
    transferReceive.trx16.p_data = data;
    transferReceive.trx16.num = 1;

    if (HAL_SPI_NO_ERR != HAL_SPI_MasterWriteBytes(DRV8323_SPI_CHANNEL, &transferReceive))
    {
      return  DRV8323_SPI_ACCESS_ERROR;
    }

    /* dummy read to clear receive buffer */
    if (HAL_SPI_NO_ERR != HAL_SPI_MasterReadBytes (DRV8323_SPI_CHANNEL, &transferReceive))
    {
      return  DRV8323_SPI_ACCESS_ERROR;
    }

    /* mask valid response data */
    (*data) &= DRV8323_DATA_MASK;

    return DRV8323_SUCCESS;
}

DRV8323_RETURN DRV8323_ReadFaultStatus1Register(DRV8323_FaultStatus1 *data)
{
  return  DRV8323_ReadWriteRegister(FAULT_STATUS_1, (uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_ReadFaultStatus2Register(DRV8323_FaultStatus2 *data)
{
  return  DRV8323_ReadWriteRegister(FAULT_STATUS_2, (uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_ReadDrvCtrlRegister(DRV8323_DrvCtrl *data)
{
  return  DRV8323_ReadWriteRegister(DRV_CTRL, (uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_WriteDrvCtrlRegister(DRV8323_DrvCtrl data)
{
  return DRV8323_ReadWriteRegister(DRV_CTRL, (uint16_t*)&data, DRV8323_READ_WRITE);
}

DRV8323_RETURN DRV8323_ReadGateDrvHsRegister(DRV8323_GateDrvHs *data)
{
  return  DRV8323_ReadWriteRegister(GATE_DRV_HS, (uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_WriteGateDrvHsRegister(DRV8323_GateDrvHs data)
{
  return DRV8323_ReadWriteRegister(GATE_DRV_HS, (uint16_t*)&data, DRV8323_READ_WRITE);
}

DRV8323_RETURN DRV8323_ReadGateDrvLsRegister(DRV8323_GateDrvLs *data)
{
  return  DRV8323_ReadWriteRegister(GATE_DRV_LS,(uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_WriteGateDrvLsRegister(DRV8323_GateDrvLs data)
{
  return DRV8323_ReadWriteRegister(GATE_DRV_LS, (uint16_t*)&data, DRV8323_READ_WRITE);
}

DRV8323_RETURN DRV8323_ReadOcpCtrlRegister(DRV8323_OcpCtrl *data)
{
  return  DRV8323_ReadWriteRegister(OCP_CTRL, (uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_WriteOcpRegister(DRV8323_OcpCtrl data)
{
  return DRV8323_ReadWriteRegister(OCP_CTRL, (uint16_t*)&data, DRV8323_READ_WRITE);
}

DRV8323_RETURN DRV8323_ReadCsaCtrlCtrlRegister(DRV8323_CsaCtrl *data)
{
  return  DRV8323_ReadWriteRegister(CSA_CTRL, (uint16_t*)data, DRV8323_READ_ONLY);
}

DRV8323_RETURN DRV8323_WriteCsaCtrlRegister(DRV8323_CsaCtrl data)
{
  return DRV8323_ReadWriteRegister(CSA_CTRL, (uint16_t*)&data, DRV8323_READ_WRITE);
}
