/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include "config.h"
#ifdef USE_LOAD_STATISTICS

#include "hal_wdt.h"

#include "FreeRTOS.h"
#include "task.h"
#include "board.h"
#include "units_enums.h"											/* system load in percent */

/* Idle counter for the different compilers */
#ifdef __TMPM_4KN__

#ifdef BOARD_CLICKER4_M4KN

  #if defined ( __CC_ARM   )
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #elif (defined (__ICCARM__))
      #define MAX_IDLE_COUNTER_NO_LOAD 10170000
  #elif (defined (__GNUC__))
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #else
      #error "Please define MAX_IDLE_COUNTER_NO_LOAD."
  #endif

#elif BOARD_SBK_M4KN

  #if defined ( __CC_ARM   )
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #elif (defined (__ICCARM__))
      #define MAX_IDLE_COUNTER_NO_LOAD 10170000
  #elif (defined (__GNUC__))
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #else
      #error "Please define MAX_IDLE_COUNTER_NO_LOAD."
  #endif

#elif BOARD_M4KN

  #if defined ( __CC_ARM   )
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #elif (defined (__ICCARM__))
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #elif (defined (__GNUC__))
      #define MAX_IDLE_COUNTER_NO_LOAD 9170000
  #else
      #error "Please define MAX_IDLE_COUNTER_NO_LOAD."
  #endif

#endif

#elif __TMPM_475__

#if defined ( __CC_ARM   )
	#define MAX_IDLE_COUNTER_NO_LOAD 2880000
#elif (defined (__ICCARM__))
	#define MAX_IDLE_COUNTER_NO_LOAD 2778000
#elif (defined (__GNUC__))
	#define MAX_IDLE_COUNTER_NO_LOAD 2830000
#else
	#error "Please define MAX_IDLE_COUNTER_NO_LOAD."
#endif

#else

#if defined ( __CC_ARM   )
	#define MAX_IDLE_COUNTER_NO_LOAD 2880000
#elif (defined (__ICCARM__))
	#define MAX_IDLE_COUNTER_NO_LOAD 6970000
#elif (defined (__GNUC__))
	#define MAX_IDLE_COUNTER_NO_LOAD 2830000
#else
	#error "Please define MAX_IDLE_COUNTER_NO_LOAD."
#endif
#endif

static unsigned char system_load = 0;
/* Last calculated load value */
static unsigned char old_load = 0;				
/* Idle counter */
static volatile long idle_counter = MAX_IDLE_COUNTER_NO_LOAD;	
static volatile long max_idle_counter = MAX_IDLE_COUNTER_NO_LOAD;	
	

/*! \brief  Idle handler
  *
  * Overloading function for FreeRTOS idle handler
  *
  * @param  None
  * @retval None
*/
void vApplicationIdleHook( void )
{
  idle_counter++;                                                               /* increase the idle counter by 1 */
#ifdef USE_WDT
  WDT_WriteClearCode();
#endif /* USE_WDT */
}

/*! \brief  Calculate the system load
  *
  * calculating the system load and taking time inside of the IRQ functions
  * also into account.
  * The norlam system load implementation of FreeRTOS only measures the time
  * spend inside of the different tasks, but not taking into account the time
  * spend inside the IRQ.
  * For a normal system this is good enougth - but here we do most of the
  * calculations inside the IRQ handlers.
  * Therefor we have to use this trick.
  *
  * @param  None
  * @retval system load as percentage
*/
unsigned char calculate_system_load(void)
{
  unsigned long current_load;
  
  /* Correct any error due to preemtion */
  if( idle_counter >= max_idle_counter )
  {
    current_load = old_load;    
  }
  else
  {
    current_load = (max_idle_counter - idle_counter);
    current_load *= 100;
    current_load /= max_idle_counter;
    
    /* Store the last successfully calcualted value */
    old_load = current_load;
  }
 
  /* Reset the counter */
  idle_counter = 0;
  
  return (unsigned char)current_load;
}

/*! \brief  Read out the system load
  *
  *
  * @param  None
  * @retval system load as percentage
*/
unsigned char SYSTEMLoad_get(void)
{
  return system_load;
}

/*! \brief  System Load Worker Task
  *
  * @param  pvParameters: None
  * @retval None
*/
void SystemLoadTaks(void* pvParameters)
{
  portTickType       xLastWakeTime;                                             /* Last wakeup time */
  const portTickType xPeriod = ( 1000 / portTICK_RATE_MS );                     /* Periode between wakeups */

  xLastWakeTime = xTaskGetTickCount();

  while (InitDone==INIT_NOT_DONE)
    vTaskDelay( 100 / portTICK_RATE_MS );

  for(;;)
  {
    system_load = calculate_system_load();                                      /* calculate the system load and remember it for the time it's needed */
    vTaskDelayUntil( &xLastWakeTime, xPeriod );                                 /* wait until 1 second till last calling has completed */
  }
}

#endif
