/** 
 *******************************************************************************
 * @file    bsp_i2c.c
 * @brief   This file provides API functions for BSP I2C driver.
 * @version V1.0.0.2
 * $Date:: 2018-07-27 15:10:15 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "hal_i2c.h"
#include "hal_gpio.h"

#if defined(HAL_I2C_H)

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */

/** 
 *  @name  Parameter Result
 *  @brief Whether the parameter is specified or not.
 *  @{
 */
#define HAL_I2C_PARAM_OK              ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define HAL_I2C_PARAM_NG              ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter Result */

/** 
 *  @name  timeout 
 *  @brief This timeouts are not based on accurate values, this just guarantee that 
           the application will not remain stuck if the I2C communication is corrupted.
 *  @{
 */
#define HAL_I2C_TIMEOUT     (100000)    /*>! fail safe. */

/**
 *  @}
 */ /* End of name timeout */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */
   
/** 
 *  @name  timeout 
 *  @brief This timeouts are not based on accurate values, this just guarantee that 
           the application will not remain stuck if the I2C communication is corrupted.
 *  @{
 */
#define I2C_TIMEOUT     (100000)    /*>! fail safe. */

/**
 *  @}
 */ /* End of name timeout */   

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */
/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Transfer State.
*/
/*----------------------------------*/
enum {
    HAL_I2C_TRANSFER_STATE_IDLE = 0U,   /*!< Idle.                      */
    HAL_I2C_TRANSFER_STATE_BUSY         /*!< Busy.                      */
} TransferState;

/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  For IRQn_Type number definition.
*/
/*----------------------------------*/
typedef struct
{
    IRQn_Type i2c;
    IRQn_Type al;
    IRQn_Type bf;
    IRQn_Type na;
} HAL_I2c_Irq_t;

/** 
 *  @}
 */ /* End of group UTILITIES_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_variables
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Channel 0 & 1 I2C handle structures.
*/
/*----------------------------------*/
HAL_I2C_t   halI2c[HAL_I2C_CH_COUNT];
/**
 *  @}
 */ /* End of group UTILITIES_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Const Table                                                                 */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_const
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Channel 0 & 1 IRQn_Type number table.
*/
/*----------------------------------*/
static const HAL_I2c_Irq_t HAL_I2C_IRQN_TBL[2] = 
{
    { INTI2C0_IRQn, INTI2C0AL_IRQn, INTI2C0BF_IRQn, INTI2C0NA_IRQn },
    { INTI2C1_IRQn, INTI2C1AL_IRQn, INTI2C1BF_IRQn, INTI2C1NA_IRQn }
};

/**
 *  @}
 */ /* End of group UTILITIES_Private_const */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_functions
 *  @{
 */
#ifdef DEBUG
__STATIC_INLINE int32_t checkParamIrqn(uint32_t irqn);
__STATIC_INLINE int32_t checkParamAddress(int32_t address);
#endif
__STATIC_INLINE void enableIrq(uint32_t irqn);
__STATIC_INLINE void disableIrq(uint32_t irqn);
__STATIC_INLINE void clearIrq(uint32_t irqn);
__STATIC_INLINE uint32_t setI2c(uint8_t ch, uint32_t *p_irqn);
__STATIC_INLINE void resetAsynch(HAL_I2C_t *p_obj);
__STATIC_INLINE int32_t waitStatus(HAL_I2C_t *p_obj);
__STATIC_INLINE int32_t statusArbitration(I2C_t *p_obj);
static void irqHandler(uint8_t channel);
static void slaveIrqHandler(uint8_t channel);
static HAL_I2C_Result initGpios( uint8_t channel, uint8_t port );

/*--------------------------------------------------*/
/**
  * @brief     Initialize the I2C Driver
  * @param     channel     :Channel Id
  * @param     sda         :SDA port.
  * @param     scl         :SCL port.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
static HAL_I2C_Result initGpios( uint8_t channel, uint8_t port )
{
  switch (channel)
  {
    case 0:
      if (HAL_GPIO_PORT_C == port)
      {
        HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_0, HAL_GPIO_I2C_SDA);
        HAL_GPIO_Init(HAL_GPIO_PORT_C, HAL_GPIO_BIT_1, HAL_GPIO_I2C_SCL);
      }
      else 
      {
        if (HAL_GPIO_PORT_T == port)
        {
          HAL_GPIO_Init(HAL_GPIO_PORT_T, HAL_GPIO_BIT_0, HAL_GPIO_I2C_SDA);
          HAL_GPIO_Init(HAL_GPIO_PORT_T, HAL_GPIO_BIT_1, HAL_GPIO_I2C_SCL);            
        }    
        else
        {
          return HAL_I2C_INVALID_CH_PORT_COMB;
        }        
      }
      break;  
    case 1:
      if (HAL_GPIO_PORT_D == port)
      {
        HAL_GPIO_Init(HAL_GPIO_PORT_D, HAL_GPIO_BIT_3, HAL_GPIO_I2C_SDA);
        HAL_GPIO_Init(HAL_GPIO_PORT_D, HAL_GPIO_BIT_4, HAL_GPIO_I2C_SCL);
      }       
      else 
      {
        if (HAL_GPIO_PORT_U == port)
        {  
          HAL_GPIO_Init(HAL_GPIO_PORT_U, HAL_GPIO_BIT_0, HAL_GPIO_I2C_SDA);
          HAL_GPIO_Init(HAL_GPIO_PORT_U, HAL_GPIO_BIT_1, HAL_GPIO_I2C_SCL);
        }
        else
        {
          return HAL_I2C_INVALID_CH_PORT_COMB;
        }        
      }
    default:
      return HAL_I2C_INVALID_CHANNEL;
    }
  return HAL_I2C_OK;
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Compare the IRQn's parameter.
  * @param  irqn         :I2C IRQn List.
  * @retval I2C_PARAM_OK :Available.
  * @retval I2C_PARAM_NG :Not Available.
  * @note   -.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t checkParamIrqn(uint32_t irqn)
{
    int32_t result = HAL_I2C_ERROR_PARAM;

    if ((irqn == (uint32_t)HAL_I2C_IRQN_TBL[0].i2c) || (irqn == (uint32_t)HAL_I2C_IRQN_TBL[1].i2c))
    {
        result = HAL_I2C_PARAM_OK;
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Compare the Slave address's parameter.
  * @param  address      :Address.
  * @retval I2C_PARAM_OK :Available.
  * @retval I2C_PARAM_NG :Not Available.
  * @note   Here, 10bit address has not supported.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t checkParamAddress(int32_t address)
{
    int32_t result = HAL_I2C_PARAM_NG;

    if ((address >= 0) && (address <= 255))
    {
        result = HAL_I2C_PARAM_OK;
    }
    return (result);
}
#endif

/*--------------------------------------------------*/
/**
  * @brief     Enable I2C IRQ
  * @param     irqn     :I2C IRQn List.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void enableIrq(uint32_t irqn)
{
    HAL_I2c_Irq_t *p_irqn = (HAL_I2c_Irq_t *)irqn;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(checkParamIrqn(irqn));
#endif /* #ifdef DEBUG */
    NVIC_EnableIRQ(p_irqn->i2c);
}

/*--------------------------------------------------*/
/**
  * @brief     Disable I2C IRQ
  * @param     irqn     :I2C IRQn List.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void disableIrq(uint32_t irqn)
{
    HAL_I2c_Irq_t *p_irqn = (HAL_I2c_Irq_t *)irqn;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(checkParamIrqn(irqn));
#endif /* #ifdef DEBUG */
    NVIC_DisableIRQ(p_irqn->i2c);
    NVIC_DisableIRQ(p_irqn->al);
    NVIC_DisableIRQ(p_irqn->bf);
    NVIC_DisableIRQ(p_irqn->na);
}

/*--------------------------------------------------*/
/**
  * @brief     ClearPending I2C IRQ
  * @param     irqn     :I2C IRQn List.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void clearIrq(uint32_t irqn)
{
    HAL_I2c_Irq_t *p_irqn = (HAL_I2c_Irq_t *)irqn;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(checkParamIrqn(irqn));
#endif /* #ifdef DEBUG */
    NVIC_ClearPendingIRQ(p_irqn->i2c);
    NVIC_ClearPendingIRQ(p_irqn->al);
    NVIC_ClearPendingIRQ(p_irqn->bf);
    NVIC_ClearPendingIRQ(p_irqn->na);
}

/*--------------------------------------------------*/
/**
  * @brief     I2C Setting
  * @param     ch       :I2C Channel.
  * @param     p_irqn   :Destination Address of a I2C IRQn List.
  * @retval    non-zero :Instance Address.
  * @retval    zero     :Channel not supported.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t setI2c(uint8_t ch, uint32_t *p_irqn)
{   
    uint32_t instance = 0;

    switch (ch)
    {
        case HAL_I2C_CH0:
            TSB_CG_FSYSMENA_IPMENA25 = 1;
            instance = (uint32_t)TSB_I2C0;
            *p_irqn = (uint32_t)&HAL_I2C_IRQN_TBL[0];
            break;
       case HAL_I2C_CH1:
            TSB_CG_FSYSMENA_IPMENA26 = 1;
            instance = (uint32_t)TSB_I2C1;
            *p_irqn = (uint32_t)&HAL_I2C_IRQN_TBL[1];
            break;            
        default:
            break;
    }
    return (instance);
}

/*--------------------------------------------------*/
/**
  * @brief     Reset Asynch Transfer
  * @param     p_obj    :i2c object
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void resetAsynch(HAL_I2C_t *p_obj)
{
  disableIrq(p_obj->info.irqn);
  I2C_disable_interrupt(&p_obj->i2c);
}

__STATIC_INLINE int32_t statusArbitration(I2C_t *p_obj)
{
#ifdef DEBUG
  if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
  {
    return ((p_obj->p_instance->SR & I2CxSR_AL) == I2CxSR_AL);
  }
  return (0);
#else
  return ((p_obj->p_instance->SR & I2CxSR_AL) == I2CxSR_AL);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Waiting i2c status
  * @param     p_obj    :i2c object
  * @retval    0        :Success.
  * @retval    -1       :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t waitStatus(HAL_I2C_t *p_obj)
{
  int32_t timeout;
  
  timeout = HAL_I2C_TIMEOUT;
  while (!I2C_int_status(&p_obj->i2c))
  {
    if (statusArbitration(&p_obj->i2c))
    {
      volatile uint32_t dummy = 0;
      dummy = I2C_read_data(&p_obj->i2c);
      return (-5);
    }
    if ((timeout--) == 0)
    {
      return (-1);
    }
  }
  if (statusArbitration(&p_obj->i2c))
  {
    volatile uint32_t dummy = 0;
    dummy = I2C_read_data(&p_obj->i2c);
    return (-5);
  }
  return (0);
}

/*--------------------------------------------------*/
/**
  * @brief     The asynchronous IRQ handler
  * @param     p_obj     :i2c object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref I2C_Events.
  * @attention This function should be implement as INTI2C0_IRQHandler.
  */
/*--------------------------------------------------*/
void INTI2C0_IRQHandler ()
{
  irqHandler(0);
}

/*--------------------------------------------------*/
/**
  * @brief     The asynchronous IRQ handler
  * @param     p_obj     :i2c object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref I2C_Events.
  * @attention This function should be implement as INTI2C0_IRQHandler.
  */
/*--------------------------------------------------*/
void INTI2C1_IRQHandler ()
{
  irqHandler(1);
}



/*--------------------------------------------------*/
/**
  * @brief     I2C Transfer handler
  * @param     p_obj     :i2c object.
  * @retval    -
  * @note      Called by i2c_irq_handler_asynch.
  */
/*--------------------------------------------------*/
static void irqHandler(uint8_t channel)
{
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
    
  p_obj = &halI2c[channel];
  
  I2C_clear_int_status(&p_obj->i2c);

  if ((!I2C_master(&p_obj->i2c)) || (p_obj->info.asynch.state != HAL_I2C_TRANSFER_STATE_BUSY))
  {
    p_obj->info.asynch.event = HAL_I2C_EVENT_ERROR;
    p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
  }
  else
  {
    if (I2C_transmitter(&p_obj->i2c))
    {
      int32_t start = I2C_restart(&p_obj->i2c);

      if (!I2C_get_ack(&p_obj->i2c))
      {
        if (p_obj->tx_buff.pos < p_obj->tx_buff.length)
        {
          I2C_write_data(&p_obj->i2c, (uint32_t)p_obj->tx_buff.p_buffer[p_obj->tx_buff.pos++]);
        }
        else if (p_obj->rx_buff.length != 0)
        {
          I2C_start_condition(&p_obj->i2c, (p_obj->info.asynch.address | 1U));
        }
        else
        {
          if (p_obj->info.asynch.stop)
          {
            I2C_stop_condition(&p_obj->i2c);
          }
          p_obj->info.asynch.event = HAL_I2C_EVENT_TRANSFER_COMPLETE;
          p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
        }
      }
      else
      {
        if (p_obj->tx_buff.pos < p_obj->tx_buff.length)
        {
            if (p_obj->tx_buff.pos == 0)
            {
              p_obj->info.asynch.event = (HAL_I2C_EVENT_ERROR | HAL_I2C_EVENT_ERROR_NO_SLAVE);
              p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
            }
            else
            {
              p_obj->info.asynch.event = (HAL_I2C_EVENT_ERROR | HAL_I2C_EVENT_TRANSFER_EARLY_NACK);
              p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
            }
        }
        else if (p_obj->rx_buff.length != 0)
        {
          if (p_obj->tx_buff.pos == 0)
          {
            p_obj->info.asynch.event = (HAL_I2C_EVENT_ERROR | HAL_I2C_EVENT_ERROR_NO_SLAVE);
            p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
          }
          else
          {
            p_obj->info.asynch.event = (HAL_I2C_EVENT_ERROR | HAL_I2C_EVENT_TRANSFER_EARLY_NACK);
            p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
          }
        }
        else
        {
          if (p_obj->info.asynch.stop)
          {
            I2C_stop_condition(&p_obj->i2c);
          }
          p_obj->info.asynch.event = HAL_I2C_EVENT_TRANSFER_COMPLETE;
          p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
        }
      }
    }
    else
    {
      int32_t start = I2C_restart(&p_obj->i2c);

      if (p_obj->rx_buff.pos < p_obj->rx_buff.length)
      {
        if (!start)
        {
          p_obj->rx_buff.p_buffer[p_obj->rx_buff.pos++] = (uint8_t)I2C_read_data(&p_obj->i2c);
        }
      }
      if (p_obj->rx_buff.pos < p_obj->rx_buff.length)
      {
        I2C_set_ack(&p_obj->i2c, ((p_obj->rx_buff.pos < (p_obj->rx_buff.length - 1)? 0: 1)));
        I2C_write_data(&p_obj->i2c, 0);
      }
      else
      {
        if (p_obj->info.asynch.stop)
        {
          I2C_stop_condition(&p_obj->i2c);
        }
        p_obj->info.asynch.event = HAL_I2C_EVENT_TRANSFER_COMPLETE;
        p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
      }
    }
  }
  if (p_obj->info.asynch.state == HAL_I2C_TRANSFER_STATE_IDLE)
  {
    resetAsynch(p_obj);
  }
}

/*--------------------------------------------------*/
/**
  * @brief     I2C Slave handler
  * @param     p_obj     :i2c object.
  * @retval    -
  * @note      Called by i2c_slave_irq_handler_asynch.
  */
/*--------------------------------------------------*/
static void slaveIrqHandler(uint8_t channel)
{
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
    
  p_obj = &halI2c[channel];
    
  I2C_clear_int_status(&p_obj->i2c);

  if ((I2C_master(&p_obj->i2c)) || (p_obj->info.asynch.state != HAL_I2C_TRANSFER_STATE_BUSY))
  {
    p_obj->info.asynch.event = HAL_I2C_EVENT_ERROR;
    p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
  }
  else
  {
    int32_t start = I2C_slave_detected(&p_obj->i2c);
    if (start)
    {
      uint8_t sa = (uint8_t)I2C_read_data(&p_obj->i2c);
    }
    if (I2C_transmitter(&p_obj->i2c))
    {
      if (!I2C_get_ack(&p_obj->i2c))
      {
        if (p_obj->tx_buff.pos < p_obj->tx_buff.length)
        {
          I2C_write_data(&p_obj->i2c, (uint32_t)p_obj->tx_buff.p_buffer[p_obj->tx_buff.pos++]);
        }
        else
        {   /* dummy, wait nack */
          I2C_write_data(&p_obj->i2c, 0);
        }
      }
      else
      {
        /* error event not be set */
        p_obj->info.asynch.event = HAL_I2C_EVENT_TRANSFER_COMPLETE;
        p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
      }
    }
    else
    {
      if (p_obj->rx_buff.pos < p_obj->rx_buff.length)
      {
        if (!start)
        {
          p_obj->rx_buff.p_buffer[p_obj->rx_buff.pos++] = (uint8_t)I2C_read_data(&p_obj->i2c);
        }
      }
      if (p_obj->rx_buff.pos < p_obj->rx_buff.length)
      {
        I2C_set_ack(&p_obj->i2c, ((p_obj->rx_buff.pos < (p_obj->rx_buff.length - 1)? 0: 1)));
        I2C_write_data(&p_obj->i2c, 0);
      }
      else
      {
        p_obj->info.asynch.event = HAL_I2C_EVENT_TRANSFER_COMPLETE;
        p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
      }
    }
  }
  if (p_obj->info.asynch.state == HAL_I2C_TRANSFER_STATE_IDLE)
  {
    resetAsynch(p_obj);
    I2C_slave_init(&p_obj->i2c);
  }
}

/*--------------------------------------------------*/
/**
  * @brief     Enable I2C IRQ
  * @param     p_obj     :i2c object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void HAL_I2C_Enable_irq(uint8_t channel)
{
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */    
  enableIrq(halI2c[channel].info.irqn);
}

/*--------------------------------------------------*/
/**
  * @brief     Disable I2C IRQ
  * @param     p_obj     :i2c object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void HAL_I2C_Disable_irq(uint8_t channel)
{
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
  disableIrq(halI2c[channel].info.irqn);
}

/**
 *  @}
 */ /* End of group UTILITIES_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/**
  * @brief     Initialize the I2C Driver
  * @param     channel   :Channel Id
  * @param     port        :I2C port.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_Init(uint8_t channel, uint8_t port)
{
  HAL_I2C_Result result = HAL_I2C_ERROR;
  uint32_t instance = 0;
  uint32_t irqn = 0;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
    
  p_obj = &halI2c[channel];

  result = initGpios( channel, port );
  
  if (HAL_I2C_SUCCESS != result)
  {
    return result;
  }
  
  instance = setI2c(channel, &irqn);
  
  if ((instance != 0) && (irqn != 0))
  {
    disableIrq(irqn);
    clearIrq(irqn);

    /* Set irqn table */
    p_obj->info.irqn = irqn;

    /* Set instance */
    p_obj->i2c.p_instance = (TSB_I2C_TypeDef *)instance;

    /* I2C Reset */
    HAL_I2C_Reset(channel);

    /* Set Frequency Default at 100KHz */
    if (HAL_I2C_Frequency(channel, 100000) == HAL_I2C_SUCCESS)
    {
        result = HAL_I2C_OK;
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Reset I2C peripheral
  * @param     channel   :Channel Id
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void HAL_I2C_Reset(uint8_t channel)
{
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */

  /* Software reset */
  I2C_reset(&halI2c[channel].i2c);
}

/*--------------------------------------------------*/
/**
  * @brief     Configure the I2C frequency
  * @param     channel   :Channel Id
  * @param     hz          :frequency in Hz.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    HAL_I2C_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_Frequency(uint8_t channel, int32_t hz)
{
  HAL_I2C_Result result = HAL_I2C_ERROR;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
    
  p_obj = &halI2c[channel];
  
  if (I2C_port_high(&p_obj->i2c))
  {
    uint32_t fval;

    SystemCoreClockUpdate();

    fval = I2C_get_clock_setting(&p_obj->i2c, (uint32_t)hz, SystemCoreClock, &p_obj->i2c.init.clock);
    if (fval != 0)
    {
      I2C_init(&p_obj->i2c);
      p_obj->info.bus_free = 0;
      p_obj->info.start = 0;
      p_obj->info.asynch.address = 0;
      p_obj->info.asynch.stop = 0;
      p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
      p_obj->info.asynch.event = 0;
      p_obj->tx_buff.p_buffer = I2C_NULL;
      p_obj->tx_buff.length = 0;
      p_obj->tx_buff.pos = 0;
      p_obj->rx_buff.p_buffer = I2C_NULL;
      p_obj->rx_buff.length = 0;
      p_obj->rx_buff.pos = 0;
      result = HAL_I2C_OK;
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Check bus free on the I2C bus.
  * @param     channel   :Channel Id
  * @retval    HAL_I2C_SUCCESS :Success.
  * @retval    HAL_I2C_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_Check_bus_free(uint8_t channel)
{
  HAL_I2C_Result result = HAL_I2C_OK;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
    
  halI2c[channel].info.bus_free = 1;

  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Creates a start condition on the I2C bus.
  * @param     channel   :Channel Id
  * @retval    HAL_I2C_SUCCESS :Success.
  * @retval    HAL_I2C_ERROR   :Failure.(now, not use)
  * @note      Start condition is not generate yet, after this function returned.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_Start(uint8_t channel)
{
  HAL_I2C_Result result = HAL_I2C_OK;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
  
  halI2c[channel].info.start = 1;

  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Creates a stop condition on the I2C bus.
  * @param     channel   :Channel Id
  * @retval    HAL_I2C_SUCCESS :Success.
  * @retval    HAL_I2C_ERROR   :Failure.
  * @note      Master and blocking function.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_Stop(uint8_t channel)
{
  HAL_I2C_Result result = HAL_I2C_OK;
  int32_t timeout;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
    
  p_obj = &halI2c[channel];
  I2C_stop_condition(&p_obj->i2c);
  p_obj->info.bus_free = 0;
  p_obj->info.start = 0;

  timeout = HAL_I2C_TIMEOUT;
  while (HAL_I2C_Active(channel))
  {
    if ((timeout--) == 0) 
    {
      result = HAL_I2C_ERROR;
      break;
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Blocking reading data
  * @param     channel   :Channel Id
  * @param     address     :Slave address(7-bit) and last bit is 0. 
  * @param     p_data      :Address of Read data.
  * @param     length      :Number of the bytes to read.
  * @param     stop        :Stop to be generated after the transfer is done.
  * @retval    Number of read bytes.
  * @note      Master and blocking function.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_Read(uint8_t channel, int32_t address, uint8_t *p_data, int32_t length, int32_t stop)
{
  int32_t result = 0;
  int32_t count = 0;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(IS_POINTER_NOT_NULL(p_data));
  assert_param(checkParamAddress(address));
#endif /* #ifdef DEBUG */    
    
  if (length > 0)
  {
    /* Start Condition */
    if (HAL_I2C_Start(channel) == HAL_I2C_SUCCESS)
    {
        /* no processing */
    }
    result = HAL_I2C_ByteWrite(channel, (int32_t)((uint32_t)address | 1U));
    if (result == HAL_I2C_ACK)
    {
      /* Read all bytes */
      while (count < length)
      {
        int32_t data = HAL_I2C_ByteRead(channel, ((count < (length - 1))? 0: 1));
        if (data < 0)
        {
          result = data;
          break;
        }
        p_data[count++] = (uint8_t)data;
      }
      result = count;
    }
    else if (result == HAL_I2C_ERROR_ARBITRATION)
    {
    }
    else if (result == HAL_I2C_ERROR_BUS_BUSY)
    {
    }
    else
    {
      stop = 1;
      result = HAL_I2C_ERROR_NO_SLAVE;
    }
    /* Stop Condition */
    if (stop)
    {
      if (HAL_I2C_Stop(channel) == HAL_I2C_SUCCESS)
      {
        /* no processing */
      }
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Blocking sending data
  * @param     channel   :Channel Id
  * @param     address     :Slave address(7-bit) and last bit is 0.
  * @param     p_data      :Destination address of Write data.
  * @param     length      :Number of the bytes to write.
  * @param     stop        :Stop to be generated after the transfer is done.
  * @retval    Number of write bytes.
  * @note      Master and blocking function.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_Write(uint8_t channel, int32_t address, uint8_t *p_data, int32_t length, int32_t stop)
{
  int32_t result = 0;
  int32_t count = 0;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(IS_POINTER_NOT_NULL(p_data));
  assert_param(checkParamAddress(address));
#endif /* #ifdef DEBUG */

  /* Start Condition */
  if (HAL_I2C_Start(channel) == HAL_I2C_SUCCESS)
  {
    /* no processing */
  }
  result = HAL_I2C_ByteWrite(channel, address);
  if (result == HAL_I2C_ACK)
  {
    /* Write all bytes */
    while (count < length)
    {
      int32_t data = HAL_I2C_ByteWrite(channel, (int32_t)p_data[count++]);
      if (data < HAL_I2C_ACK)
      {
        result = data;
        break;
      }
    }
    if(result >=0){
      result = count;
    }
  }
  else if (result == HAL_I2C_ERROR_ARBITRATION)
  {
  }
  else if (result == HAL_I2C_ERROR_BUS_BUSY)
  {
  }
  else
  {
    stop = 1;
    result = HAL_I2C_ERROR_NO_SLAVE;
  }
  /* Stop Condition */
  if (stop)
  {
    if (HAL_I2C_Stop(channel) == HAL_I2C_SUCCESS)
    {
        /* no processing */
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Read one byte
  * @param     channel   :Channel Id
  * @param     last        :last acknowledge.
  * @retval    The read byte (but -1 is timout error).
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_ByteRead(uint8_t channel, int32_t last)
{
  int32_t result;
  HAL_I2C_t *p_obj;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
      
  p_obj = &halI2c[channel];
  I2C_clear_int_status(&p_obj->i2c);
  I2C_set_ack(&p_obj->i2c, last);
  I2C_write_data(&p_obj->i2c, 0);
  result = waitStatus(p_obj);
  if (result < 0)
  {
  }
  else
  {
    result = (int32_t)I2C_read_data(&p_obj->i2c);
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Write one byte
  * @param     p_obj     :i2c object.
  * @param     data      :Write data.
  * @retval    0         :NACK was received.
  * @retval    1         :ACK was received.
  * @retval    -1        :Timout error.
  * @note      Macro definition of return values is @ref I2C_ACK.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_ByteWrite(uint8_t channel, int32_t data)
{
  int32_t result;
  int32_t timeout;
  HAL_I2C_t *p_obj;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
      
  p_obj = &halI2c[channel];

  I2C_clear_int_status(&p_obj->i2c);
  if (p_obj->info.start == 1)
  {
    p_obj->info.start = 0;
    if (p_obj->info.bus_free == 1)
    {
      timeout = HAL_I2C_TIMEOUT;
      while (HAL_I2C_Active(channel))
      {
        if ((timeout--) == 0) 
        {
          p_obj->info.bus_free = 0;
          return (-1);
        }
      }
    }
    /* Start Condition */
    I2C_start_condition(&p_obj->i2c, (uint32_t)data);
    if (p_obj->info.bus_free == 1)
    {
      if(!I2C_master(&p_obj->i2c))
      {
        p_obj->i2c.p_instance->CR2 = (I2CxCR2_INIT | I2CxCR2_PIN_CLEAR);
        p_obj->info.bus_free = 0;
        if (statusArbitration(&p_obj->i2c))
        {
            return (-5);
        }
        return (-2);
      }
    }
  }
  else
  {
    I2C_write_data(&p_obj->i2c, (uint32_t)data);
  }
  p_obj->info.bus_free = 0;
  result = waitStatus(p_obj);
  if (result < 0)
  {
    return (result);
  }
  if (!I2C_get_ack(&p_obj->i2c))
  {
    result = 1;
  }
  else
  {
    result = 0;
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Attempts to determine if the I2C bus is already in use
  * @param     p_obj     :i2c object.
  * @retval    0         :Non-active.
  * @retval    1         :Active.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint8_t HAL_I2C_Active(uint8_t channel)
{
  uint8_t result;

  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */

  if (I2C_status_busy(&halI2c[channel].i2c))
  {
    result = 1;     
  }
  else
  {
    result = 0;
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Start I2C asynchronous transfer
  * @param     p_obj     :i2c object.
  * @param     p_tx      :Buffer of write data.
  * @param     tx_length :Length of write data.
  * @param     p_rx      :Buffer of read data.
  * @param     rx_length :Length of read data.
  * @param     address   :Slave address(7-bit) and last bit is 0.
  * @param     stop      :Stop to be generated after the transfer is done.
  * @retval    HAL_I2C_SUCCESS :Success.
  * @retval    HAL_I2C_ERROR   :Failure.
  * @note      Master and non-blocking function. 
  * @note      Events of this function will be notified on i2c_irq_handler_asynch.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_TransferAsynch(uint8_t channel, uint8_t *p_tx, int32_t tx_length, uint8_t *p_rx, int32_t rx_length, int32_t address, int32_t stop)
{
  HAL_I2C_Result result = HAL_I2C_ERROR;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(checkParamAddress(address));
#endif /* #ifdef DEBUG */
      
  p_obj = &halI2c[channel];  
    
  if (p_obj->info.asynch.state == HAL_I2C_TRANSFER_STATE_IDLE)
  {
    if (((p_tx != I2C_NULL) && (tx_length > 0)) || ((p_rx != I2C_NULL) && (rx_length > 0)))
    {
      resetAsynch(p_obj);
      I2C_clear_int_status(&p_obj->i2c);
      clearIrq(p_obj->info.irqn);
      p_obj->info.asynch.address = (uint32_t)address;
      p_obj->info.asynch.event = 0;
      p_obj->info.asynch.stop = (uint32_t)stop;
      p_obj->tx_buff.p_buffer = p_tx;
      p_obj->tx_buff.length = (uint32_t)tx_length;
      p_obj->tx_buff.pos = 0;
      p_obj->rx_buff.p_buffer = p_rx;
      p_obj->rx_buff.length = (uint32_t)rx_length;
      p_obj->rx_buff.pos = 0;
      p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_BUSY;
      I2C_enable_interrupt(&p_obj->i2c);
      if ((tx_length == 0) && (rx_length != 0))
      {
        I2C_start_condition(&p_obj->i2c, (uint32_t)((uint32_t)address | 1U));
      }
      else
      {
        I2C_start_condition(&p_obj->i2c, (uint32_t)address);
      }
      p_obj->info.bus_free = 0;
      p_obj->info.start = 0;
      enableIrq(p_obj->info.irqn);
      result = HAL_I2C_OK;
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     The asynchronous IRQ handler
  * @param     p_obj     :i2c object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref I2C_Events.
  * @attention This function should be implement as INTI2Cx_IRQHandler.
  */
/*--------------------------------------------------*/
uint32_t HAL_I2C_IrqHandlerAsynch(uint8_t channel)
{
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
      
  slaveIrqHandler(channel);

  return (halI2c[channel].info.asynch.event & HAL_I2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     Abort asynchronous transfer
  * @param     p_obj     :i2c object.
  * @retval    -
  * @note      After error event occurred on i2c_irq_handler_asynch, 
  * @note      call this function and clear error status. 
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void HAL_I2C_AbortAsynch(uint8_t channel)
{
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
      
  p_obj = &halI2c[channel];
  
  resetAsynch(p_obj);
  if (HAL_I2C_Stop(channel) == HAL_I2C_SUCCESS)
  {
    /* no processing */
  }
  p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
  HAL_I2C_Reset(channel);
  I2C_init(&p_obj->i2c);
  clearIrq(p_obj->info.irqn);
}

/*--------------------------------------------------*/
/**
  * @brief     Configure I2C as slave or master.
  * @param     p_obj        :i2c object.
  * @param     enable_slave :Enable slave mode.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void HAL_I2C_SlaveMode(uint8_t channel, int32_t enable_slave)
{
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
      
  p_obj = &halI2c[channel];

  disableIrq(p_obj->info.irqn);

  if (enable_slave)
  {
    I2C_slave_init(&p_obj->i2c);
  }
  else
  {
    /* Slave Disable Settings. */
    HAL_I2C_Reset(channel);
    I2C_init(&p_obj->i2c);
  }
  p_obj->info.bus_free = 0;
  p_obj->info.start = 0;
  I2C_clear_int_status(&p_obj->i2c);
}

/*--------------------------------------------------*/
/**
  * @brief     Check to see if the I2C slave has been addressed.
  * @param     p_obj               :i2c object.
  * @retval    I2C_NO_DATA         :The slave has not been addressed.
  * @retval    I2C_READ_ADDRESSED  :Read addresses.
  * @retval    I2C_WRITE_GENERAL   :Write to all slaves(now, not support).
  * @retval    I2C_WRITE_ADDRESSED :Write addressed.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_SlaveReceive(uint8_t channel)
{
  int32_t result = HAL_I2C_NO_DATA;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */
      
  p_obj = &halI2c[channel];

  if (I2C_slave_detected(&p_obj->i2c))
  {
    uint32_t sa = I2C_read_data(&p_obj->i2c);

    if (I2C_transmitter(&p_obj->i2c))
    {
      result = HAL_I2C_WRITE_ADDRESSED;
    }
    else
    {
      result = HAL_I2C_READ_ADDRESSED;
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Blocking reading data.
  * @param     p_obj     :i2c object.
  * @param     p_data    :Destination address of read data.
  * @param     length    :Number of bytes to read.
  * @retval    Number of read bytes.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_SlaveRead(uint8_t channel, uint8_t *p_data, int32_t length)
{
  int32_t count = 0;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(IS_POINTER_NOT_NULL(p_data));
#endif /* #ifdef DEBUG */

  p_obj = &halI2c[channel];
  
  /* Read all bytes */
  while (count < length)
  {
    I2C_clear_int_status(&p_obj->i2c);
    I2C_set_ack(&p_obj->i2c, ((count < (length - 1))? 0: 1));
    I2C_write_data(&p_obj->i2c, 0);
    if (waitStatus(p_obj) < 0)
    {
      break;
    }
    if (I2C_slave_detected(&p_obj->i2c))
    {
      return (count);
    }
    p_data[count++] = (uint8_t)I2C_read_data(&p_obj->i2c);
  }
  I2C_slave_init(&p_obj->i2c);
  return (count);
}

/*--------------------------------------------------*/
/**
  * @brief     Blocking sending data.
  * @param     p_obj     :i2c object.
  * @param     p_data    :Source address of write data.
  * @param     length    :Number of bytes to write.
  * @retval    Number of written bytes.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t HAL_I2C_SlaveWrite(uint8_t channel, uint8_t *p_data, int32_t length)
{
  int32_t count = 0;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(IS_POINTER_NOT_NULL(p_data));
#endif /* #ifdef DEBUG */

  p_obj = &halI2c[channel];
  
  /* Write all bytes */
  while (count < length)
  {
    I2C_clear_int_status(&p_obj->i2c);
    I2C_write_data(&p_obj->i2c, (uint32_t)p_data[count++]);
    if (waitStatus(p_obj) < 0)
    {
      break;
    }
    if (!I2C_get_ack(&p_obj->i2c))
    {
      /* continue */
    }
    else
    {
      break;
    }
  }
  I2C_slave_init(&p_obj->i2c);
  return (count);
}

/*--------------------------------------------------*/
/**
  * @brief     Configure I2C slave address.
  * @param     p_obj     :i2c object.
  * @param     address   :Address to be set.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void HAL_I2C_SlaveAddress(uint8_t channel, uint32_t address)
{
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
  assert_param(checkParamAddress((int32_t)address));
#endif /* #ifdef DEBUG */

  I2C_set_address(&halI2c[channel].i2c, address);
}


/*--------------------------------------------------*/
/**
  * @brief     Start I2C asynchronous transfer
  * @param     p_obj     :i2c object.
  * @param     p_tx      :Buffer of write data.
  * @param     tx_length :Length of write data.
  * @param     p_rx      :Buffer of read data.
  * @param     rx_length :Length of read data.
  * @retval    HAL_I2C_SUCCESS :Success.
  * @retval    HAL_I2C_ERROR   :Failure.
  * @note      Slave and non-blocking function.
  * @note      Events of this function will be notified on i2c_slave_irq_handler_asynch.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_SlaveTransferAsynch(uint8_t channel, uint8_t *p_tx, int32_t tx_length, uint8_t *p_rx, int32_t rx_length)
{
  HAL_I2C_Result result = HAL_I2C_ERROR;
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */

  p_obj = &halI2c[channel];

  if (p_obj->info.asynch.state == HAL_I2C_TRANSFER_STATE_IDLE)
  {
    if (((p_tx != I2C_NULL) && (tx_length > 0)) || ((p_rx != I2C_NULL) && (rx_length > 0)))
    {
      resetAsynch(p_obj);
      I2C_clear_int_status(&p_obj->i2c);
      clearIrq(p_obj->info.irqn);
      p_obj->info.asynch.address = 0;
      p_obj->info.asynch.event = 0;
      p_obj->info.asynch.stop = 0;
      p_obj->tx_buff.p_buffer = p_tx;
      p_obj->tx_buff.length = (uint32_t)tx_length;
      p_obj->tx_buff.pos = 0;
      p_obj->rx_buff.p_buffer = p_rx;
      p_obj->rx_buff.length = (uint32_t)rx_length;
      p_obj->rx_buff.pos = 0;
      p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_BUSY;
      I2C_enable_interrupt(&p_obj->i2c);
      enableIrq(p_obj->info.irqn);
      result = HAL_I2C_OK;
    }
  }
  return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     The asynchronous IRQ handler
  * @param     p_obj     :i2c object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref I2C_Events.
  * @attention This function should be implement as INTI2Cx_IRQHandler.
  */
/*--------------------------------------------------*/
uint32_t HAL_I2C_SlaveIrqHandlerAsynch(uint8_t channel)
{
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */

  p_obj = &halI2c[channel];

  irqHandler(channel);

  return (p_obj->info.asynch.event & HAL_I2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     Abort asynchronous transfer
  * @param     p_obj     :i2c object.
  * @retval    -
  * @note      For a non-blocking function.
  * @note      After error event occurred on i2c_slave_irq_handler_asynch, 
  * @note      call this function and clear error status. 
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void HAL_I2C_SlaveAbortAsynch(uint8_t channel)
{
  HAL_I2C_t *p_obj;
  /*------------------------------*/
  /*  Parameter Check             */
  /*------------------------------*/
#ifdef DEBUG
  assert_param(channel < HAL_I2C_CH_COUNT);
#endif /* #ifdef DEBUG */

  p_obj = &halI2c[channel];

  resetAsynch(p_obj);
  p_obj->info.asynch.state = HAL_I2C_TRANSFER_STATE_IDLE;
  I2C_slave_init(&p_obj->i2c);
  I2C_clear_int_status(&p_obj->i2c);
  clearIrq(p_obj->info.irqn);
}

/**
 *  @}
 */ /* End of group UTILITIES_Exported_functions */

/**
 *  @} 
 */ /* End of group UTILITIES */

/**
 *  @} 
 */ /* End of group Example */


/*--------------------------------------------------*/
/**
  * @brief     Writing and Reading by Master Blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_MasterBlockingWriteRead(uint8_t channel, uint8_t *master_tx_data, int32_t tx_length, uint8_t *master_rx_data, int32_t rx_length, int32_t slave_address)
{
    int32_t res;
    int32_t master_tx_rx_length = 0;

    if ( tx_length != 0)
    {
        /* write with no stop condition */
        res = HAL_I2C_Write(channel, slave_address, master_tx_data, tx_length, 0);
        if (res > 0)
        {
            master_tx_rx_length = res;
        }
        else if (res == HAL_I2C_ERROR_NO_SLAVE)
        {
            return HAL_I2C_ERROR_NO_SLAVE1;
        }
        else
        {
            return HAL_I2C_ERROR_WRIITE;
        }
        if (master_tx_rx_length < tx_length)
        {
            /* early nack? */
            HAL_I2C_Stop(channel);
            return HAL_I2C_ERROR;
        }
    }
    if (rx_length != 0)
    {
        /* read with stop condition */
        res = HAL_I2C_Read(channel, slave_address, master_rx_data, rx_length, 1);
        if (res > 0)
        {
            master_tx_rx_length = res;
        }
        else if (res == HAL_I2C_ERROR_NO_SLAVE)
        {
            return HAL_I2C_ERROR_NO_SLAVE1;
        }
        else if ((res == 0) && (rx_length == 0))
        {
            return HAL_I2C_ERROR_READ_ZERO;/* no request */
        }
        else
        {
            return HAL_I2C_ERROR_READ;
        }
     }
    return HAL_I2C_OK;
}

/*--------------------------------------------------*/
/**
  * @brief     Writing by Master Blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
HAL_I2C_Result HAL_I2C_MasterBlockingWrite(uint8_t channel, uint8_t *master_tx_data, int32_t tx_length, int32_t slave_address)
{
    int32_t res;

    /* write with stop condition */
    res = HAL_I2C_Write(channel, (int32_t)slave_address, master_tx_data, tx_length , 1);
    if (res < 0)
    {
        if (res == HAL_I2C_ERROR_NO_SLAVE)
        {
            return HAL_I2C_ERROR_NO_SLAVE1;
        }
        else
        {
            return HAL_I2C_ERROR_WRIITE;
        }
    }
    if (tx_length != res)
    {
      return HAL_I2C_ERROR_WRIITE;
    }
    return HAL_I2C_OK;
}

#endif /* defined(__BSP_I2C_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */
