/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <stdio.h>
#include <string.h>

#include "config.h"
#include "txz_driver_def.h"
#include "hal_cg.h"

#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_0 */

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_0 */

#include "FreeRTOS.h"
#include "task.h"

#include "debug.h"
#include "hal_pmd.h"
#include "motor_control.h"
#include "dso.h"
#include "hsdso.h"
#include "hal_ve.h"
#include "board.h"
#include "hal_gpio.h"
#include "performance_measurement.h"

#ifndef USE_OVERVOLTAGE_SIGNAL
/*! \brief  Disable Overvoltage
  *
  * Do not react on overvoltage signal
  *
  * @param  uint8_t channel_number: PMD channel to use
  * @retval None
*/
static void HAL_PMD_OvervoltageDetectDisable(uint8_t channel_number)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;
  uint16_t  mdout;

  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  if (pPMDx->EMGSTA & PMD_EMG_STATUS_INPUT)
  {
    mdout         = pPMDx->MDOUT;
    pPMDx->MDOUT  = 0;
    pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;
    pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;

    pPMDx->EMGREL = (uint8_t) (PMD_EMG_KEY >> 8);
    pPMDx->EMGREL = (uint8_t) PMD_EMG_KEY;

    pPMDx->OVVCR &= ~PMD_EMGCR_ENABLE;
    pPMDx->MDOUT  = mdout;
  }
}

#else

/*! \brief  Reset Overvoltage
  *
  * Reset the overvoltage detection
  *
  * @param  channel_number: PMD channel to use
  * @retval None
*/
void HAL_PMD_OvervoltageReset(uint8_t channel_number)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;                                           /* Pointer to PMD instance */
  uint16_t  mdout;

  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  if (pPMDx->EMGSTA & PMD_EMG_STATUS_INPUT)
  {
    mdout         = pPMDx->MDOUT;
    pPMDx->MDOUT  = 0;
    pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;
    pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;
    pPMDx->EMGCR |= 0xf << 8;                                                   /* Filter short pulses */

    pPMDx->EMGREL = (uint8_t) (PMD_EMG_KEY >> 8);
    pPMDx->EMGREL = (uint8_t) PMD_EMG_KEY;

    pPMDx->OVVCR &= ~PMD_EMGCR_ENABLE;
    pPMDx->OVVCR |= PMD_EMGCR_ENABLE;
    pPMDx->MDOUT  = mdout;
  }
}

/*! \brief  Enable Overvoltage Detect
  *
  * Enable the overvoltage detection
  *
  * @param  channel_number: PMD channel to use
  * @retval None
*/
static void HAL_PMD_OvervoltageEnable(uint8_t channel_number)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;                                           /* Pointer to PMD instance */
  
  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  pPMDx->OVVCR =  PMD_OVVCR_ENABLE
                | PMD_OVVCR_PROT_REL
                | PMD_OVVCR_PROT_MODE_LOWER;
}

/*! \brief  Interrupt handler of PMD OVV Channel 0
  *
  * @param  None
  * @retval None
*/
void INTOVV0_IRQHandler (void)
{
  MotorErrorField[0].Error |= VE_OVERVOLTAGE;
  MotorControl[0].ActualStage.main  = Stage_Emergency;                                    /* emergency case exists */
}

/*! \brief  Interrupt handler of PMD OVV Channel 1
  *
  * @param  None
  * @retval None
*/
void INTOVV1_IRQHandler (void)
{
  MotorErrorField[1].Error |= VE_OVERVOLTAGE;
  MotorControl[1].ActualStage.main  = Stage_Emergency;                                    /* emergency case exists */
}

/*! \brief  Interrupt handler of PMD OVV Channel 1
  *
  * @param  None
  * @retval None
*/
void INTOVV2_IRQHandler (void)
{
  MotorErrorField[2].Error |= VE_OVERVOLTAGE;
  MotorControl[2].ActualStage.main  = Stage_Emergency;                                    /* emergency case exists */
}

#endif /* USE_OVERVOLTAGE_SIGNAL */

/*! \brief  Disable Emergency
  *
  * Do not react on emergency signal
  *
  * @param  channel_number: PMD channel to use
  * @retval None
*/
void HAL_PMD_EmergencyDisable(uint8_t channel_number)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;                                           /* Pointer to PMD instance */
  uint16_t  mdout;

  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  mdout         = pPMDx->MDOUT;
  pPMDx->MDOUT  = 0;
  pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;
  pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;
  pPMDx->EMGCR |= 0xf << 8;                                                     /* Filter short pulses */

  pPMDx->EMGREL = (uint8_t) (PMD_EMG_KEY >> 8);
  pPMDx->EMGREL = (uint8_t) PMD_EMG_KEY;

  pPMDx->EMGCR &= ~PMD_EMGCR_ENABLE;
  pPMDx->MDOUT  = mdout;
}

/*! \brief  Emergency Reset
  *
  * Reset the Emergency Detection
  *
  * @param  channel_number: PMD channel to use
  * @retval None
*/
void HAL_PMD_EmergencyReset(uint8_t channel_number)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;
  uint16_t  mdout;

  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  if (pPMDx->EMGSTA & PMD_EMG_STATUS_INPUT)
  {
    mdout         = pPMDx->MDOUT;
    pPMDx->MDOUT  = 0;
    pPMDx->EMGCR |= PMD_EMGCR_PROT_REL;

    pPMDx->EMGREL = (uint8_t) (PMD_EMG_KEY >> 8);
    pPMDx->EMGREL = (uint8_t) PMD_EMG_KEY;
    pPMDx->EMGCR |= 0xf << 8;                                                   /* Filter short pulses */

    pPMDx->EMGCR &= ~PMD_EMGCR_ENABLE;
    pPMDx->EMGCR |= PMD_EMGCR_ENABLE;

    pPMDx->MDOUT  = mdout;
  }
}

/*! \brief  Interrupt handler of PMD Emergency Channel 0
  *
  * @param  None
  * @retval None
*/
void INTEMG0_IRQHandler (void)
{
  MotorErrorField[0].Error |= VE_EMERGENCY;
  MotorControl[0].ActualStage.main  = Stage_Emergency;                                    /* emergency case exists */
}

/*! \brief  Interrupt handler of PMD Emergency Channel 1
  *
  * @param  None
  * @retval None
*/
void INTEMG1_IRQHandler (void)
{
  MotorErrorField[1].Error |= VE_EMERGENCY;
  MotorControl[1].ActualStage.main  = Stage_Emergency;                                    /* emergency case exists */
}

/*! \brief  Interrupt handler of PMD Emergency Channel 1
  *
  * @param  None
  * @retval None
*/
void INTEMG2_IRQHandler (void)
{
  MotorErrorField[2].Error |= VE_EMERGENCY;
  MotorControl[2].ActualStage.main  = Stage_Emergency;                                    /* emergency case exists */
}

/*! \brief  Initialize the PMD output pins
  *
  * Switch the according pins to PMD output
  *
  * @param  channel_number: channel to use
  * @retval None
*/
static void PMD_IOInit(unsigned char channel_number)
{
  HAL_GPIO_Port  USE_Port = HAL_GPIO_PORT_B;

  switch (channel_number)
  {
  case 0:
    USE_Port = HAL_GPIO_PORT_B;
    break;
  case 1:
    USE_Port = HAL_GPIO_PORT_E;
    break;
  case 2:
    USE_Port = HAL_GPIO_PORT_U;
    break;
  default:
    assert_param(0);
    break;
  }

  HAL_GPIO_Init(USE_Port,
                HAL_GPIO_BIT_0 |
                HAL_GPIO_BIT_1 |
                HAL_GPIO_BIT_2 |
                HAL_GPIO_BIT_3 |
                HAL_GPIO_BIT_4 |
                HAL_GPIO_BIT_5,
                HAL_GPIO_PMD_OUTPUT
                );

#define BIT8(a, b, c, d, e, f, g, h) ((a) * 0x80 + (b) * 0x40 + (c) * 0x20 + (d) * 0x10 + (e) * 0x08 + (f) * 0x04 + (g) * 0x02 + (h) * 0x01)

  TSB_PB->FR4 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD0 PORT output Enable */
  TSB_PB->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD0 PORT output Enable */

  TSB_PE->FR6 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD1 PORT output Enable */
  TSB_PE->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD1 PORT output Enable */

  TSB_PU->FR6 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD2 PORT output Enable */
  TSB_PU->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD2 PORT output Enable */


#ifdef USE_EMERGENCY_SIGNAL
  HAL_GPIO_Init(USE_Port,HAL_GPIO_BIT_6,HAL_GPIO_PMD_INPUT);
#endif /* USE_EMERGENCY_SIGNAL */

#ifdef USE_OVERVOLTAGE_SIGNAL
  HAL_GPIO_Init(USE_Port,HAL_GPIO_BIT_7,HAL_GPIO_PMD_INPUT);
#endif /* USE_OVERVOLTAGE_SIGNAL */
}

/*! \brief  PMD_HandleParameterChangeSystem
  *
  * Handle change in System parameters
  *
  * @param  channel_number: channel to use
  * @retval None
*/
void HAL_PMD_HandleParameterChangeSystem(uint8_t channel_number)
{
  TSB_PMD_TypeDef*  pPMD  = NULL;

  switch (channel_number)
  {
  case 0:
    pPMD    = TSB_PMD0;
    break;
  case 1:
    pPMD    = TSB_PMD1;
    break;
  case 2:
    pPMD   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  pPMD->RATE = PERIPHERIAL_CLOCK/2/SystemValues[channel_number].PwmFrequency;  /* Triangular Wave */
}

/*! \brief  Normal output for the PMD Signals
  *
  * Normal state while motor is turning
  *
  * @param  channel_number: channel to use
  * @retval None
*/
void HAL_PMD_NormalOutput(unsigned char channel_number)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;

  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  pPMDx->PORTMD   = PMD_OUT_ALL_HIGH_Z;                                            /* Port Output Mode Register */
  pPMDx->MODESEL  = 0xf;                                                /* Mode Select Register */

  pPMDx->MDOUT    = PMD_MDOUT_PWM_U
                  | PMD_MDOUT_PWM_V
                  | PMD_MDOUT_PWM_W
                  | PMD_MDOUT_OC_U3
                  | PMD_MDOUT_OC_V3
                  | PMD_MDOUT_OC_W3;
}

/*! \brief output off for the PMD Signals
  *
  * Normal state while motor is turning
  *
  * @param  channel_number: channel to use
  * @retval None
*/
void HAL_PMD_OutputOff(unsigned char channel_number)
{
  HAL_PMD_OutputPattern(channel_number, PMD_OPEN_ALL);
}

/*! \brief  PMD Signals for Bootstrapping / Braking
  *
  * Shortcut one half of the drivers
  *
  * @param  channel_number: channel to use
  * @param  pattern:        off / low / high side
  * @retval None
*/
void HAL_PMD_OutputPattern(unsigned char channel_number, PMD_PATTERN pattern)
{
  TSB_PMD_TypeDef*    pPMDx   = NULL;

  switch (channel_number)
  {
  case 0:
    pPMDx   = TSB_PMD0;
    break;
  case 1:
    pPMDx   = TSB_PMD1;
    break;
  case 2:
    pPMDx   = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  switch (pattern)
  {
  case PMD_SHORTCUT_LOW_SIDE:
    pPMDx->MDOUT    = PMD_MDOUT_OC_U1                                           /* PMD Output Control Register - shortcut low side*/
                    | PMD_MDOUT_OC_V1
                    | PMD_MDOUT_OC_W1;
    break;
  case PMD_SHORTCUT_HIGH_SIDE:
    pPMDx->MDOUT    = PMD_MDOUT_OC_U2                                           /* PMD Output Control Register - shortcut high side*/
                    | PMD_MDOUT_OC_V2
                    | PMD_MDOUT_OC_W2;
    break;
  case PMD_OPEN_ALL:
    pPMDx->MDOUT    = PMD_MDOUT_OC_U0                                           /* PMD Output Control Register - no output*/
                    | PMD_MDOUT_OC_V0
                    | PMD_MDOUT_OC_W0;
    break;
  default:
    return;
  }

  pPMDx->PORTMD   = PMD_OUT_ALL_HIGH_Z;                                            /* Port Output Mode Register */
  pPMDx->MODESEL  = PMD_MODE_BUS;                                               /* Mode Select Register */

}


static void HAL_PMD_ExecuteSoftBrake(unsigned char channel_number)
{
  static PMD_PATTERN pattern     = PMD_SHORTCUT_LOW_SIDE;
  static uint8_t     brake_count = 0;

  brake_count++;
  if (brake_count>100)
    brake_count=0;

  if (pattern==PMD_SHORTCUT_HIGH_SIDE)
    pattern = PMD_SHORTCUT_LOW_SIDE;
  else
    pattern = PMD_SHORTCUT_HIGH_SIDE;

  if (brake_count <= SystemValues[channel_number].Brakepercentage)
    HAL_PMD_OutputPattern(channel_number, pattern);
  else
    HAL_PMD_OutputPattern(channel_number, PMD_OPEN_ALL);
}

/*! \brief  Interrupt handler of PMD Channel 0
  *
  * @param  None
  * @retval None
*/
void INTPWM0_IRQHandler (void)
{

/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32PMD0PrevTime = u32PMD0StartTime;
    /* Get the new start time */
    u32PMD0StartTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32PMD0LoopOccuranceRate = u32PMD0StartTime - u32PMD0PrevTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */
    
#ifdef FIX_155_PMD_DELAY
  if((ChannelValues[0].MeasurementType == CURRENT_SHUNT_3) && (ve_irq_state[0] == VE_IRQ_OFF))
  {
    updatePmdRegisterSetting_Ch0();
  }
#endif
    
  if ((MotorControl[0].ParameterChange & MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD) != 0)
  {
    HAL_PMD_HandleParameterChangeSystem(0);
    MotorControl[0].ParameterChange &= ~MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD;
  }

  if (MotorControl[0].ActualStage.main == Stage_Brake)
    HAL_PMD_ExecuteSoftBrake(0);

#ifdef USE_DSO
  DSO_Log (PMD_CALLING , TSB_VE0 );
#endif
#ifdef USE_HSDSO
  HsDSO_Log_PMD(0);
  HsDSO_LogSWFOC(0);
#endif
  
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32PMD0StopTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32PMD0ProcessingTime = u32PMD0StopTime - u32PMD0StartTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */  
}

/*! \brief  Interrupt handler of PMD Channel 1
  *
  * @param  None
  * @retval None
*/
void INTPWM1_IRQHandler (void)
{
/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32PMD1PrevTime = u32PMD1StartTime;
    /* Get the new start time */
    u32PMD1StartTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32PMD1LoopOccuranceRate = u32PMD1StartTime - u32PMD1PrevTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */  
    
  if ((MotorControl[1].ParameterChange & MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD) != 0)
  {
    HAL_PMD_HandleParameterChangeSystem(1);
    MotorControl[1].ParameterChange &= ~MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD;
  }

  if (MotorControl[1].ActualStage.main == Stage_Brake)
    HAL_PMD_ExecuteSoftBrake(1);

#ifdef USE_DSO
  DSO_Log (PMD_CALLING , (void*)1 );
#endif
#ifdef USE_HSDSO
  HsDSO_Log_PMD(1);
  HsDSO_LogSWFOC(1);
#endif
  
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32PMD1StopTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32PMD1ProcessingTime = u32PMD1StopTime - u32PMD1StartTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */    
}

/*! \brief  Interrupt handler of PMD Channel 1
  *
  * @param  None
  * @retval None
*/
void INTPWM2_IRQHandler (void)
{
/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32PMD2PrevTime = u32PMD2StartTime;
    /* Get the new start time */
    u32PMD2StartTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32PMD2LoopOccuranceRate = u32PMD2StartTime - u32PMD2PrevTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */  
    
  if ((MotorControl[2].ParameterChange & MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD) != 0)
  {
    HAL_PMD_HandleParameterChangeSystem(2);
    MotorControl[2].ParameterChange &= ~MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD;
  }

  if (MotorControl[2].ActualStage.main == Stage_Brake)
    HAL_PMD_ExecuteSoftBrake(2);

#ifdef USE_DSO
  DSO_Log (PMD_CALLING , (void*)2 );
#endif
#ifdef USE_HSDSO
  HsDSO_Log_PMD(2);
  HsDSO_LogSWFOC(2);
#endif
  
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32PMD2StopTime = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32PMD2ProcessingTime = u32PMD2StopTime - u32PMD2StartTime;
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */    
}


/*! \brief  PMD_HandleParameterChangeBoard
  *
  * Handle change in Board parameters
  *
  * @param  channel_number: channel to use
  * @retval None
*/
void HAL_PMD_HandleParameterChangeBoard(uint8_t channel_number)
{
  TSB_PMD_TypeDef*  pPMD  = NULL;

  switch (channel_number)
  {
  case 0:
    pPMD    = TSB_PMD0;
    break;
  case 1:
    pPMD    = TSB_PMD1;
    break;
  case 2:
    pPMD    = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }


  /* PMD Output Setting Register */
  pPMD->MDPOT    = PMD_PSYNC_CNT_2 |
                   ChannelValues[channel_number].PolL<<2 |
                   ChannelValues[channel_number].PolH<<3 ;
  pPMD->TRGSYNCR = PMD_TRGSYNCR_TSYNCS;

  pPMD->DTR      = ChannelValues[channel_number].Deadtime / 50;

  pPMD->RATE    = ((uint32_t)(((uint64_t)(1 << 24U) * SystemValues[channel_number].PwmFrequency) / (80000000)));

  /* Set Dead Time Register(100ns@80MHz) */
  switch(ChannelValues[channel_number].MeasurementType)
  {
  case CURRENT_SHUNT_1:
    pPMD->TRGCR  = PMD_TRG_1SHUNT;                                              /* Trigger Control Register */
    pPMD->TRGMD  = PMD_TRG_MODE_FIXED;                                          /* Trigger Output Mode Setting */
    pPMD->TRGSEL = PMD_TRGSEL_PMDTRG0;                                          /* Trigger Output Select Register */
    break;
  case CURRENT_SHUNT_3:
    pPMD->TRGCR  = PMD_TRG_3SHUNT;                                              /* Trigger is PWM peek */
    pPMD->TRGMD  = PMD_TRG_MODE_VARIABLE;                                       /* TRG0 change to TRGx (x:Sector No) */
    pPMD->TRGSEL = PMD_TRGSEL_PMDTRG0;                                          /* TRG No = Sector = 0r */
    break;
  case CURRENT_SENSOR_2:
    pPMD->TRGCR  = PMD_TRG_2SENSOR;                                             /* Trigger is PWM peek */
    pPMD->TRGMD  = PMD_TRG_MODE_VARIABLE;                                       /* TRG0 change to TRGx (x:Sector No) */
    pPMD->TRGSEL = PMD_TRGSEL_PMDTRG0;                                          /* TRG No = Sector = 0r */
    break;
  default:
    assert_param(0);
    break;
  }

  switch (channel_number)
  {
  case 0:
    NVIC_SetPriority(INTPWM0_IRQn, INTERRUPT_PRIORITY_PMD);
    NVIC_EnableIRQ(INTPWM0_IRQn);
    
#ifdef USE_EMERGENCY_SIGNAL
    HAL_PMD_EmergencyReset(channel_number);
    NVIC_SetPriority(INTEMG0_IRQn, INTERRUPT_PRIORITY_ERROR);
    NVIC_ClearPendingIRQ(INTEMG0_IRQn);
    NVIC_EnableIRQ(INTEMG0_IRQn);
#else
    HAL_PMD_EmergencyDisable(channel_number);
#endif /* USE_EMERGENCY_SIGNAL */
    
#ifdef USE_OVERVOLTAGE_SIGNAL
    HAL_PMD_OvervoltageReset(channel_number);
    NVIC_SetPriority(INTOVV0_IRQn, INTERRUPT_PRIORITY_ERROR);
    NVIC_ClearPendingIRQ(INTOVV0_IRQn);
    NVIC_EnableIRQ(INTOVV0_IRQn);
    HAL_PMD_OvervoltageEnable(channel_number);
#else
    HAL_PMD_OvervoltageDetectDisable(channel_number);
#endif /* USE_OVERVOLTAGE_SIGNAL */
    break;
    
  case 1:
    NVIC_SetPriority(INTPWM1_IRQn, INTERRUPT_PRIORITY_PMD);
    NVIC_EnableIRQ(INTPWM1_IRQn);
    
#ifdef USE_EMERGENCY_SIGNAL
    HAL_PMD_EmergencyReset(channel_number);
    NVIC_SetPriority(INTEMG1_IRQn, INTERRUPT_PRIORITY_ERROR);
    NVIC_ClearPendingIRQ(INTEMG1_IRQn);
    NVIC_EnableIRQ(INTEMG1_IRQn);
#else
    HAL_PMD_EmergencyDisable(channel_number);
#endif /* USE_EMERGENCY_SIGNAL */
    
#ifdef USE_OVERVOLTAGE_SIGNAL
    HAL_PMD_OvervoltageReset(channel_number);
    NVIC_SetPriority(INTOVV1_IRQn, INTERRUPT_PRIORITY_ERROR);
    NVIC_ClearPendingIRQ(INTOVV1_IRQn);
    NVIC_EnableIRQ(INTOVV1_IRQn);
    HAL_PMD_OvervoltageEnable(channel_number);
#else
    HAL_PMD_OvervoltageDetectDisable(channel_number);
#endif /* USE_OVERVOLTAGE_SIGNAL */
    break;
    
  case 2:
    NVIC_SetPriority(INTPWM2_IRQn, INTERRUPT_PRIORITY_PMD);
    NVIC_EnableIRQ(INTPWM2_IRQn);
    
#ifdef USE_EMERGENCY_SIGNAL
    HAL_PMD_EmergencyReset(channel_number);
    NVIC_SetPriority(INTEMG2_IRQn, INTERRUPT_PRIORITY_ERROR);
    NVIC_ClearPendingIRQ(INTEMG2_IRQn);
    NVIC_EnableIRQ(INTEMG2_IRQn);
#else
    HAL_PMD_EmergencyDisable(channel_number);
#endif /* USE_EMERGENCY_SIGNAL */
    
#ifdef USE_OVERVOLTAGE_SIGNAL
    HAL_PMD_OvervoltageEnable(channel_number);
    HAL_PMD_OvervoltageReset(channel_number);
    NVIC_SetPriority(INTOVV2_IRQn, INTERRUPT_PRIORITY_ERROR);
    NVIC_ClearPendingIRQ(INTOVV2_IRQn);
    NVIC_EnableIRQ(INTOVV2_IRQn);
#else
    HAL_PMD_OvervoltageDetectDisable(channel_number);
#endif /* USE_OVERVOLTAGE_SIGNAL */
    break;
    
  default:
    assert_param(0);
    break;
  }
  
}

/*! \brief  Initialize the PMD Unit
  *
  * @param  channel_number: channel to configure
  * @param  deadtime:       Deadtime of the FETs
  * @param  shunt_type:     selects between 1-shunt & 3-shunt mode
  * @retval None
*/
void HAL_PMD_HWInit (uint8_t channel_number)
{
  TSB_PMD_TypeDef* pPMD= NULL;  

  switch (channel_number)
  {
  case 0:
    pPMD    = TSB_PMD0;
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_PMD_CH0);
    break;
  case 1:
    pPMD    = TSB_PMD1;
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_PMD_CH1);
    break;
  case 2:
    pPMD   = TSB_PMD2;
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_PMD_CH2);
    break;
  default:
    assert_param(0);
    break;
  }
  
  pPMD->MDEN &= ~(uint32_t)PMD_ENABLE;

  PMD_IOInit(channel_number);

  HAL_PMD_HandleParameterChangeSystem(channel_number);
  HAL_PMD_HandleParameterChangeBoard(channel_number);
  HAL_PMD_EmergencyDisable(channel_number);

  pPMD->MDCR     =
                   (1 << 14) | (1 << 12) | (1 << 10) |
                   (0x2 <<8) |
                   (1 << 4)|
                    (1 << 1);
  
  pPMD->MDEN |= PMD_ENABLE;
  
}

