/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */


/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/

#include "config.h"
#include TMPM_HEADER_FILE
#include TMPM_SPI_HEADER_FILE
#include "hal_spi.h"
#include "hal_cg.h"

#define HAL_SPI_CH_COUNT (2)
#define HAL_SPI_TIMEOUT  (0xFFFFFFFF)
/*----------------------------------*/
/**
 * @brief  Channel 0 & 1 I2C handle structures.
*/
/*----------------------------------*/
HAL_SPI_Data   halSpi[HAL_SPI_CH_COUNT];


/**
 *  @defgroup SPI Error Flags
 *  @brief    Available SPI error Macro Definisiton.
 *  @{
 */
#define CR_PARITY_MASK      ((uint32_t) 0x0000009F)
#define CR_ERROR_MASK       ((uint32_t) 0x0000000F)
#define CR_TRGERR_FLAG      ((uint8_t) 0x01)
#define CR_UDRERR_FLAG      ((uint8_t) 0x02)
#define CR_OVRERR_FLAG      ((uint8_t) 0x04)
#define CR_PERR_FLAG        ((uint8_t) 0x08)
/**
 *  @}
 */ /* End of group SPI Error Flags */


/*! \brief  Enable SPI channel
  *
  * @param  SPIx:       channel to enable
  * @retval None
*/
void HAL_SPI_Enable(uint8_t channel)
{
  assert_param(channel>HAL_SPI_CH_COUNT);                                       /* Check the parameters */

  halSpi[channel].tspi.p_instance->CR0 |= TSPI_ENABLE;                           /* Set TSPIxCR0<TSPIE> to enable SPIx */

}

/*! \brief  Disable SPI channel
  *
  * @param  SPIx:       channel to disable
  * @retval None
*/
void HAL_SPI_Disable(uint8_t channel)
{
  assert_param(channel>HAL_SPI_CH_COUNT);                                       /* Check the parameters */

  halSpi[channel].tspi.p_instance->CR0 &= ~TSPI_ENABLE;                         /* Clear TSPIxCR0<TSPIE> to disable SPIx */
}

/*! \brief  Perform a SW reset of SPI channel
  *
  * @param  SPIx:       channel to reset
  * @retval None
*/
void HAL_SPI_SWReset(uint8_t channel)
{

  assert_param(channel>HAL_SPI_CH_COUNT);                                       /* Check the parameters */

  halSpi[channel].tspi.p_instance->CR0 = TSPI_RESET10 | TSPI_ENABLE;

  halSpi[channel].tspi.p_instance->CR0 = TSPI_RESET01 | TSPI_ENABLE;

  __NOP();
  __NOP();

}

/*! \brief  Get data from SPI channel
  *
  * @param  SPIx:       channel to use
  * @retval Received Data
*/
uint32_t HAL_SPI_GetRxData(uint8_t channel)
{
  assert_param(channel>HAL_SPI_CH_COUNT);                                       /* Check the parameters */

  return halSpi[channel].tspi.p_instance->DR;                                   /* return received packet */
}

/*! \brief  Send Data on SPI channel
  *
  * @param  SPIx:       channel to use
  * @param  Data:       data to send
  * @retval None
*/
void HAL_SPI_SetTxData(uint8_t channel, uint32_t Data)
{
  volatile unsigned char wait;

  halSpi[channel].tspi.p_instance->DR = Data;

  /* start SPI communication */

  while ((halSpi[channel].tspi.p_instance->SR & 0xf0000) != 0)
  {
    ;   /*wait until transmission is finished */
  }

}

HAL_SPI_Err  HAL_SPI_MasterTransfer(uint8_t channel, const HAL_SPI_TransRec_t *transfer)
{
    tspi_transmit_t transmit;
    assert_param(channel < HAL_SPI_CH_COUNT);

    transmit.tx8.num = transfer->trx8.num;
    transmit.tx8.p_data= transfer->trx8.p_data;

    if (SUCCESS == tspi_master_write(&halSpi[channel].tspi, &transmit, HAL_SPI_TIMEOUT))
    {
      return HAL_SPI_NO_ERR;
    }
    return HAL_SPI_TRANSMIT_ERR;
}


HAL_SPI_Err  HAL_SPI_MasterWriteBytes(uint8_t channel, HAL_SPI_TransRec_t *write)
{
    tspi_transmit_t transmit;
    assert_param(channel < HAL_SPI_CH_COUNT);

    /* blocking call is only supported for frame length 8bit (1 byte) only  */
    transmit.tx8.num = write->trx8.num;
    transmit.tx8.p_data= write->trx8.p_data;

    if (SUCCESS == tspi_master_write(&halSpi[channel].tspi, &transmit, HAL_SPI_TIMEOUT))
    {
      return HAL_SPI_NO_ERR;
    }
    return HAL_SPI_TRANSMIT_ERR;
}

HAL_SPI_Err HAL_SPI_MasterReadBytes(uint8_t channel, const HAL_SPI_TransRec_t *read)
{
    tspi_receive_t receive;
    assert_param(channel < HAL_SPI_CH_COUNT);                                       /* Check the parameters */

    receive.rx8.num = read->trx8.num;
    receive.rx8.p_data = read->trx8.p_data;

    if (SUCCESS == tspi_master_read(&halSpi[channel].tspi, &receive, HAL_SPI_TIMEOUT))
    {
      return HAL_SPI_NO_ERR;
    }
    return HAL_SPI_RECEIVE_ERR;
}

HAL_SPI_Err  HAL_SPI_MasterReceive(uint8_t channel, HAL_SPI_TransRec_t *receive)
{
    tspi_receive_t tReceive;
    assert_param(channel < HAL_SPI_CH_COUNT);   /* Check the parameters */

    tReceive.rx8.num = receive->trx8.num;
    tReceive.rx8.p_data = receive->trx8.p_data;

    if (SUCCESS == tspi_master_read(&halSpi[channel].tspi, &tReceive, HAL_SPI_TIMEOUT))
    {
      return HAL_SPI_NO_ERR;
    }
    return HAL_SPI_RECEIVE_ERR;
}

/*! \brief  Get error state of the SPI channel
  *
  * @param  SPIx:       channel to configure
  * @retval SPI_Err
*/
HAL_SPI_Err HAL_SPI_GetErrState(uint8_t channel)
{
  HAL_SPI_Err retval = HAL_SPI_NO_ERR;

  assert_param(channel>HAL_SPI_CH_COUNT);                                       /* Check the parameters */

  switch((uint8_t) (halSpi[channel].tspi.p_instance->ERR & CR_ERROR_MASK))
  {
  case CR_TRGERR_FLAG:                                                            /* Check overrun flag */
    retval = HAL_SPI_TRGERR;
    break;
  case CR_UDRERR_FLAG:                                                            /* Check framing flag */
    retval = HAL_SPI_UNDERRUN;
    break;
  case CR_OVRERR_FLAG:                                                            /* Check overrun flag */
    retval = HAL_SPI_OVERRUN;
    break;
  case CR_PERR_FLAG:                                                            /* Check framing flag */
    retval = HAL_SPI_PARITY_ERR;
    break;
  default:
    break;
  }
  return retval;
}

void transmitHandler(uint32_t id, Result result)
{
    if ((0 == id) || (1 == id))
    {
        if (halSpi[id].transmitHandler)
        {
            if (SUCCESS != result)
            {
                halSpi[id].transmitHandler (HAL_SPI_TRANSMIT_ERR);
            }
            else
            {
                halSpi[id].transmitHandler (HAL_SPI_NO_ERR);
            }
        }
    }
}

void receiveHandler(uint32_t id, Result result, tspi_receive_t *p_info)
{
    HAL_SPI_TransRec_t receive;

    if ((0 == id) || (1 == id))
    {
        if (halSpi[id].receiveHandler)
        {
            if (SUCCESS != result)
            {
                halSpi[id].receiveHandler (HAL_SPI_RECEIVE_ERR, &receive);
            }
            else
            {
                receive.trx8.p_data = p_info->rx8.p_data;
                receive.trx8.num = p_info->rx8.num;
                halSpi[id].receiveHandler (HAL_SPI_NO_ERR, &receive);
            }
        }
    }
}

static void HAL_SPI_CalculateClockParameter ( HAL_SPI_Speed speed, uint32_t *brck, uint32_t* brs)
{
  switch (speed)
  {
    case HAL_SPI_20_MHZ:
      *brck= TSPI_BR_CLOCK_1;
      *brs=  TSPI_BR_DIVIDER_1;
      break;
    case HAL_SPI_10_MHZ:
      *brck= TSPI_BR_CLOCK_1;
      *brs=  TSPI_BR_DIVIDER_2;
      break;
    case HAL_SPI_5_MHZ:
      *brck= TSPI_BR_CLOCK_2;
      *brs=  TSPI_BR_DIVIDER_2;
      break;
    case HAL_SPI_3_33_MHZ:
      *brck= TSPI_BR_CLOCK_2;
      *brs=  TSPI_BR_DIVIDER_3;
      break;
    case HAL_SPI_2_5_MHZ:
      *brck= TSPI_BR_CLOCK_2;
      *brs=  TSPI_BR_DIVIDER_4;
      break;
   case HAL_SPI_500_KHZ:
      *brck= TSPI_BR_CLOCK_8;
      *brs=  TSPI_BR_DIVIDER_5;
      break;
   case HAL_SPI_250_KHZ:
      *brck= TSPI_BR_CLOCK_8;
      *brs=  TSPI_BR_DIVIDER_10;
      break;
   case HAL_SPI_125_KHZ:
   default:
     *brck= TSPI_BR_CLOCK_16;
      *brs=  TSPI_BR_DIVIDER_10;
      break;
  }
}


/*! \brief  Initialize the SPI channel
  *
  * @param  SPIx:       channel to configure
  * @param  InitStruct: configuration data for the channel (such as speed)
  * @retval None
*/
HAL_SPI_Err  HAL_SPI_Init(const HAL_SPI_CfgData* initStruct)
{
  assert_param(initStruct != (void*)0);
  assert_param(initStruct->channel < HAL_SPI_CH_COUNT);                                       /* Check the parameters */

  switch (initStruct->channel)
  {
  case 0:
      /* (1) Enable the peripheral clock of TSPI0  */
      HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_TSPI_CH0);
      halSpi[initStruct->channel].tspi.p_instance = TSB_TSPI0;
      halSpi[initStruct->channel].tspi.init.id = initStruct->channel;
      break;
  case 1:
      /* (1) Enable the peripheral clock of TSPI1  */
      HAL_CG_EnableMiddleSpeedClockA(HAL_CG_CLK_TSPI_CH1);
      halSpi[initStruct->channel].tspi.p_instance = TSB_TSPI1;
      halSpi[initStruct->channel].tspi.init.id = initStruct->channel;
      break;
  default:
    return HAL_SPI_INVALID_PARAMETER;
  }
  switch (initStruct->chipSelect)
  {
  case 0:
      halSpi[initStruct->channel].tspi.init.cnt1.cssel = TSPI_TSPIxCS0_ENABLE;
      break;
  case 1:
      halSpi[initStruct->channel].tspi.init.cnt1.cssel = TSPI_TSPIxCS1_ENABLE;
      break;
  default:
      return HAL_SPI_INVALID_PARAMETER;
  }

  halSpi[initStruct->channel].tspi.init.cnt1.trxe = TSPI_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt1.tspims = TSPI_SPI_MODE;

  /* Enable or disable transmission or reception */
  switch(initStruct->transferMode)
  {
  case SPI_ENABLE_RX:
    halSpi[initStruct->channel].tspi.init.cnt1.tmmd = TSPI_RX_ONLY;
    break;

  case SPI_ENABLE_TX:
    halSpi[initStruct->channel].tspi.init.cnt1.tmmd = TSPI_TX_ONLY;
    break;

  default:
    halSpi[initStruct->channel].tspi.init.cnt1.tmmd = TSPI_TWO_WAY;
    break;
  }

  if (initStruct->transferMode)
  {
    halSpi[initStruct->channel].tspi.init.cnt1.mstr = TSPI_MASTER_OPEARTION;
  }
  else
  {
    halSpi[initStruct->channel].tspi.init.cnt1.mstr = TSPI_SLAVE_OPERATION;
  }

  halSpi[initStruct->channel].tspi.init.cnt1.fc = TSPI_TRANS_RANGE_SINGLE;
  /* Set Initial parameter of TSPIxCR2 */
  halSpi[initStruct->channel].tspi.init.cnt2.tidle = TSPI_TIDLE_HI;
  halSpi[initStruct->channel].tspi.init.cnt2.txdemp = TSPI_TXDEMP_HI;
  halSpi[initStruct->channel].tspi.init.cnt2.rxdly = TSPI_RXDLY_40MHz_OVER;
  halSpi[initStruct->channel].tspi.init.cnt2.til = TSPI_TX_FILL_LEVEL_7;
  halSpi[initStruct->channel].tspi.init.cnt2.ril = TSPI_RX_FILL_LEVEL_7;
  halSpi[initStruct->channel].tspi.init.cnt2.inttxwe = TSPI_TX_INT_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt2.intrxwe = TSPI_RX_INT_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt2.inttxfe = TSPI_TX_FIFO_INT_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt2.intrxfe = TSPI_RX_FIFO_INT_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt2.interr = TSPI_ERR_INT_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt2.dmate = TSPI_TX_DMA_INT_DISABLE;
  halSpi[initStruct->channel].tspi.init.cnt2.dmare = TSPI_RX_DMA_INT_DISABLE;
  HAL_SPI_CalculateClockParameter (initStruct->speed,
                           &halSpi[initStruct->channel].tspi.init.brd.brck,
                           &halSpi[initStruct->channel].tspi.init.brd.brs);

  /* Set Initial parameter of TSPIxFMTR0 */
  halSpi[initStruct->channel].tspi.init.fmr0.dir = TSPI_DATA_DIRECTION_MSB;
  halSpi[initStruct->channel].tspi.init.fmr0.fl = initStruct->length << 24;
  halSpi[initStruct->channel].tspi.init.fmr0.fint = TSPI_INTERVAL_TIME_0;
  halSpi[initStruct->channel].tspi.init.fmr0.cs3pol = TSPI_TSPIxCS3_NEGATIVE;
  halSpi[initStruct->channel].tspi.init.fmr0.cs2pol = TSPI_TSPIxCS2_NEGATIVE;
  halSpi[initStruct->channel].tspi.init.fmr0.cs1pol = TSPI_TSPIxCS1_NEGATIVE;
  halSpi[initStruct->channel].tspi.init.fmr0.cs0pol = TSPI_TSPIxCS0_NEGATIVE;
  if (HAL_SPI_CK_1ST_EDGE == initStruct->clkSampling)
  {
    halSpi[initStruct->channel].tspi.init.fmr0.ckpha = TSPI_SERIAL_CK_1ST_EDGE;
  }
  else
  {
    halSpi[initStruct->channel].tspi.init.fmr0.ckpha = TSPI_SERIAL_CK_2ND_EDGE;
  }

  if (HAL_SPI_CK_IDLE_LOW == initStruct->clkIdleLvl)
  {
    halSpi[initStruct->channel].tspi.init.fmr0.ckpol = TSPI_SERIAL_CK_IDLE_LOW;
  }
  else
  {
    halSpi[initStruct->channel].tspi.init.fmr0.ckpol = TSPI_SERIAL_CK_IDLE_HI;
  }
  halSpi[initStruct->channel].tspi.init.fmr0.csint = TSPI_MIN_IDLE_TIME_1;
  halSpi[initStruct->channel].tspi.init.fmr0.cssckdl = TSPI_SERIAL_CK_DELAY_1;
  halSpi[initStruct->channel].tspi.init.fmr0.sckcsdl = TSPI_NEGATE_1;
  /* Set Initial parameter of TSPIxFMTR1 */
  halSpi[initStruct->channel].tspi.init.fmr1.vpe = TSPI_PARITY_DISABLE;
  halSpi[initStruct->channel].tspi.init.fmr1.vpm = TSPI_PARITY_BIT_ODD;
  if (initStruct->transmitHandler)
  {
      /* register callback function for spi-driver */
      halSpi[initStruct->channel].tspi.transmit.handler = transmitHandler;
      /* register callback function for hal spi-driver */
      halSpi[initStruct->channel].transmitHandler = initStruct->transmitHandler;
  }
  else
  {
      halSpi[initStruct->channel].tspi.transmit.handler = (void*)0;
  }
  if (initStruct->receiveHandler)
  {
      /* register callback function for spi driver */
      halSpi[initStruct->channel].tspi.receive.handler = receiveHandler;
      /* register callback function for spi driver */
      halSpi[initStruct->channel].receiveHandler = initStruct->receiveHandler;
  }
  else
  {
    halSpi[initStruct->channel].tspi.receive.handler = (void*)0;
  }
  if (SUCCESS == tspi_init(&halSpi[initStruct->channel].tspi))
  {
     return HAL_SPI_NO_ERR;
  }
  return HAL_SPI_INIT_ERR;
}


void (*spi_irq_callback[4][2])();

static void irq_callback (uint8_t channel, HAL_SPI_Direction direction)
{
  assert_param(spi_irq_callback[channel][((uint8_t)direction)>>1]);

  if (HAL_SPI_RX == direction)
  {
    tspi_irq_handler_receive(&halSpi[channel].tspi);
  }
  else
  {
    tspi_irq_handler_transmit(&halSpi[channel].tspi);
  }
  spi_irq_callback[channel][((uint8_t)direction)>>1]();
}

void INTTX0_IRQHandler(void)
{
  irq_callback(0,HAL_SPI_TX);
}

void INTRX0_IRQHandler(void)
{
  irq_callback(0,HAL_SPI_RX);
}

void INTTX1_IRQHandler(void)
{
  irq_callback(1,HAL_SPI_TX);
}

void INTRX1_IRQHandler(void)
{
  irq_callback(1,HAL_SPI_RX);
}


void HAL_SPI_EnableIRQ(uint8_t channel, HAL_SPI_Direction direction)
{
  switch (direction)
  {
  case HAL_SPI_TX:
    switch (channel)
    {
    case 0:
      NVIC_EnableIRQ(INTSC0TX_IRQn);
      break;
    case 1:
      NVIC_EnableIRQ(INTSC1TX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_SPI_RX:
    switch (channel)
    {
    case 0:
      NVIC_EnableIRQ(INTSC0RX_IRQn);
      break;
    case 1:
      NVIC_EnableIRQ(INTSC1RX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  default:
    assert_param(0);
    break;
  }
}

void HAL_SPI_DisableIRQ(uint8_t channel, HAL_SPI_Direction direction)
{
  switch (direction)
  {
  case HAL_SPI_TX:
    switch (channel)
    {
    case 0:
      NVIC_DisableIRQ(INTSC0TX_IRQn);
      break;
    case 1:
      NVIC_DisableIRQ(INTSC1TX_IRQn);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_SPI_RX:
    switch (channel)
    {
    case 0:
      NVIC_DisableIRQ(INTSC0RX_IRQn);
      break;
    case 1:
      NVIC_DisableIRQ(INTSC1RX_IRQn);
      break;
    default:
      assert_param(0);
    break;
    }
    break;
  default:
    assert_param(0);
  }
}

void HAL_SPI_SetIRQPriority(uint8_t channel, HAL_SPI_Direction direction, uint32_t prio)
{
  switch (direction)
  {
  case HAL_SPI_TX:
    switch (channel)
    {
    case 0:
      NVIC_SetPriority(INTSC0TX_IRQn,prio);
      break;
    case 1:
      NVIC_SetPriority(INTSC1TX_IRQn,prio);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  case HAL_SPI_RX:
    switch (channel)
    {
    case 0:
      NVIC_SetPriority(INTSC0RX_IRQn,prio);
      break;
    case 1:
      NVIC_SetPriority(INTSC1RX_IRQn,prio);
      break;
    default:
      assert_param(0);
      break;
    }
    break;
  default:
    assert_param(0);
    break;
  }
}


