/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "config.h"
#include TMPM_HEADER_FILE
#include BOARD_CONFIG_HEADER_FILE
#include "gain.h"
#include "hal_cg.h"
#include "hal_gpio.h"
#include "debug.h"
#include "board.h"
#include "motor_control.h"
#include "hal_ve.h"
#include "dso.h"
#include "hsdso.h"
#include "hal_encoder.h"
#include "stall_detect.h"
#include "hv_serial_communication.h"
#include "board.h"
#include "turn_control.h"
#include "external_speed_control.h"
#include "temperature_control.h"
#include "fwmathlib.h"
#include "speed_controller.h"
#include "torque_controller.h"
#include "user_callbacks.h"
#include "txz_driver_def.h"
#include "performance_measurement.h"
#ifdef FIX_155_PMD_DELAY
#include "hal_pmd.h"
#endif

#ifdef FIX_155_PMD_DELAY
uint8_t ve_irq_state[3] = {0};
#endif

/******************************************************************************/
/******************************************************************************/
/***************************** Helper Functions *******************************/
/******************************************************************************/
/******************************************************************************/

/*! \brief  Vector Engine Interrupts
  *
  * @param  channel_number: channel to work on
  * @param  state: enable or disable
  * @retval None
*/
void HAL_VE_IRQ(unsigned char channel_number,VE_IRQ_State state)
{
  switch (state)
  {
  case VE_IRQ_OFF:
    switch (channel_number)
    {
    case 0:
      NVIC_DisableIRQ(INTVCN0_IRQn);                                            /* disable VE0 interrupt while configure */
#ifdef FIX_155_PMD_DELAY
      ve_irq_state[channel_number] = VE_IRQ_OFF;
#endif
      break;
    default:
      assert_param(0);
      break;
    }
    break;

  case VE_IRQ_ON:
    switch (channel_number)
    {
    case 0:
      NVIC_EnableIRQ(INTVCN0_IRQn);                                             /* enable VE0 interrupt while configure */
#ifdef FIX_155_PMD_DELAY
      ve_irq_state[channel_number] = VE_IRQ_ON;
#endif
      break;
    default:
      assert_param(0);
      break;
    }
    break;

  default:
    assert_param(0);
    break;
  }
}

void HAL_VE_PMD_Output(uint8_t channel_number, VE_PMD_State state)
{
  TSB_VE_TypeDef * pVEPMDx = NULL;

  switch (channel_number)
  {
  case 0:
    pVEPMDx = TSB_VE0;
    break;
  default:
    assert_param(0);
    break;
  }

  if (state)
    pVEPMDx->OUTCR = 0x1ff;
  else
    pVEPMDx->OUTCR = 0x0;
}

void HAL_VE_PMD_EMGReturn(uint8_t channel_number)
{
  TSB_VE_TypeDef * pVEPMDx = NULL;

  switch (channel_number)
  {
  case 0:
    pVEPMDx = TSB_VE0;
    break;
  default:
    assert_param(0);
    break;
  }

  pVEPMDx->EMGRS = 0x1;
}

/******************************************************************************/
/******************************************************************************/
/************************* Precalculations and Inits **************************/
/******************************************************************************/
/******************************************************************************/


/*! \brief  Initialize Vector Engine PI Registers
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void HAL_VE_InitPISettings(unsigned char channel_number)
{
  TSB_VE_TypeDef*     pVEx=NULL;

  switch (channel_number)
  {
  case 0:
    pVEx    = TSB_VE0;
    break;
  default:
    assert_param(0);
    break;
  }

  pVEx->TPWM  = MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency;

  pVEx->CIDKI = PIControl[channel_number].PiIdKi;                               /* Initialize PI-Control value */
  pVEx->CIDKP = PIControl[channel_number].PiIdKp;                               /* Initialize PI-Control value */
  pVEx->CIQKI = PIControl[channel_number].PiIqKi;                               /* Initialize PI-Control value */
  pVEx->CIQKP = PIControl[channel_number].PiIqKp;                               /* Initialize PI-Control value */

}

/*! \brief  Initialize Vector Engine PWM Registers
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void InitPWMSettings(unsigned char channel_number)
{
  TSB_VE_TypeDef*     pVEx=NULL;

  switch (channel_number)
  {
  case 0:
    pVEx    = TSB_VE0;
    break;
  default:
    assert_param(0);
    break;
  }

  pVEx->TPWM    = MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency;
  TSB_VE0->TADC = (uint16_t) ( (uint64_t) 185                                   /* AD Conversion Time is 1.85us */
                                        * CPU_CLOCK
                                        / 1000000
                                        / 100);
}

/*! \brief  Initialize the Vector Engine
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void HAL_VE_Init(unsigned char channel_number)
{
  TSB_VE_TypeDef*     pVEx=NULL;

  switch (channel_number)
  {
  case 0:
    HAL_CG_EnableMiddleSpeedClockB(HAL_CG_CLK_A_VE_CH0);
    pVEx    = TSB_VE0;
    NVIC_SetPriority(INTVCN0_IRQn, INTERRUPT_PRIORITY_VE);
    break;
  default:
    assert_param(0);
    break;
  }

  TSB_VE0->EN |= VE_ENABLE;                                                     /* VE Enable */

  pVEx->FMODE = (channel_number << 6) |                                         /* Always set this bits (madatory due to TD) */
                (channel_number << 4);

  switch (ChannelValues[channel_number].MeasurementType)
  {
  case CURRENT_SHUNT_1:
    pVEx->FMODE |=  VE_FMODE_CURR_DETECT_1SHUNT_DOWN |
                    VE_FMODE_2PHASE_MODULATION       |
                    VE_FMODE_PWM_SHIFT_ENABLE;
    pVEx->FPWMCHG = FIXPOINT_15/200;                                            /* Set PWM Shift Change Frq to MzMAX/200 */
    break;

  case CURRENT_SHUNT_3:
    pVEx->FMODE |=  VE_FMODE_3PHASE_MODULATION;
    break;

  case CURRENT_SENSOR_2:
    pVEx->FMODE |=  VE_FMODE_CURR_DETECT_2SENSOR |
                    VE_FMODE_3PHASE_MODULATION;
    break;
  }

  TSB_VE0->REPTIME |= (VE_REPTIME_1 << (channel_number*4) );                    /* Repeat time =1time */
  TSB_VE0->TRGMODE |= ( (1<<channel_number) << (channel_number*2) );

  InitPWMSettings(channel_number);
  HAL_VE_InitPISettings(channel_number);

  TSB_VE0->CPURUNTRG |= (1<<channel_number);

  __DSB();                                                                      /* flush the pipeline */
  __enable_irq();                                                               /* enable global interrupts */

}

float data1,data2;
int16_t VE_ActualMotorPower[3];

extern uint32_t jump_cnt=0;

extern int32_t d_theta_diff0;
extern int32_t d_theta_diff1;
int32_t d_theta_diff2;
int32_t d_theta_diff;
extern int32_t d_sec_diff1;


/******************************************************************************/
/******************************************************************************/
/*************************** IRQ part of VE ***********************************/
/******************************************************************************/
/******************************************************************************/

/*! \brief  Common part for the Vector Engine IRQ
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void IRQ_Common(uint8_t channel_number)
{
  TSB_VE_TypeDef*     pVEx    = NULL;
#ifdef USE_MOTOR_DISCONNECT_DETECTION
  uint32_t meas_current,calc_current;
#endif /* USE_MOTOR_DISCONNECT_DETECTION */

/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32IRQLoopTimePrev[channel_number] = u32IRQLoopTimeStart[channel_number]; 
    /* Get the new start time */
    u32IRQLoopTimeStart[channel_number] = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32IRQLoopOccuranceRate[channel_number] = u32IRQLoopTimeStart[channel_number] - u32IRQLoopTimePrev[channel_number];                             
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */

  switch (channel_number)
  {
  case 0:
    pVEx    = TSB_VE0;
    break;
  default:
    assert_param(0);
    break;
  }

  /*****************************************/
  /************* Task selection ************/
  /*****************************************/
  switch (MotorControl[channel_number].ActualStage.main)
  {
  /************** Brake Stage **************/
  case Stage_Emergency:
    pVEx->MODE        = VE_EMGRETURN;                                           /* Emergency Return */

    TSB_VE0->TASKAPP  = (TSB_VE0->TASKAPP                                       /* VE start from sin/cos computation */
                         & ~(VE_TASKAPP_CLEAN_MASK<<(channel_number*4)))
                         | VE_TASKAPP_OUTPUT_CONTROL<<(channel_number*4);
    TSB_VE0->ACTSCH   = (TSB_VE0->ACTSCH
                         & ~(VE_ACTSCH_CLEAN_MASK<<(channel_number*4)))
                         | VE_ACTSCH_SCHEDULE_9<<(channel_number*4);            /* Schedule 9 */
    break;

  /*********** Bootstrap Stage *************/
  /*************** Stop Stage **************/
  /************** Brake Stage **************/
  case Stage_Bootstrap:
  case Stage_Stop:
  case Stage_Brake:
    pVEx->MODE        = VE_MODE_OFF;                                            /* Output OFF */
    break;

  /******** Zero Current Measurement *******/
  case Stage_ZeroCurrentMeasure:
    pVEx->MODE        = VE_ZEROCURRENTEN;                                       /* Input 0 current */
    TSB_VE0->TASKAPP  = (TSB_VE0->TASKAPP                                       /* VE start from current control */
                         & ~VE_TASKAPP_CLEAN_MASK)
                         | VE_TASKAPP_OUTPUT_CONTROL;
    TSB_VE0->ACTSCH   = (TSB_VE0->ACTSCH
                         & ~VE_ACTSCH_CLEAN_MASK)
                         | VE_ACTSCH_SCHEDULE_9;                                /* Schedule 9 */
    break;

  /********** Position Stage ***************/
  /******* Forced Commutation Stage ********/
  /*********** ChangeUp Stage **************/
  /************* FOC Stage *****************/
  case Stage_Initposition:
  case Stage_Force:
  case Stage_FOC:
    pVEx->MODE        = VE_OUTPUTENABLE;                                        /* Output ON */
#ifndef USE_USER_CALLBACKS
    TSB_VE0->TASKAPP  = (TSB_VE0->TASKAPP                                       /* VE start from current control */
                         & ~VE_TASKAPP_CLEAN_MASK)
                         | VE_TASKAPP_CURRENT_CONTROL;
    TSB_VE0->ACTSCH   = (TSB_VE0->ACTSCH
                         & ~VE_ACTSCH_CLEAN_MASK)
                         | VE_ACTSCH_SCHEDULE_1;                                /* Schedule 1 */
#else
    if ( (MotorControl[channel_number].ActualStage.main == Stage_Initposition)
       &&(MotorControl[channel_number].ActualStage.sub  == Substage_Step0)
       &&(TSB_VE0->ACTSCH == VE_ACTSCH_SCHEDULE_9 ))
    {
      TSB_VE0->TASKAPP  = (TSB_VE0->TASKAPP                                     /* VE start from current control */
                           & ~VE_TASKAPP_CLEAN_MASK)
                           | VE_TASKAPP_CURRENT_CONTROL;
      TSB_VE0->ACTSCH   = (TSB_VE0->ACTSCH
                           & ~VE_ACTSCH_CLEAN_MASK)
                           | CallbackTable[VE_TASKAPP_CURRENT_CONTROL].NextSchedule;
    }
    else
    {
      TSB_VE0->TASKAPP  = (TSB_VE0->TASKAPP
                           & ~VE_TASKAPP_CLEAN_MASK)
                           | CallbackTable[TSB_VE0->SCHTASKRUN>>1].NextTask;

      TSB_VE0->ACTSCH   = (TSB_VE0->ACTSCH
                           & ~VE_ACTSCH_CLEAN_MASK)
                           | CallbackTable[TSB_VE0->SCHTASKRUN>>1].NextSchedule;
    }

    CallbackTable[TSB_VE0->SCHTASKRUN>>1].Function();

    if (TSB_VE0->TASKAPP != VE_TASKAPP_CURRENT_CONTROL)
    {
      TSB_VE0->CPURUNTRG	|= 1;                                           /* VE start */
      return;
    }

#endif /* USE_USER_CALLBACKS */
    break;
  default:
    assert_param(0);
    break;
  }

#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
#endif
#ifdef USE_HV_COMMUNICATION                                                     /* Galvanic isolation prevents direct measurement - use HV communication instead */
  if (channel_number==0)
    pVEx->VDC = HV_Communication_GetValue(0)<<5;                                /* ADC valus from Device 10 bit - max is 0x3ff - blown up to 15 bit */
#endif
#ifdef BOARD_VDC_CHANNEL_0_VALUE                                                /* If there is no measurement channel for VDC, we have to give a constant value */
  if (channel_number==0)
  {
    int8_t unit = MotorControl[channel_number].v_max.unit-BOARD_VDC_CHANNEL_0_UNIT;
    if (unit >= 0)                                                              /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      pVEx->VDC =(0xfff<<3)*BOARD_VDC_CHANNEL_0_VALUE/MotorControl[channel_number].v_max.value/pow10(unit);
    else                                                                        /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      pVEx->VDC =(0xfff<<3)*BOARD_VDC_CHANNEL_0_VALUE/MotorControl[channel_number].v_max.value*pow10(-unit);
  }
#endif
#include "pwr_undefine.h"

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
#endif /* BOARD_PWR_HEADER_FILE_1 */
#ifdef USE_HV_COMMUNICATION                                                     /* Galvanic isolation prevents direct measurement - use HV communication instead */
  if (channel_number==1)
    pVEx->VDC = HV_Communication_GetValue(0)<<5;                                /* ADC valus from Device 10 bit - max is 0x3ff - blown up to 15 bit */
#endif
#ifdef BOARD_VDC_CHANNEL_1_VALUE                                                /* If there is no measurement channel for VDC, we have to give a constant value */
  if (channel_number==1)
  {
    int8_t unit = MotorControl[channel_number].v_max.unit-BOARD_VDC_CHANNEL_1_UNIT;
    if (unit >= 0)                                                              /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      pVEx->VDC =(0xfff<<3)*BOARD_VDC_CHANNEL_1_VALUE/MotorControl[channel_number].v_max.value/pow10(unit);
    else                                                                        /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      pVEx->VDC =(0xfff<<3)*BOARD_VDC_CHANNEL_1_VALUE/MotorControl[channel_number].v_max.value*pow10(-unit);
  }
#endif
#include "pwr_undefine.h"

  if (ChannelValues[channel_number].MeasurementDirection==CURRENT_MEASUREMENT_INVERTED)
  {
    pVEx->ID = (unsigned long)(-1 * (signed long)pVEx->ID);                     /* Invert for 2 Sensor solution */
    pVEx->IQ = (unsigned long)(-1 * (signed long)pVEx->IQ);                     /* Invert for 2 Sensor solution */
  }

  MotorControl[channel_number].Vd.value  =pVEx->VD;                                            /* Read Vd from VE */
  MotorControl[channel_number].Id  = (int16_t)(pVEx->ID>>16);                             /* Read Id from VE */
  MotorControl[channel_number].Iq  = (int16_t)(pVEx->IQ>>16);                             /* Read Iq from VE */


  data1 =  (float)abs(MotorControl[channel_number].Iq_reference_I.part.reg);
  data1 *= MotorControl[channel_number].a_max.value;
  data1 /= FIXPOINT_15;
  data1 /= pow10(-MotorControl[channel_number].a_max.unit-3);


  data2 = (float)(pVEx->VQIH>>15) * (float)MotorControl[channel_number].v_max.value / FIXPOINT_15;

  VE_ActualMotorPower[channel_number] = (uint16_t) ( data1*data2/2);

#ifdef USE_SW_OVERCURRENT_DETECTION
  if (( MotorControl[channel_number].Id*MotorControl[channel_number].Id
       +MotorControl[channel_number].Iq*MotorControl[channel_number].Iq ) > MotorControl[channel_number].PreCalculation.SWOvercurrentSquare )
  {
    MotorControl[channel_number].ActualStage.main    = Stage_Emergency;
    MotorErrorField[channel_number].Error |= VE_SWOVERCURRENT;
  }
#endif /* USE_SW_OVERCURRENT_DETECTION */

  /*****************************************/
  /****** Current / Pos calculation ********/
  /*****************************************/
  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Stop:
  case Stage_Bootstrap:
  case Stage_Emergency:
  case Stage_ZeroCurrentMeasure:
  case Stage_Initposition:
  case Stage_Brake:
    break;
  case Stage_Force:
  case Stage_FOC:
  switch (MotorExtendedStateValues[channel_number].ControlMethod)
  {
    case MOTORCONTROL_CONTROL_BY_TORQUE:
      TorqueController_Omega_Theta(channel_number);
      TorqueController_Calculate_Id_Iq_ref(channel_number);
      break;

    case MOTORCONTROL_CONTROL_BY_SPEED:
      SpeedController_Omega_Theta(channel_number);
      SpeedController_Calculate_Id_Iq_ref(channel_number);
      break;
  }
    break;
  default:
    assert_param(0);
    break;
  }

  switch (MotorControl[channel_number].ActualStage.main)
  {
  /*****************************************/
  /************ Emergency Stage ************/
  /************** STOP Stage ***************/
  /*********** Bootstrap Stage *************/
  /************** Brake Stage **************/
  /*****************************************/
  case Stage_Emergency:
  case Stage_Stop:
  case Stage_Bootstrap:
  case Stage_Brake:
    pVEx->OMEGA= 0x0;
    pVEx->ID   = 0x0;
    pVEx->IQ   = 0x0;
    pVEx->VD   = 0x0;
    pVEx->VQ   = 0x0;
    pVEx->IDREF= 0x0;
    pVEx->IQREF= 0x0;
    pVEx->THETA= 0x0;
    MotorControl[channel_number].Omega_command.value  = 0;
    break;

  /*****************************************/
  /******** Measure Zero Current ***********/
  /*****************************************/
  case Stage_ZeroCurrentMeasure:
    MotorControl[channel_number].StageCounter++;
    if(MotorControl[channel_number].StageCounter >= 4)                                    /* check timeout - 2 times on 1/PWM_Frequency */
    {
      if (MotorControl[channel_number].Theta.value == 0)                                  /* first time of startup or nit shutdown gentle mode */
        MotorControl[channel_number].ActualStage.main = Stage_Initposition;               /* set next stage */
      else
        MotorControl[channel_number].ActualStage.main = Stage_Force;                      /* set next stage */

      MotorControl[channel_number].ActualStage.sub  = Substage_Step0;                     /* set next substage */
      MotorControl[channel_number].StageCounter     = 0;                                  /* stage counter to limit time */
    }
    break;

  /*****************************************/
  /************ Position Stage *************/
  /******* Forced Commutation Stage ********/
  /************* FOC Stage *****************/
  /*****************************************/
  case Stage_Force:
#ifdef USE_MOTOR_DISCONNECT_DETECTION
    meas_current = abs(pVEx->IAADC-pVEx->IAO) + abs(pVEx->IBADC-pVEx->IBO) + abs(pVEx->IBADC-pVEx->ICO);
    calc_current = (MotorParameterValues[channel_number].IdStart.value + MotorParameterValues[channel_number].IqStart.value) * MotorControl[channel_number].a_max.value / 0x7ff;

    if(  ( meas_current < calc_current / 5)
       &&( abs(MotorControl[channel_number].Omega_command.part.reg) >= 1000) )
       MotorControl[channel_number].disconnect_counter++;
    else
      MotorControl[channel_number].disconnect_counter=0;

    if (MotorControl[channel_number].disconnect_counter > 1000)
    {
      MotorErrorField[channel_number].Error |= VE_NOMOTOR;
      MotorControl[channel_number].ActualStage.main = Stage_Emergency;
    }
#endif /* USE_MOTOR_DISCONNECT_DETECTION */
  case Stage_Initposition:
  case Stage_FOC:
    pVEx->IDREF = MotorControl[channel_number].Id_reference;                              /* Set Id reference */
    pVEx->IQREF = MotorControl[channel_number].Iq_reference;                              /* Set Iq reference */
    pVEx->THETA = MotorControl[channel_number].Theta.part.reg;                            /* Theta set */
    pVEx->OMEGA = MotorControl[channel_number].Omega.part.reg;                            /* Omega set */
    break;

  /*****************************************/
  /***************** ERROR *****************/
  /*****************************************/
  default:
    assert_param(0);
    break;
  }

  /* Count electric turns - if motor deasn't have an encoder / hall sensors */
  /* Count sector changes - if motor deasn't have an encoder / hall sensors */
  if ((MotorParameterValues[channel_number].EncoderUsage == ENCODER_NO_ENCODER) )
    if (MotorControl[channel_number].Omega_Target != 0)
  {

    EncoderData[channel_number].sector_nr = (uint8_t)((((int32_t)MotorControl[channel_number].Theta.part.reg) & 0x0000ffff) * MAX_SECTOR / FIXPOINT_16 )  ;

    if (MotorControl[channel_number].Omega.value >= 0)
        /* Clockwise is "1" */
        EncoderData[channel_number].CW = E_ENC_ROTATE_CLOCKWISE;
    else
        EncoderData[channel_number].CW = E_ENC_ROTATE_COUNTERCLOCKWISE;

    uint32_t Theta0 = ((int32_t)EncoderData[channel_number].Theta0.part.reg & 0x0000ffff);
    uint32_t Theta  = ((int32_t)MotorControl[channel_number].Theta.part.reg & 0x0000ffff);

    /* Theta0 = 90o, set the trigger for CW at transition 359o -> 0o to consider the final 90o  */
    if (EncoderData[channel_number].CW == E_ENC_ROTATE_CLOCKWISE)
    {
      if ((MotorControl[channel_number].trigger_cw == 0) && ( Theta0 > Theta))
        MotorControl[channel_number].trigger_cw = 1;

      if ((MotorControl[channel_number].trigger_cw == 1) && (Theta0  <= Theta))
      {
          MotorControl[channel_number].trigger_cw = 0;
          //Increment Turns for CW
          EncoderData[channel_number].FullTurns++;
      }
    }
    /* Theta0 = -90o, set the trigger for CCW at transition 0o -> 359o to consider the final 90o  */
    else
    {
      if ((MotorControl[channel_number].trigger_ccw == 0) && ( Theta0 < Theta))
        MotorControl[channel_number].trigger_ccw = 1;

      if ((MotorControl[channel_number].trigger_ccw == 1) && (Theta0 >= Theta))
      {
          MotorControl[channel_number].trigger_ccw = 0; 
          //Decrement Turns for CCW
          EncoderData[channel_number].FullTurns--;
      }
    }
  }

#ifdef USE_EXTERNAL_SPEED_CONTROL
#ifdef SPEED_CONTROL_FG_PORT
  if (SystemValues[ESC_CHANNEL].ExternalSpeedControl!=EXTERNAL_SPEED_NONE)
  {
    static uint8_t      sector;
    static uint8_t      oldsector;

    sector = pVEx->SECTOR;                                                      /* Get actual sector */

    if (oldsector !=  sector)
    {
      HAL_GPIO_SetBit(SPEED_CONTROL_FG_PORT,
                      SPEED_CONTROL_FG_PIN,
                        (~HAL_GPIO_GetBit(SPEED_CONTROL_FG_PORT,
                                           SPEED_CONTROL_FG_PIN) & 0x01));

      oldsector=sector;                                                         /* remember old sector */
    }
  }
#endif /* SPEED_CONTROL_FG_PORT */
#endif /* USE_EXTERNAL_SPEED_CONTROL */

  if( MOTORCONTROL_CONTROL_BY_SPEED == MotorExtendedStateValues[channel_number].ControlMethod )
  {
      /* Used for actual speed statistics */    
      TurnStateValues[channel_number].CalculatedEncoderValue = MotorExtendedStateValues[channel_number].ActualControlledParameter;
  }
  else
  {
      /* Used for actual speed statistics */
      TurnStateValues[channel_number].CalculatedEncoderValue = MotorExtendedStateValues[channel_number].NonControlParameter;
  }

  /* if final angle is not yet set continue with the current position estimation */
  if (MotorControl[channel_number].stop_calculate == 0) {
  /* Calcualte the current number of lines based on the start value, current value and the line enocedr resolution */
  TurnStateValues[channel_number].ActualCalculatedEncoderValue = (EncoderData[channel_number].FullTurns - StartEncValue[channel_number].FullTurns ) * LINE_ENCODER_RESOLUTION;

  /* Get the current angle Theta */
  d_theta_diff1= (((int32_t)MotorControl[channel_number].Theta.part.reg) & 0x0000ffff);
  /* Obtain the start angle Theta0 */
  d_theta_diff2= (((int32_t)EncoderData[channel_number].Theta0.part.reg) & 0x0000ffff);
  /* Find out the difference, adapt to the current reference encoder resolution and nomralize the value to 360o */
  d_theta_diff = (d_theta_diff2 - d_theta_diff1 ) * LINE_ENCODER_RESOLUTION / FIXPOINT_16 ;

  /*
     We have start angle of 90o, thus normalization is needed upon 360->0transition, e.g.:
         Theta0 = 90o - start angle
         Theta = 10o - current angle
        DeltaTetha = (10-90) + 360 => -80 + 360 = 280o
  */
  if (d_theta_diff < 0)
    d_theta_diff += LINE_ENCODER_RESOLUTION;

  /* A glitch due to rounding failures in the diff calculation compensation - round to another turn */
  if ((d_theta_diff == 0) && (d_theta_diff1 < d_theta_diff2))
    d_theta_diff += LINE_ENCODER_RESOLUTION;

  TurnStateValues[channel_number].ActualCalculatedEncoderValue +=   d_theta_diff;
  }

#ifdef USE_ENCODER
  /* Hall sensor has 6 pulses per mechanical turn given one polepair. The number of pole pairs shall be compensated in the motor definition */
  TurnStateValues[channel_number].HallEncoderValue = ENC_GetEncCounter(0) * LINE_ENCODER_RESOLUTION / 6 ;
  /* Adapt to the line encoder connection and direction */
  TurnStateValues[channel_number].LineEncoderValue = ENC_GetEncCounter(2) * (LINE_ENCODER_DIR_ADAPTATION);
#endif /* USE_ENCODER */

  /* Start the VE */
  TSB_VE0->CPURUNTRG |= (1<<channel_number);
  
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32IRQLoopTimeStop[channel_number] = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32IRQLoopProcessingTime[channel_number] = u32IRQLoopTimeStop[channel_number] - u32IRQLoopTimeStart[channel_number];
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */       
}

#ifdef FIX_155_PMD_DELAY
void updatePmdRegisterSetting_Ch0()
{
    uint8_t iCnt;
    
    for(iCnt = 0; iCnt < PMD_OvervoltageReset_Force; iCnt++)
    {
        switch(gPmdRegisterSetting[0][iCnt])
        {
        case PMD_OutputOff_EmergencyDisable:
            HAL_PMD_OutputOff(0);
            HAL_PMD_EmergencyDisable(0);
            break;
            
        case PMD_NormalOutput_VE_PMD_Output:
            HAL_PMD_NormalOutput(0);  
            HAL_VE_PMD_Output(0,PMD_ON);
            break;
            
        case PMD_OutputPattern:
            HAL_PMD_OutputPattern(0,PMD_SHORTCUT_LOW_SIDE);
            break;
            
        case PMD_NormalOutput:
            HAL_PMD_NormalOutput(0);
            break;
            
        case PMD_EmergencyReset_Initposition:
		case PMD_EmergencyReset_Force:	
            HAL_PMD_EmergencyReset(0);
            break;
            
        case PMD_OvervoltageReset_Initposition:
		case PMD_OvervoltageReset_Force:
            HAL_PMD_OvervoltageReset(0);
            break;            
              
        default:
            break;                
        }
		gPmdRegisterSetting[0][iCnt] = 0x00;
    }
}
#endif

/*! \brief  IRQ Handler for Vector Engine Channel 0
  *
  * @param  None
  * @retval None
*/
void INTVCN0_IRQHandler(void)
{
  /* Fix input processing problem with single shunt */
  static uint32_t tmp_id,tmp_iq;
  static uint8_t  sector;
  
#ifdef FIX_155_PMD_DELAY
  if(ChannelValues[0].MeasurementType == CURRENT_SHUNT_3)
  {
    updatePmdRegisterSetting_Ch0();
  }
#endif

  if ((TSB_VE0->SECTOR!=sector) && (TSB_VE0->MCTLF & 0x1))
    switch (TSB_VE0->SECTOR)
    {
    case 1:
    case 3:
    case 5:
    case 7:
    case 9:
    case 11:
      TSB_VE0->ID = tmp_id;
      TSB_VE0->IQ = tmp_iq;
      break;
    default:
      break;
    }

  tmp_id = TSB_VE0->ID;
  tmp_iq = TSB_VE0->IQ;

  sector  = TSB_VE0->SECTOR;

#ifdef USE_DSO
  DSO_Log (VE_CALLING , TSB_VE0 );
#endif // USE_DSO
#ifdef USE_HSDSO
  HsDSO_Log_VE(0);
#endif // USE_HSDSO
#ifdef USE_STALL_DETECT
  STALL_Detect(0);
#endif // USE_STALL_DETECT
  if (MotorControl[0].ParameterChange!=0)
    MotorControl_HandleParameterChange(0);

  IRQ_Common(0);
}


