/** 
 *******************************************************************************
 * @file    hal_i2c.h
 * @brief   This file provides all the functions prototypes for I2C Driver.
 * @version V1.0.0.0
 * $Date:: 2017-10-30 17:22:07 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef HAL_I2C_H
#define HAL_I2C_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky_i2c.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UTILITIES
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_define
 *  @{
 */
/** 
 *  @defgroup I2C_NullPointer  Null Pointer
 *  @brief    I2C NULL Pointer.
 *  @{
 */
#define I2C_NULL        ((void *)0)
/**
 *  @}
 */ /* End of group I2C_NullPointer */

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_define
 *  @{
 */

 #define HAL_I2C_CH0                     (0)         /*!< I2C Channel 0.            */ 
 #define HAL_I2C_CH1                     (1)         /*!< I2C Channel 1.            */ 
 
 #define HAL_I2C_CH_COUNT                (2)         /*!< number of I2C Channels    */
  
/**
 * @defgroup I2C_ACK I2C ACK Macros
 * @brief    I2C Type of Acknowledge.
 * @{
 */
#define HAL_I2C_NACK                    (0)         /*!< NACK Received.             */
#define HAL_I2C_ACK                     (1)         /*!< ACK Received.              */
/**
 *  @}
 */ /* End of group I2C_ACK */


/**
 * @defgroup I2C_ERROR I2C ERROR Macros
 * @brief    I2C Error definitions.
 * @{
 */
#define HAL_I2C_SUCCESS                 (0)         /*!< No Error.                          */
#define HAL_I2C_ERROR_NO_SLAVE          (-1)        /*!< No Slave Error.                    */
#define HAL_I2C_ERROR_BUS_BUSY          (-2)        /*!< Bus Busy Error.(now, not support)  */
#define HAL_I2C_ERROR_PARAM             (-3)        /*!< Parameter Error.                   */
#define HAL_I2C_ERROR_OTHERS            (-4)        /*!< Others Error.                      */
#define HAL_I2C_ERROR_ARBITRATION       (-5)        /*!< Arbitration Error.                 */
/**
 *  @}
 */ /* End of group I2C_ERROR */

/**
 * @defgroup I2C_Events I2C Events Macros
 * @brief    I2C Asynch Events.
 * @{
 */
#define HAL_I2C_EVENT_ERROR               (1 << 1)  /*!< Error.                     */
#define HAL_I2C_EVENT_ERROR_NO_SLAVE      (1 << 2)  /*!< No Slave.                  */
#define HAL_I2C_EVENT_TRANSFER_COMPLETE   (1 << 3)  /*!< Transfer Complete.         */
#define HAL_I2C_EVENT_TRANSFER_EARLY_NACK (1 << 4)  /*!< End of Transfer.           */
#define HAL_I2C_EVENT_ALL                 (HAL_I2C_EVENT_ERROR |  HAL_I2C_EVENT_TRANSFER_COMPLETE | HAL_I2C_EVENT_ERROR_NO_SLAVE | HAL_I2C_EVENT_TRANSFER_EARLY_NACK)
/**
 *  @}
 */ /* End of group I2C_Events */

/**
 * @defgroup I2C_SlaveReceive I2C Slave Receive Return Macros
 * @brief    I2C Received Contents of Slave.
 * @{
 */
#define HAL_I2C_NO_DATA             (0)     /*!< the slave has not been addressed. */
#define HAL_I2C_READ_ADDRESSED      (1)     /*!< the master has requested a read from this slave. */
#define HAL_I2C_WRITE_GENERAL       (2)     /*!< the master is writing to all slave.(now, not support) */
#define HAL_I2C_WRITE_ADDRESSED     (3)     /*!< the master is writing to this slave. */
/**
 *  @}
 */ /* End of group I2C_SlaveReceive */

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  i2c Port Enumerated Type Definition.
*/
/*----------------------------------*/
typedef enum {
  HAL_I2C_OK = 0,
  HAL_I2C_ERROR,
  HAL_I2C_INVALID_CH_PORT_COMB,
  HAL_I2C_INVALID_CHANNEL,
  HAL_I2C_ERROR_NO_SLAVE1,
  HAL_I2C_ERROR_WRIITE,
  HAL_I2C_ERROR_READ,
  HAL_I2C_ERROR_READ_ZERO,
}HAL_I2C_Result;

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  I2C internal information structure definenition.
*/
/*----------------------------------*/
typedef struct
{
   uint8_t         bus_free;       /*!< Bus free information.          */
   uint8_t         start;          /*!< Start condition information.   */
   uint32_t        irqn;           /*!< IRQ number table pointer.      */
   struct
   {
       uint32_t    address;        /*!< Slave address.                 */
       uint32_t    stop;           /*!< Stop control                   */
       uint32_t    event;          /*!< I2C Event information.         */
       uint32_t    state;          /*!< Transfer State.                */
   } asynch;
} HAL_I2C_Internal_Info_t;

/*----------------------------------*/
/** 
 * @brief  I2C buffer structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t     *p_buffer;          /*!< Buffer address.      */
    uint32_t    length;             /*!< Buffer length.       */
    uint32_t    pos;                /*!< Buffer pointer.      */
} HAL_I2C_Buffer_t;

/*----------------------------------*/
/** 
 * @brief  I2C handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t                   channel;   /*!< I2C channel.   */
    I2C_t                     i2c;      /*!< I2C class structure.   */
    HAL_I2C_Internal_Info_t   info;     /*!< Internal Information.  */
    HAL_I2C_Buffer_t          tx_buff;  /*!< Tx buffer structure.   */
    HAL_I2C_Buffer_t          rx_buff;  /*!< Rx buffer structure.   */
} HAL_I2C_t;

/**
 *  @}
 */ /* End of group UTILITIES_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_functions
 *  @{
 */
HAL_I2C_Result HAL_I2C_Init(uint8_t channel, uint8_t port);
HAL_I2C_Result HAL_I2C_Frequency(uint8_t channel, int32_t hz);
void           HAL_I2C_Reset(uint8_t channel);
HAL_I2C_Result HAL_I2C_Check_Bus_Free(uint8_t channel);
HAL_I2C_Result HAL_I2C_Start(uint8_t channel);
HAL_I2C_Result HAL_I2C_Stop(uint8_t channel);
int32_t        HAL_I2C_Read(uint8_t channel, int32_t address, uint8_t *p_data, int32_t length, int32_t stop);
int32_t        HAL_I2C_Write(uint8_t channel, int32_t address, uint8_t *p_data, int32_t length, int32_t stop);
int32_t        HAL_I2C_ByteRead(uint8_t channel, int32_t last);
int32_t        HAL_I2C_ByteWrite(uint8_t channel, int32_t data);
uint8_t        HAL_I2C_Active(uint8_t channel);
HAL_I2C_Result HAL_I2C_TransferAsynch(uint8_t channel, uint8_t *p_tx, int32_t tx_length, uint8_t *p_rx, int32_t rx_length, int32_t address, int32_t stop);
uint32_t       HAL_I2C_Irq_HandlerAsynch(uint8_t channel);
void           HAL_I2C_AbortAsynch(uint8_t channel);


HAL_I2C_Result HAL_I2C_MasterBlockingWriteRead(uint8_t channel, uint8_t *master_tx_data, int32_t tx_length, uint8_t *master_rx_data, int32_t rx_length, int32_t slave_address);
HAL_I2C_Result HAL_I2C_MasterBlockingWrite(uint8_t channel, uint8_t *master_tx_data, int32_t tx_length, int32_t slave_address);


/* For slave */
void           HAL_I2C_SlaveMode(uint8_t channel, int32_t enable_slave);
int32_t        HAL_I2C_SlaveReceive(uint8_t channel);
int32_t        HAL_I2C_SlaveRead(uint8_t channel, uint8_t *p_data, int32_t length);
int32_t        HAL_I2C_SlaveWrite(uint8_t channel, uint8_t *p_data, int32_t length);
void           HAL_I2C_SlaveAddress(uint8_t channel, uint32_t address);
HAL_I2C_Result HAL_I2C_SlaveTransfer_Asynch(uint8_t channel, uint8_t *p_tx, int32_t tx_length, uint8_t *p_rx, int32_t rx_length);
uint32_t       HAL_I2C_SlaveIrqHandlerAsynch(uint8_t channel);
void           HAL_I2C_SlaveAbortAsynch(uint8_t channel);

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_I2C_H */
