/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */
#include <stdint.h>
#include "motor_control.h"
#include "fwmathlib.h"

/******************************************************************************/
/******************************************************************************/
/************************* Position Estimators ********************************/
/******************************************************************************/
/******************************************************************************/

/*! \brief  Calculate Omega BLDC
  *
  * Position (Omega) Calculator for BLDC motors
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void Estimator_Calculate_Omega_BLDC(uint8_t channel_number)
{
  int64_t temp1;                                                                /*< intermediate (temp) values for better debugging */
  int64_t temp2;
  int64_t temp3;
  int64_t temp4;
  int64_t Ed_PI;                                                                /*< intermediate calculation of position */
  int64_t Ed_I;                                                                 /*< intermediate calculation of measured current over R */
  int64_t Ed;                                                                   /*< intermediate calculation of d-axis induced voltage */

  int32_t Omega_Calculated;                                                     /*< calculated Omega */


  /* equations :\n
   * -# Vd = R*Id - OMEGAest * Iq + Ed  ->\n
   * -# Ed = Vd - R * Id + OMEGAest * Lq * Iq */

  /* Calculate the Dropout of Motor resistance\n
   *   R_Ed = R_Vd - Motor_R * R_Id + R_omega * Motor_Lq * R_Iq \ n
   * = temp1 * V_MAX = FIXPOINT_15 * Motor_R * R_Id * A_MAX */
  temp1 = MotorControl[channel_number].PreCalculation.R_mult_amax_div_vmax;
  temp1 = ((int32_t)MotorControl[channel_number].Id * temp1 )<<1;

  /* Calculate the Dropout of inductance\n
   * temp2 * V_MAX = FIXPOINT_15 * 2Pi * R_omega * Hz_MAX * Motor_Lq / (1000 * R_Iq * A_MAX) */
  temp2 = MotorControl[channel_number].PreCalculation.InductanceDropoutFactor;
  temp2 = (((MotorControl[channel_number].Omega.part.reg * temp2)<<1) + ROUND_BIT15)>>16;
  temp2 = (temp2 * MotorControl[channel_number].Iq)<<1;

  /* calculate the d-axis induced voltage */
  Ed = ((MotorControl[channel_number].Vd.value - temp1 + temp2 + ROUND_BIT15)>>16);

  /* Calculate the estimated position
   * OMEGAest = OMWEGAref - K * ed\n
   * OmegaErr(I) * Hz_MAX = FIXPOINT_15 * PositionKi * CtrlPrd * R_Ed * V_MAX */
  temp3  = (int64_t)2;
  temp3 *= MotorControl[channel_number].PreCalculation.PositionKiPreCalculation;
  temp3 *= Ed;

  /* the calculation was unsigned: correct Saturation */
  if(MotorControl[channel_number].Omega_command.part.reg < 0)
    temp3 *= (-1);

  Ed_I = - temp3;                                                               /* Saturation subtraction */

  /* Calculate the speed from proportional part:
   * Speed calculation value = Speed command value + Error I value
   * OmegaErr(P) * Hz_MAX = FIXPOINT_15 * PositionKp * R_Ed * V_MAX */
  temp4 = MotorControl[channel_number].PreCalculation.PositionKpPreCalculation;
  temp4 = ( temp4 * (int32_t)Ed)<<1;

  /* the calculation was unsigned: */
  if(MotorControl[channel_number].Omega_command.part.reg < 0)
    temp4 *= (-1);
  Ed_PI = Ed_I - temp4;                                                         /* error PI value */

  /* Calculate the speed from integrative part:\n
   * Speed calculation value = Speed command value + Error PI value */
  Omega_Calculated = MotorControl[channel_number].Omega_command.value + Ed_PI;

  /* set new omega from calculation */
  MotorControl[channel_number].Omega.value      = Omega_Calculated;
  MotorControl[channel_number].OmegaCalc        = Omega_Calculated>>16;                   /* for DSO log */
}

