/* THE SOURCE CODE AND ITS
* RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
* WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
* AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
* INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
* DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
* INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
* MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
* TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
* LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
* FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
* LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
* OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
* ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
* ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
* FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
* OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
*/

#include <stdint.h>
#include <cmsis_iccarm.h>
#include "fwmathlib.h"

uint32_t pow10(uint8_t power)
{
  uint32_t value=1;																/* result of calculation */
  int i;																		/* counter */
  
  for (i=0;i<power;i++)
    value *=10;
  
  return value;
}

/**
*********************************************************************************************
* @brief		Cal Sine
*			sin(x) = 3.140625x + 0.02026367x^2 -5.325196x^3 + 0.5446778x^4 + 1.800293x^5
*
* @param		int16_t	theta	theta
*
* @return		int16_t	sin(theta)
*********************************************************************************************
*/
int16_t Sine(int16_t theta)
{
  int16_t x;
  int16_t x1;
  long temp32;
  
  x = (int16_t)(theta & 0x7fff);
  if (x >= ELE_DEG(90))
  {
    x = (int16_t)(ELE_DEG(180) - x);
  }
  x1 = x;
  temp32 = (cSine1 * (long)x1) << 1;
  x1 = (int16_t)(x1 * (long)x >> (16 - 1));
  temp32 += (cSine2 * (long)x1) << 1;
  x1 = (int16_t)(x1 * (long)x >> (16 - 1));
  temp32 += (cSine3 * (long)x1) << 1;
  x1 = (int16_t)(x1 * (long)x >> (16 - 1));
  temp32 += (cSine4 * (long)x1) << 1;
  x1 = (int16_t)(x1 * (long)x >> (16 - 1));
  temp32 += (cSine5 * (long)x1) << 1;
  temp32 <<= 3;
  
  if ((unsigned long)temp32 >= 0x7fffffff)
  {
    temp32 = 0x7fffffff;
  }
  if (theta <= 0)
  {
    temp32 *= ( -1);
  }
  
  return ((int16_t)(temp32 >> 16));
}


/**
*********************************************************************************************
* @brief		Cosine
*
* @param		int16_t	theta
*
* @return		int16_t
*********************************************************************************************
*/
int16_t Cosine(int16_t theta)
{
  return (Sine((int16_t)(ELE_DEG(90) + theta)));
}

/**
*********************************************************************************************
* @brief		Clarke
*
* @param		int16_t 		_iu			U-phase current
* @param		int16_t 		_iv			V-phase current
* @param		int16_t 		_iw			W-phase current
* @param		int16_t* 		_ialpha		alpha-axis current storage address
* @param		int16_t* 		_ibeta		beta-axis current storage address
*
* @return		none	
*********************************************************************************************
*/
void ClarkeTransformation(int16_t _iu, int16_t _iv, int16_t _iw, int16_t* _ialpha, int16_t* _ibeta)
{
  
  int32_t temp_1;
  
  /* Ialpha=Iu */
  *_ialpha = _iu;
  /* Ibeta=(Iu + 2*Iv)/sqrt3 */
  temp_1 = ((int32_t)_iu  + (int32_t)(_iv * 2));
  *_ibeta = (temp_1 * (int32_t)((1L << 15) * (1.0f / 1.7320508f))) >> 15;
}

/**
*********************************************************************************************
* @brief		Park
*
* @param		int16_t 		_ialpha		alpha-axis 
* @param		int16_t		_ibeta		bata-axis
* @param		uint16_t	theta		Theta(position)
* @param		int16_t* 		_id			d-axis storage address
* @param		int16_t* 		_iq			q-axis storage address
*
* @return		none	
*********************************************************************************************
*/
void ParkTransformation(int16_t _ialpha, int16_t _ibeta, uint16_t _theta, int16_t* _id, int16_t* _iq)
{
  int32_t temp_1, temp_2, temp_3, temp_4;
  int16_t temp_sin, temp_cos;
  
  /*alpha_beta/dq conversion (Park.t) */
  temp_cos = Cosine(_theta);
  temp_sin = Sine(_theta);
  
  /* Id = Ialpha * cos(theta)+Ibeta * sin(theta) */
  temp_1 = ((int32_t)_ialpha * temp_cos) << 1;
  temp_2 = ((int32_t)_ibeta  * temp_sin) << 1;
  *_id = (int16_t)(__QADD(temp_1, temp_2) >> 16);
  
  /* Iq = Ialpha * -sin(theta) + Ibeta * cos(theta) */
  temp_3 = ((int32_t)_ialpha * (int32_t)(-temp_sin)) << 1;
  temp_4 = ((int32_t)_ibeta  * (int32_t)temp_cos) << 1;
  *_iq = (int16_t)(__QADD(temp_3, temp_4) >> 16);
}

/**
*********************************************************************************************
* @brief		Limitation.
*
* @param		int64_t			_indata			Q31 input data(Double precision).
* @param		int32_t			_limlo			Q31 lower limit value.
* @param		int32_t			_limup			Q31 upper limit value.
*
* @return		int32_t			Q31
*********************************************************************************************
*/
static int32_t Limit_sub(int64_t _indata, int32_t _limlo, int32_t _limup)
{
  int32_t ret = (int32_t)_indata;
  
  if (_indata > (int64_t)_limup)
  {
    ret = _limup;
  }
  if (_indata < (int64_t)_limlo)
  {
    ret = _limlo;
  }
  return (ret);
}

/**
*********************************************************************************************
* @brief		D_SVM
*
* @param		int32_t 	_valpha		alpha-axis Valpha
* @param		int32_t 	_vbeta		beta-axis Vbeta
* @param		int16_t	 	_vdc		DC voltage
* @param		uint8_t 	_sector		sector
* @param		uint8_t 	_modul		modul
* @param		uint16_t* 	_p_vu		U phase duty ratio storage address
* @param		uint16_t* 	_p_vv		V phase duty ratio storage address
* @param		uint16_t* 	_p_vw		W phase duty ratio storage address
*
* @return		none	
*********************************************************************************************
*/

  int32_t temp_1, temp_2;
  int32_t d0, d1, d2;

void SpaceVectorModulation(int32_t _valpha, int32_t _vbeta, int16_t _vdc, uint8_t _sector, uint8_t _modul, uint16_t* _p_vu, uint16_t* _p_vv, uint16_t* _p_vw)
{
#define cSVM_K				(int64_t)((1L<<15) * 3.0f / 2.0f)			/* Q15 3/2 */
#define	cSVM_1_DIV_SQRT3	(int64_t)((1L<<15) * 1.0f / 1.7320508f)		/* Q15 1/sqrt3 */
#define	cSVM_2_DIV_SQRT3	(int64_t)((1L<<15) * 2.0f / 1.7320508f)		/* Q15 2/sqrt3 */
  
#define cSVM_100_PERCENT	(int32_t)(((1LL<<31) * 1) - 1) /* Q31 1.0 */
  
 
  switch (_sector)
  {
  case 0:
    /* Sector0 */
    /* D1 = k * (Valpha - 1/sqrt3 * Vbeta) / Vdc */
    /* D2 = k * ( 2/sqrt3 * Vbeta) / Vdc */
    /* D0 = 100% - D1 - D2 */
    temp_1 = (int32_t)(__QSUB(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15)));
    temp_2 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
    d1 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
    d2 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
    d0 = (int32_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
    /* Set Duty */
    if (_modul == 0)
    {
      /* 3phase modulation */
      /* U = D1 + D2 + (D0 / 2) */
      /* V = D2 + (D0 / 2) */
      /* W = D0 / 2 */
      *_p_vu = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
      *_p_vv = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
      *_p_vw = (uint16_t)((d0 / 2) >> 16);
    }
    else
    {
      /* 2phase modulation */
      /* U = D1 + D2 */
      /* V = D2 */
      /* W = 0 */
      *_p_vu = (uint16_t)(__QADD(d1, d2) >> 16);
      *_p_vv = (uint16_t)(d2 >> 16);
      *_p_vw = 0;
    }
    break;
    
  case 1:
    /* Sector1 */
    /* D1 = k * (-Valpha + 1/sqrt3 * Vbeta) / Vdc */
    /* D2 = k * (Valpha + 1/sqrt3 * Vbeta) / Vdc */
    /* D0 = 100% - D1 - D2 */
    temp_1 = (int32_t)__QADD((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    temp_2 = (int32_t)__QADD(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    d1 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
    d2 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
    d0 = (int32_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
    /* Set Duty */
    if (_modul == 0)
    {
      /* 3phase modulation */
      /* U = D2 + (D0 / 2) */
      /* V = D1 + D2 + (D0 / 2) */
      /* W = D0 / 2 */
      *_p_vu = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
      *_p_vv = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
      *_p_vw = (uint16_t)((d0 / 2) >> 16);
    }
    else
    {
      /* 2phase modulation */
      /* U = D2 */
      /* V = D1 + D2 */
      /* W = 0 */
      *_p_vu = (uint16_t)(d2 >> 16);
      *_p_vv = (uint16_t)(__QADD(d1, d2) >> 16);
      *_p_vw = 0;
    }
    break;
    
  case 2:
    /* Sector2 */
    /* D1 = k * ( 2/sqrt3 * Vbeta) / Vdc */
    /* D2 = k * (-Valpha - 1/sqrt3 * Vbeta) / Vdc */
    /* D0 = 100% - D1 - D2 */
    temp_1 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
    temp_2 = (int32_t)__QSUB((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    d1 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
    d2 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
    d0 = (int32_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
    /* Set Duty */
    if (_modul == 0)
    {
      /* 3phase modulation */
      /* U = D0 / 2 */
      /* V = D1 + D2 + (D0 / 2) */
      /* W = D2 + (D0 / 2) */
      *_p_vu = (uint16_t)((d0 / 2) >> 16);
      *_p_vv = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
      *_p_vw = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
    }
    else
    {
      /* 2phase modulation */
      /* U = 0 */
      /* V = D1 + D2 */
      /* W = D2 */
      *_p_vu = 0;
      *_p_vv = (uint16_t)(__QADD(d1, d2) >> 16);
      *_p_vw = (uint16_t)(d2 >> 16);
    }
    break;
    
  case 3:
    /* Sector3 */
    /* D1 = k * (-2/sqrt3 * Vbeta) / Vdc */
    /* D2 = k * (-Valpha + 1/sqrt3 * Vbeta) / Vdc */
    /* D0 = 100% - D1 - D2 */
    temp_1 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
    temp_2 = (int32_t)__QADD((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    d1 = (int32_t)Limit_sub((int64_t)(cSVM_K * (-temp_1 /_vdc)), 0, cSVM_100_PERCENT);
    d2 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
    d0 = (int32_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
    /* Set Duty */
    if (_modul == 0)
    {
      /* 3phase modulation */
      /* U = D0 / 2*/
      /* V = D2 + (D0 / 2) */
      /* W = D1 + D2 + (D0 / 2) */
      *_p_vu = (uint16_t)((d0 / 2) >> 16);
      *_p_vv = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
      *_p_vw = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
    }
    else
    {
      /* 2phase modulation */
      /* U = 0 */
      /* V = D2 */
      /* W = D1 + D2 */
      *_p_vu = 0;
      *_p_vv = (uint16_t)(d2 >> 16);
      *_p_vw = (uint16_t)(__QADD(d1, d2) >> 16);
    }
    break;
    
  case 4:
    /* Sector4 */
    /* D1 = k * (-Valpha - 1/sqrt3 * Vbeta) / Vdc */
    /* D2 = k * (Valpha - 1/sqrt3 * Vbeta) / Vdc */
    /* D0 = 100% - D1 - D2 */
    temp_1 = (int32_t)__QSUB((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    temp_2 = (int32_t)__QSUB(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    d1 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
    d2 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
    d0 = (int32_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
    /* Set Duty */
    if (_modul == 0)
    {
      /* 3phase modulation */
      /* U = D2 + (D0 / 2) */
      /* V = D0 / 2 */
      /* W = D1 + D2 + (D0 / 2) */
      *_p_vu = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
      *_p_vv = (uint16_t)((d0 / 2) >> 16);
      *_p_vw = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
    }
    else
    {
      /* 2phase modulation */
      /* U = D2 */
      /* V = 0 */
      /* W = D1 + D2 */
      *_p_vu = (uint16_t)(d2 >> 16);
      *_p_vv = 0;
      *_p_vw = (uint16_t)(__QADD(d1, d2) >> 16);
    }
    break;
    
  case 5:
    /* Sector5 */
    /* D1 = k * (Valpha + 1/sqrt3 * Vbeta) / Vdc */
    /* D2 = k * ( -2/sqrt3 Vbeta) / Vdc */
    /* D0 = 100% - D1 - D2 */
    temp_1 = (int32_t)__QADD(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
    temp_2 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
    d1 = (int32_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
    d2 = (int32_t)Limit_sub((int64_t)(cSVM_K * (-temp_2 / _vdc)), 0, cSVM_100_PERCENT);
    d0 = (int32_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
    /* Set Duty */
    if (_modul == 0)
    {
      /* 3phase modulation */
      /* U = D1 + D2 + (D0 / 2) */
      /* V = D0 / 2 */
      /* W = D2 + (D0 / 2) */
      *_p_vu = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
      *_p_vv = (uint16_t)((d0 / 2) >> 16);
      *_p_vw = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
    }
    else
    {
      /* 2phase modulation */
      /* U = D1 + D2 */
      /* V = 0 */
      /* W = D2 */
      *_p_vu = (uint16_t)(__QADD(d1, d2) >> 16);
      *_p_vv = 0;
      *_p_vw = (uint16_t)(d2 >> 16);
    }
    break;
  default:
    break;
  }
}


/**
*********************************************************************************************
* @brief		InvPark
*
* @param		int32_t 		_d			d-axis
* @param		int32_t 		_q			q-axis
* @param		uint16_t	        theta		Theta(position)
* @param		int32_t* 		_alpha		alpha-axis storage address
* @param		int32_t* 		_beta		bata-axis storage address
*
* @return		none	
*********************************************************************************************
*/
void InvParkTransformation(int32_t _d, int32_t _q, uint16_t _theta, int32_t* _alpha, int32_t* _beta)
{
  int32_t temp_1, temp_2, temp_3, temp_4;
  int16_t temp_sin, temp_cos;
  
  temp_cos = Cosine(_theta);
  temp_sin = Sine(_theta);
  
  /* Valpha = Vd * cos(theta) - Vq * sin(theta) */
  temp_1 = (int32_t)(((int64_t)(_d) * (temp_cos)) >> 15);
  temp_2 = (int32_t)(((int64_t)(_q) * (temp_sin)) >> 15);
  *_alpha = __QSUB(temp_1, temp_2);
  
  /* Vbeta = Vd * sin(theta) + Vq * cos(theta) */
  temp_3 = (int32_t)(((int64_t)(_d) * (temp_sin)) >> 15);
  temp_4 = (int32_t)(((int64_t)(_q) * (temp_cos)) >> 15);
  *_beta = __QADD(temp_3, temp_4);
}

/**
 *********************************************************************************************
 * @brief		Get the medium value of three data.
 *
 * @param		int32_t _a			Data a.
 * @param		int32_t _b			Data b.
 * @param		int32_t _c			Data c.
 *
 * @return		Medium value
 *********************************************************************************************
 */
int32_t MediumOf3(int32_t _a, int32_t _b, int32_t _c)
{
	int32_t	ret;
	
	if (_a >= _b)
	{
		ret = (_c < _b) ? _b : (_c > _a) ? _a : _c;
	}
	else
	{
		ret = (_c < _a) ? _a : (_c > _b) ? _b : _c;
	}
	return (ret);
}

/**
 *********************************************************************************************
 * @brief		Get the minimum value of three data.
 *
 * @param		int32_t _a			Data a.
 * @param		int32_t _b			Data b.
 * @param		int32_t _c			Data c.
 *
 * @return		Minimum value
 *********************************************************************************************
 */
int32_t MinimumOf3(int32_t _a, int32_t _b, int32_t _c)
{
	int32_t	ret;

	if (_a >= _b)
	{
		ret = (_c < _b) ? _c : _b;
	}
	else
	{
		ret = (_c < _a) ? _c : _a;
	}
	return (ret);
}

/**
 *********************************************************************************************
 * @brief		Get the maximum value of three data.
 *
 * @param		int32_t _a			Data a.
 * @param		int32_t _b			Data b.
 * @param		int32_t _c			Data c.
 *
 * @return		Max value
 *********************************************************************************************
 */
int32_t MaximumOf3(int32_t _a, int32_t _b, int32_t _c)
{
	int32_t	ret;

	if (_a >= _b)
	{
		ret = (_a < _c) ? _c : _a;
	}
	else
	{
		ret = (_b < _c) ? _c : _b;
	}
	return (ret);
}


/**
 ********************************************************************************************
 * @brief		SortOf3.
 *
 * @param[in,out]	int32_t*	_n1	.
 * @param[in,out]	int32_t*	_n2	.
 * @param[in,out]	int32_t*	_n3	.
 *
 * @return		none
 ********************************************************************************************
 */
void SortOf3(int32_t* _n1, int32_t* _n2, int32_t* _n3)
{
	int32_t	tmp;

	if (*_n1 > *_n2)
	{
		tmp = *_n1;
		*_n1 = *_n2;
		*_n2 = tmp;
	}
	if (*_n2 > *_n3)
	{
		tmp = *_n2;
		*_n2 = *_n3;
		*_n3 = tmp;
	}
	
	if (*_n1 > *_n2)
    {
		tmp = *_n1;
		*_n1 = *_n2;
		*_n2 = tmp;
	}
}

