/* THE SOURCE CODE AND ITS
* RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
* WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
* AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
* INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
* DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
* INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
* MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
* TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
* LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
* FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
* LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
* OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
* ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
* ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
* FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
* OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2012 All rights reserved
*/

#include <stdint.h>
#include <stdlib.h>

#include "config.h"
#ifdef USE_MOTION_CONTROL

#include "FreeRTOS.h"
#include "task.h"
#include "board.h"
#include "hal_ve.h"
#include "motor_control.h"
#include "hal_encoder.h"
#include TMPM_HEADER_FILE
#include "fwmathlib.h"

#include "motion_control.h"

/* RPM for the final approach */
#define LOW_SPEED_BORDER                    2

#if defined __TMPM_4KN__
#include "txz_driver_def.h"
#endif

static int32_t MaxSpeed[MAX_CHANNEL];                                                  /* The maximum speed at which the motion control shall be done - one for each channel */
static int32_t ForcedSpeed[MAX_CHANNEL];                                               /* The reduced speed prior to final approach */
static int32_t WaitValueForForced[MAX_CHANNEL];                                        /* Number of turns with full speed */
static int32_t WaitValueForShutdown[MAX_CHANNEL];                                      /* Number of turns for speed reduction */
static int32_t TargetValue[MAX_CHANNEL];                                        /* Where do we want to go */
static int32_t WaitValueForTarget[MAX_CHANNEL];                                        /* Start Position of the actual turn */

static int16_t  FinalApproachRpm[MAX_CHANNEL];                                         /* Speed to position */


/*! \brief  Absolute position parameters initialization
*
* Set the absolute postion and limits
*
* @param  channel_number: channel number
* @retval None
*/
static void InitializeAbsolutePosition(long channel_number)
{
    MotorControl[channel_number].CurrentAbsPos = MotorParameterValues[channel_number].EncoderStartPos;
}

/*! \brief  Absolute position actualization
*
* Align the curent absolute position with the encoder readings
*
* @param  channel_number: channel number
* @retval None
*/
static void UpdateAbsolutePosition(long channel_number)
{
    uint32_t u32EncoderCnt = 0;
    u32EncoderCnt = ENC_GetEncCounter(channel_number);

    /* Encoder counter "0" is mapped to start_position. Thus any positive increment that's below the max is simple the counter */
    if( u32EncoderCnt <= MotorParameterValues[channel_number].EncoderMaxPos )
    {
        /* 0 - Max is always a CW increment */
        MotorControl[channel_number].CurrentAbsPos = MotorParameterValues[channel_number].EncoderStartPos + u32EncoderCnt;
    }
    else
    {
        /* Below 0, e.g. 0xFFFFFFFD means we have done several steps in the CCW. Make it "negative" and substract */
        MotorControl[channel_number].CurrentAbsPos = MotorParameterValues[channel_number].EncoderStartPos - (0xFFFFFFFFUL - u32EncoderCnt + 1);
    }
}

/*! \brief  Set a new zero postion
*
* Sets the curent absolute position as new zero position and clears the encoder value
*
* @param  channel_number: channel number
* @retval None
*/
void SetAbsoluteZeroPositionForMotionControl(long channel_number)
{
    /* Clear encoder value */
    ENC_ClearEncCounter (channel_number);
    /* Set the current position as new absolute zero position */
    MotorControl[channel_number].CurrentAbsPos = 0;
}

/*
        +------------------------------------------------------------+
        |                                                            |
        |                       curAbsPos                            |
        |    0xffff_ffff  | 0      /                                 | d = newAbsPos - curAbsPos
        |             #######    /----+ d = newAbsPos - curAbsPos    | if(d >= 0x8000_0000)
        |         ######   ######      | if(d < 0x8000_0000)         |   0x1_0000_0000-d is used
        |         ##           ##     V  d is used as-is             |   this is -d in 32-bit
        |       ####           ####----- newAbsPos                   |
        |       ##               ##                                  |
        |       ##               ##                                  V
        |       ####           ####------------------------------------ newAbsPos
        |         ##           ##      ^
        |         ######   ######      | this is -d
        |             #######    \-----+
        |    0x8000_0000  |      /\
        |                       / curAbsPos
        |                      /
        +---------------------+

<examples>

|      | curAbsPos | newAbsPos |          d | d         | u         |     u |
| type | uint32    | uint32    |     uint32 | uint32    | int32     | int32 |
| base |           |           |        dec | hex       | hex       |   dec |
|------|-----------|-----------|------------|-----------|-----------|-------|
|      | 10        | 30        |         20 | 0000_0014 | 0000_0014 |    20 |
|      | -10       | 20        |         30 | 0000_001E | 0000_001E |    30 |
|      | 30        | 10        | 4294967276 | FFFF_FFEC | FFFF_FFEC |   -20 |
|      | 7FFF_FFFF | 8000_0001 |          2 | 0000_0002 | 0000_0002 |     2 |
|      | 8000_0001 | 7FFF_FFFF | 4294967294 | FFFF_FFFE | FFFF_FFFE |    -2 |
*/
#pragma inline=forced
int32_t getPulseDistance(uint32_t curAbsPos, uint32_t newAbsPos)
{
  uint32_t d;
  d = newAbsPos - curAbsPos;
  // If the reslt exceeds the half of uint32, treat it as minus.
  if(d >= 0x80000000)
    { return((int32_t)d); }
  return((int32_t)d);
}

#pragma inline=forced
uint32_t isLessThan(uint32_t a, uint32_t b)
{
  int32_t u;
  u = getPulseDistance(a, b);
  if(u > 0)
    { return(1); }
  return(0);
}
#pragma inline=never

/*         Omega
*              |
*              |                                      -> WaitForForced
*  MaxSpeed   _|        ______________________________|
*              |       /                              \
*              |      /                                \
*              |     /                                  \
*              |    /                                    \
*              |   /                                      \   -> WaitForShutdown
*              |  /                                        \__|
*              | /                                             \
*              |x_____________________________________________  --x______________________ T
*                                                             |    |
*                                                              <--> Final approach
*/

/*! \brief  Calculate Turn Ramp
*
* Calculate the different values for the Motion Control Ramp
*
* @param  channel_number: channel number to use
* @param  new_abs_position_in_pulses: new absolute position in terms of encoder pulses
* @param  max_speed: maximum rpm to reach while turning
* @retval None
*/
void CalculatePulsesRampAndTurn(uint8_t channel_number, uint32_t new_abs_position_in_pulses, uint16_t max_speed)
{
    int32_t nr_pulses = 0;
    int16_t nr_turns = 0;

    uint16_t SpeedUpTime = 0;
    uint16_t PulsesWhileSpeedUp = 0;
    uint16_t SpeedDownTime = 0;
    uint16_t PulsesWhileSpeedDown = 0;
    rot_direction_e eRotDir = E_MC_ROTATE_CLOCKWISE;

    /* Ensure the current position matches the actual rotor position - manual motor rotation, improper stopping in the previous run, etc. */
    UpdateAbsolutePosition(channel_number);

    /* If not in the desired position calculate the needed action */
    if( new_abs_position_in_pulses != MotorControl[channel_number].CurrentAbsPos )
    {
        /* Get the relative number of encoder pulses and the direction */        
        if( isLessThan( MotorControl[channel_number].CurrentAbsPos, new_abs_position_in_pulses ) )
        {
            /* The new position shall not exceed the maximum allowed/supported. If so set to the limit */            
            if( isLessThan(MotorParameterValues[channel_number].EncoderMaxPos, new_abs_position_in_pulses ) )
            {
                new_abs_position_in_pulses = MotorParameterValues[channel_number].EncoderMaxPos;
            }
            /* Calcualte the relative number of pulses to be done */
            nr_pulses = getPulseDistance( MotorControl[channel_number].CurrentAbsPos, new_abs_position_in_pulses );
        }
        else
        {
            /* The new position shall not exceed the minimum allowed/supported. If so set to the limit */            
            if( isLessThan( new_abs_position_in_pulses, MotorParameterValues[channel_number].EncoderMinPos ) )
            {
                new_abs_position_in_pulses = MotorParameterValues[channel_number].EncoderMinPos;
            }
            /* Calcualte the relative number of pulses to be done */
            nr_pulses = getPulseDistance( new_abs_position_in_pulses, MotorControl[channel_number].CurrentAbsPos );
            /* Reverse the direction */
            eRotDir = E_MC_ROTATE_COUNTERCLOCKWISE;
        }
        nr_turns = (nr_pulses / MotorParameterValues[channel_number].EncoderResolution);

        if (nr_pulses % MotorParameterValues[channel_number].EncoderResolution)
        {
            /* Calcualte the number of full turns to perform including the direction (+1 to eliminate roundings errors) */
            nr_turns += 1;
        }

        /* Set target value of pulse position */
        TargetValue[channel_number] = new_abs_position_in_pulses;

recalculate_linear:
      /* Remember the requested Maximum RPM */
      MaxSpeed[channel_number] = max_speed;

      /* Calculate the border of Forced to Field-Oriented-Commutation */
      switch( MotorParameterValues[channel_number].SpeedChange.unit )
      {
          case HZ:
              ForcedSpeed[channel_number] = 60 * MotorParameterValues[channel_number].SpeedChange.value / MotorParameterValues[channel_number].PolePairs;
              break;

          case RPM:
              ForcedSpeed[channel_number] =  MotorParameterValues[channel_number].SpeedChange.value;
              break;

          default:
              assert_param(0);
              break;
      }
      /* Addd security margin - 80% */
      ForcedSpeed[channel_number] *= 8;
      ForcedSpeed[channel_number] /= 10;

      /* Enusre the set forced speed does not exceed the maximum speed requested for the motion */
      if( ForcedSpeed[channel_number] >= max_speed )
      {
          ForcedSpeed[channel_number] = max_speed;
      }

      /* Calcualte times to reach the target speed and stop afterwards */
      switch (MotorParameterValues[channel_number].MaxAngAcc.unit)
      {
          case RAD_BY_S2:
              /* Calculate the time needed to reach target rpm in ms */
              SpeedUpTime = (uint16_t)  ( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * max_speed
                                           * MotorParameterValues[channel_number].PolePairs
                                             * PAI2_VALUE                           /* 2* PI */
                                               / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                   / SECONDS_PER_MINUTE);

              /* Calculate the time for stopping from forced commutation stage */
              SpeedDownTime = (uint16_t)( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * ForcedSpeed[channel_number]
                                           * MotorParameterValues[channel_number].PolePairs
                                             * PAI2_VALUE                           /* 2* PI */
                                               / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                   / SECONDS_PER_MINUTE);
              break;

          case HZ_BY_S:
              /* Calculate the time needed to reach target rpm in ms */
              SpeedUpTime = (uint16_t)  ( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * max_speed
                                           * MotorParameterValues[channel_number].PolePairs
                                             / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                               / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);

              /* Calculate the time for stopping from forced commutation stage */
              SpeedDownTime = (uint16_t)( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * ForcedSpeed[channel_number]
                                           * MotorParameterValues[channel_number].PolePairs
                                             / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                               / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);
              break;

          case RPM_BY_S:
              /* Calculate the time needed to reach target rpm in ms */
              SpeedUpTime = (uint16_t)  ( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * max_speed
                                           / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                             / MotorParameterValues[channel_number].MaxAngAcc.value
                                               / SECONDS_PER_MINUTE);

              /* Calculate the time for stopping from forced commutation stage */
              SpeedDownTime = (uint16_t)( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * ForcedSpeed[channel_number]
                                           / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                             / MotorParameterValues[channel_number].MaxAngAcc.value
                                               / SECONDS_PER_MINUTE/10);
              break;

          case RPM_BY_MIN:
              /* Calculate the time needed to reach target rpm in ms */
              SpeedUpTime = (uint16_t)  ( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * max_speed
                                           / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                             / MotorParameterValues[channel_number].MaxAngAcc.value);

              /* Calculate the time for stopping from forced commutation stage */
              SpeedDownTime = (uint16_t)( (long long)1000                                 /* calculate time in ms for accuracy */
                                         * ForcedSpeed[channel_number]
                                           / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                             / MotorParameterValues[channel_number].MaxAngAcc.value);
              break;

          default:
              assert_param(0);
              break;
      }

      /* Calculate the number of pulses that shall be counted during speed up */
      PulsesWhileSpeedUp   = max_speed
                              * SpeedUpTime
                              * MotorParameterValues[channel_number].EncoderResolution
                              / SECONDS_PER_MINUTE
                              / 1000;  /* Time was in ms */

      /* Calculate the number of pulses during stopping from forced commutation speed */
      PulsesWhileSpeedDown = ForcedSpeed[channel_number]
                              * SpeedDownTime
                              *  MotorParameterValues[channel_number].EncoderResolution
                              / SECONDS_PER_MINUTE
                              / 1000;  /* Time was in ms */

      /* Safety check - when speeding up / down + safety margin makes more turns than just reaching the target speed */
      if( abs(nr_pulses) < abs(5*PulsesWhileSpeedUp) )
      {
          max_speed -= max_speed/10;
          goto recalculate_linear;
      }

      if( PulsesWhileSpeedDown < MotorParameterValues[channel_number].EncoderResolution )
      {
          PulsesWhileSpeedDown = MotorParameterValues[channel_number].EncoderResolution;
      }

      if( PulsesWhileSpeedUp < MotorParameterValues[channel_number].EncoderResolution )
      {
          PulsesWhileSpeedUp = MotorParameterValues[channel_number].EncoderResolution;
      }

      WaitValueForTarget[channel_number] = TargetValue[channel_number] - MotorParameterValues[channel_number].EncoderStartPos;

      if( E_MC_ROTATE_COUNTERCLOCKWISE == eRotDir )
      {
          MaxSpeed[channel_number] *= -1;
          ForcedSpeed[channel_number] *= -1;
          WaitValueForShutdown[channel_number] = WaitValueForTarget[channel_number] + PulsesWhileSpeedDown;
          WaitValueForForced[channel_number]   = WaitValueForTarget[channel_number] + PulsesWhileSpeedUp + PulsesWhileSpeedDown;
      }
      else
      {
          WaitValueForShutdown[channel_number] = WaitValueForTarget[channel_number] - PulsesWhileSpeedDown;
          WaitValueForForced[channel_number]   = WaitValueForTarget[channel_number] - PulsesWhileSpeedUp - PulsesWhileSpeedDown;
      }

      /* Add safety margins */
      if( nr_turns < LOW_SPEED_BORDER )
      {
          if ( E_MC_ROTATE_COUNTERCLOCKWISE != eRotDir )
          {
              WaitValueForShutdown[channel_number] += MotorParameterValues[channel_number].EncoderResolution;
              WaitValueForForced[channel_number] += MotorParameterValues[channel_number].EncoderResolution;
          }
          else
          {
              WaitValueForShutdown[channel_number] -= MotorParameterValues[channel_number].EncoderResolution;
              WaitValueForForced[channel_number] -= MotorParameterValues[channel_number].EncoderResolution;
          }
      }
      /* Set the flag to start the linear motion control task */
      MotorControl[channel_number].do_pulses_control = 1;
    }
}

/*! \brief  Absolute position parameters retrieval
*
* Get the absolute postion and limits
*
* @param  channel_number: channel number
* @retval Current position
*/
uint32_t GetPosition(uint8_t channel)
{
    if( channel < MAX_CHANNEL )
    {
        /* Ensure the current position matches the actual rotor position - manual motor rotation, improper stopping in the previous run, etc. */
        UpdateAbsolutePosition(channel);
        return MotorControl[channel].CurrentAbsPos;
    }
    return 0;
}

/*! \brief  Abort linear motion command
*
* Abort the current linear motion command
*
* @param  channel_number: channel number
* @retval Operation result
*/
int AbortLinearMotionCommand(uint8_t channel)
{
    int iRetVal = -1;

    if( channel < MAX_CHANNEL )
    {
        ENC_DisableIRQ(channel);
        MotorSetValues[channel].TargetSpeed = 0;
        MotorControl[channel].do_pulses_control = 0;
        iRetVal = 0;
    }

    return iRetVal;
}

/*! \brief
*
*
*
* @param  channel_number: channel number
* @retval Current set
*/
void MotionControl_IRQ_Callback(uint8_t channel, uint8_t irq_nr)
{
    UpdateAbsolutePosition(channel);
    /* IRQ stage handling */
    switch( irq_nr )
    {
        /* Rotate up to shutdown point */
        case 0:
            MotorSetValues[channel].TargetSpeed = ForcedSpeed[channel];
            ENC_SetEncoderIRQValue(channel, 1, WaitValueForShutdown[channel]);
            break;

        /* Configure the approach */
        case 1:
            /* Limit speed to approach level */
            ENC_SetEncoderIRQValue(channel, 2, WaitValueForTarget[channel]);
            MotorSetValues[channel].TargetSpeed = FinalApproachRpm[channel];
            if( ForcedSpeed[channel] < 0 )
            {
              MotorSetValues[channel].TargetSpeed *= -1;
            }
            break;

        /* Target position reached */
        case 2:
            MotorSetValues[channel].TargetSpeed = 0;
#ifdef USE_MOTION_CONTROL_WITH_MOTOR_STOP            
            /* Stop the motor - was not part of the EW2020 demo, thus has to be carefully tested with the AR3 Robot */
            MotorControl_Stop(channel);
#endif /* USE_MOTION_CONTROL_WITH_MOTOR_STOP */            
            MotorControl[channel].do_pulses_control = 0;
            ENC_DisableIRQ(channel);            
            break;
    }
}

/*! \brief  Motion Task
*
* Control Task for reaching an absolute position
*
* @param  pvParameters: Channel number
* @retval None
*/
void MotionTask(void* pvParameters)
{
    long channel = (long) pvParameters;

    /* Wait until all tasks intialization has completed */
    while( INIT_NOT_DONE == InitDone )
    {
        vTaskDelay( 100 / portTICK_RATE_MS );
    }

    /* One time position initialization, either with fixed values or slowly turining to the min/max limits */
    InitializeAbsolutePosition(channel);

    while( 1 )
    {
        /* Wait until linear motion control for 'channel' is requested, e.g. the 'do_pulses_control' flag is set by the corresponding task */
        if( 1 == MotorControl[channel].do_pulses_control )
        {
            /* Command can only be processed if the motor is not in motion */
            if(( 0 == MotorExtendedStateValues[channel].ActualControlledParameter )
               &&( MotorExtendedStateValues[channel].ControlMethod == MOTORCONTROL_CONTROL_BY_SPEED ))
            {   
                /* Set the comapre value to ensure that that motor is not rotating before encoder is configured*/
                ENC_SetEncoderIRQValue(channel, 0, WaitValueForForced[channel]);
                ENC_EnableIRQ(channel);
                
                /* Set the approach speed for the current channel */
                FinalApproachRpm[channel] = MotorParameterValues[channel].EncoderLMApproach * MotorParameterValues[channel].EncoderGearFactor;

                /* Start with the maximum requested/possible speed */
                MotorSetValues[channel].TargetSpeed = MaxSpeed[channel];
                if( 0 == MotorControl[channel].IsStarted )
                {
                    /* Start the motor */
                    MotorControl_Start(channel);
                }
                ENC_SetEncoderIRQValue(channel, 0, WaitValueForForced[channel]);
                ENC_EnableIRQ(channel);

                while( (1 == MotorControl[channel].do_pulses_control) )
                {
                    /* Sleep a while */
                    vTaskDelay(1/portTICK_RATE_MS);
                }

                /* Ensure the current position matches the actual rotor position - manual motor rotation, improper stopping in the previous run, etc. */
                UpdateAbsolutePosition(channel);                
            }
            else
            {
                /* Invalidate the request */
                MotorControl[channel].do_pulses_control = 0;
            }
        }

        /* Sleep a while */
        vTaskDelay(100/portTICK_RATE_MS);
    }
}
#endif // USE_MOTION_CONTROL
