/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include "motor_control.h"
#include "fwmathlib.h"
#include "units_enums.h"
#include "hal_encoder.h"
#include "board.h"
#include "txz_driver_def.h"
#include "speed_controller.h"
#include "torque_controller.h"
#include "sw_foc.h"
#include "temperature_control.h"


#define VE_FOC_HYSTERESIS       5


#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
#ifdef USE_HV_COMMUNICATION
#define HV_COMM 1
#endif /* USE_HV_COMMUNICATION */
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_0 */

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
#ifdef USE_HV_COMMUNICATION
#define HV_COMM 1
#endif /* USE_HV_COMMUNICATION */
#include "pwr_undefine.h"
#endif /* BOARD_PWR_HEADER_FILE_1 */

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "led.h"
#include "rgb_led.h"
#include "hal_timer.h"
#include "hal_pmd.h"
#include "hal_adc.h"

#include "performance_measurement.h"

/*! \brief  Calculate new target speed
  *
  * Calculate the new setting Omega if the user gives a new target speed (in rpm)
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void calculate_new_target_speed(uint8_t channel_number)
{
  static int32_t old_target=0;                                                  /* remember old value */
  static int64_t electrical_speed_per_minute;                                   /* local target speed (multiplied with 60) */
         int64_t limit = 0;

  if ((MotorSetValues[channel_number].TargetSpeed == old_target) ||
      ((MotorSetValues[channel_number].TargetSpeed ==0) && (MotorControl[channel_number].Omega_Target ==0)))
    return;                                                                     /* nothing new to calculate */

  old_target = MotorSetValues[channel_number].TargetSpeed;                      /* remember new setting */

  electrical_speed_per_minute =   (int64_t)(MotorSetValues[channel_number].TargetSpeed)
                                * MotorParameterValues[channel_number].PolePairs;

  switch (MotorParameterValues[channel_number].SpeedLimit.unit)
  {
  case HZ:
    limit = (int64_t)SECONDS_PER_MINUTE * MotorParameterValues[channel_number].SpeedLimit.value;
    break;

  case RPM:
    limit = (int64_t)(MotorParameterValues[channel_number].SpeedLimit.value) * MotorParameterValues[channel_number].PolePairs;
    break;

  default:
    assert_param(0);
    break;
  }

  if (electrical_speed_per_minute > limit)
    electrical_speed_per_minute = limit;

  if (electrical_speed_per_minute < -limit)
    electrical_speed_per_minute = -limit;

  MotorErrorField[channel_number].Error &= ~VE_WRONGDIRECTION;

  /* check  CW only specified by the motor parameter */
  if (( MotorParameterValues[channel_number].Direction == MOTOR_CW_ONLY )  && ( electrical_speed_per_minute < 0))
  {
    electrical_speed_per_minute=0;
    MotorErrorField[channel_number].Error |= VE_WRONGDIRECTION;
  }

  /* check CCW only specified by the motor parameter */
  if (( MotorParameterValues[channel_number].Direction == MOTOR_CCW_ONLY ) && ( electrical_speed_per_minute > 0))
  {
    electrical_speed_per_minute=0;
    MotorErrorField[channel_number].Error |= VE_WRONGDIRECTION;
  }

  /* convert target speed from rpm to Hz:
   * 1Hz = 60sec = 1/60sec, targetspeed includes the pole count */
  MotorControl[channel_number].Omega_Target = (int32_t)( electrical_speed_per_minute
                                               * FIXPOINT_15
                                               / MotorControl[channel_number].HzMax
                                               / SECONDS_PER_MINUTE);
}

/*! \brief  Compute the values for the GUI
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void collect_data_for_display (uint8_t channel_number)
{
  int64_t         av_out;                                                       /* average value output */
  uint32_t        current;
  static uint8_t  av_count[MAX_CHANNEL];                                        /* average build counter value output */
  static int64_t  speedAverage[MAX_CHANNEL][5];                                 /*< output for GUI */

  /* Calculate rpm average over 5 ms */
  speedAverage[channel_number][av_count[channel_number]++] = MotorControl[channel_number].Omega.part.reg/5;
  if (av_count[channel_number] > 4)
  {
    av_count[channel_number] = 0;
  }
  av_out = speedAverage[channel_number][0] + speedAverage[channel_number][1] +
           speedAverage[channel_number][2] + speedAverage[channel_number][3] +
           speedAverage[channel_number][4];

  /* Calculater current normalized to A_MAX and FIXPOINT_15 */
  current =  (uint32_t)abs(MotorControl[channel_number].Iq_reference_I.part.reg);
  current *= MotorControl[channel_number].a_max.value;
  current /= FIXPOINT_15;
  current /= pow10(-MotorControl[channel_number].a_max.unit-3);

  /* Calculate actual rpm */
  /* VE_Omega (-> av_out) is electrical speed normalized with HzMax and FIXPOINT_15 */
  /* therefor it's dependant of: */
  /* HzMax, Polepairs and FIXPOINT_15 */

  switch (MotorExtendedStateValues[channel_number].ControlMethod)
  {
  case MOTORCONTROL_CONTROL_BY_SPEED:
    MotorExtendedStateValues[channel_number].ActualControlledParameter
      = (
         (av_out
          * SECONDS_PER_MINUTE
            * MotorControl[channel_number].HzMax
              / FIXPOINT_15
                / MotorParameterValues[channel_number].PolePairs
                  )
           * 2+1)/2;
    /* Target speed in rpm */
    MotorExtendedStateValues[channel_number].TargetControlledParameter = MotorSetValues[channel_number].TargetSpeed;

    /* Current in mA */
    MotorExtendedStateValues[channel_number].Current      = current;

    /* Torque in Ncm */
    /* TorqueFactor is in mNm */
    /* VE_Current is in mA */
    MotorExtendedStateValues[channel_number].NonControlParameter       = MotorParameterValues[channel_number].TorqueFactor.value
      * current;

    if ( (-MotorParameterValues[channel_number].TorqueFactor.unit-2) >=0 )
      MotorExtendedStateValues[channel_number].NonControlParameter     /= pow10(-MotorParameterValues[channel_number].TorqueFactor.unit-2);
    else
      MotorExtendedStateValues[channel_number].NonControlParameter     *= pow10(+MotorParameterValues[channel_number].TorqueFactor.unit+2);

    break;
  case MOTORCONTROL_CONTROL_BY_TORQUE:
    MotorExtendedStateValues[channel_number].NonControlParameter
      = (
         (av_out
          * SECONDS_PER_MINUTE
            * MotorControl[channel_number].HzMax
              / FIXPOINT_15
                / MotorParameterValues[channel_number].PolePairs
                  )
           * 2+1)/2;


    /* Current in mA */
    MotorExtendedStateValues[channel_number].Current      = current;

    /* Torque in Ncm */
    /* TorqueFactor is in mNm */
    /* VE_Current is in mA */
    MotorExtendedStateValues[channel_number].ActualControlledParameter       = MotorParameterValues[channel_number].TorqueFactor.value
      * current;

    if ( (-MotorParameterValues[channel_number].TorqueFactor.unit-2) >=0 )
      MotorExtendedStateValues[channel_number].ActualControlledParameter     /= pow10(-MotorParameterValues[channel_number].TorqueFactor.unit-2);
    else
      MotorExtendedStateValues[channel_number].ActualControlledParameter     *= pow10(+MotorParameterValues[channel_number].TorqueFactor.unit+2);

    MotorExtendedStateValues[channel_number].TargetControlledParameter  = MotorExtendedStateValues[channel_number].ActualControlledParameter;
    break;
  default:
    assert_param(0);
    break;
  }
}

  /*! \brief  Limit Omega to speed up borders
  *
  * @param  now:      Actual Omega
  * @param  target:   Target Omega
  * @param  limit:    Accelleration limit
*/
static int32_t Limit_Omega(int64_t now, int64_t target, int64_t limit)
{
  int64_t delta;                                                                /* local data with 64 bit */

  delta = target - now;

  if(delta == 0)                                                                /* nothing to do : useless call */
    return(target);                                                             /* target value is reached */
  else                                                                          /* tendencies */
  {
    if(delta > 0)                                                               /* positive delta -> positive tendency */
      if(delta > limit)                                                         /* determine staep wise or rest */
        return(now + limit);                                                    /* -> returns new value up step */
      else
        return(now + delta);                                                    /* or -> returns new value up rest */
    else                                                                        /* negative delta -> negative tendency */
    {
      delta = -delta;                                                           /* determine step wise or rest */
      if(delta > limit)
        return(now - limit);                                                    /* -> returns new value down step */
      else
        return(now - delta);                                                    /* or -> returns new value down rest */
    }
  }
}

/*! \brief  Calculate some values for further use
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void Precalculate_ChannelValues(unsigned char channel_number)
{
  uint64_t temp;

  temp  = (uint64_t)25;                                                         /* 2.5V * 10 (10 from gaintable) */
  temp *= pow10(6);                                                             /* using mA as base */
  temp /= gaintable[ChannelValues[channel_number].GainCurrentMeasure];
  temp /= ChannelValues[channel_number].SensitivityCurrentMeasure.value;

  MotorControl[channel_number].a_max.value = temp;

  MotorControl[channel_number].a_max.unit  = -6-ChannelValues[channel_number].SensitivityCurrentMeasure.unit;

  temp  = (uint64_t)5;                                                          /* 5V */
  temp *= pow10(-ChannelValues[channel_number].SensitivityVoltageMeasure.unit+3);
  temp /= ChannelValues[channel_number].SensitivityVoltageMeasure.value;
  temp /= pow10(3);

  MotorControl[channel_number].v_max.value = temp;

  MotorControl[channel_number].v_max.unit  = 0;

#ifdef BOARD_VDC_CHANNEL_0_VALUE
  MotorControl[0].v_max.value              = BOARD_VDC_CHANNEL_0_VALUE;
  MotorControl[0].v_max.unit               = BOARD_VDC_CHANNEL_0_UNIT;
#endif
#ifdef BOARD_VDC_CHANNEL_1_VALUE
  MotorControl[1].v_max.value              = BOARD_VDC_CHANNEL_1_VALUE;
  MotorControl[1].v_max.unit               = BOARD_VDC_CHANNEL_1_UNIT;
#endif
}

/*! \brief  Do some precalculations of Motor constants to reduce cpu load while running
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void Precalculate_MotorValues(unsigned char channel_number)
{
  uint64_t temp = 0;

  /* CAUTION: The following cruel usage of temp for single step calculations
              is needed due to some very cruel compiler behavior when using -O3 */

  /* Wait time calculations */

  MotorControl[channel_number].PreCalculation.WaitTime_Position
    =   MotorParameterValues[channel_number].PositionDelay
      * VE_CONTROL_LOOP_FREQUENCY;                                              /* Time of Positioning (Force) */

  MotorControl[channel_number].PreCalculation.WaitTime_Bootstrap
    =  ChannelValues[channel_number].BootstrapDelay
      * VE_CONTROL_LOOP_FREQUENCY;                                              /* Time of Bootstapping */

  MotorControl[channel_number].PreCalculation.WaitTime_Brake
    =   SystemValues[channel_number].Braketime
      * VE_CONTROL_LOOP_FREQUENCY;                                              /* Time of Braking */

  /* Calculations due to speed */

  switch (MotorParameterValues[channel_number].SpeedLimit.unit)                 /* put HzMax scaling factor 20 % over HzLimit */
  {
  uint32_t Speed_Hz;

  case HZ:
    MotorControl[channel_number].HzMax
      =   MotorParameterValues[channel_number].SpeedLimit.value
        + MotorParameterValues[channel_number].SpeedLimit.value / 5;
    break;

  case RPM:
    Speed_Hz =   MotorParameterValues[channel_number].SpeedLimit.value
               * MotorParameterValues[channel_number].PolePairs
               / SECONDS_PER_MINUTE ;
    MotorControl[channel_number].HzMax
      = Speed_Hz + Speed_Hz / 5;
    break;

  default:
    assert_param(0);
    break;
  }

  /* Change up frequency normed to HzMax */

  switch (MotorParameterValues[channel_number].SpeedChange.unit)
  {
  case RPM:
    temp =   (uint64_t)MotorParameterValues[channel_number].SpeedChange.value
           * MotorParameterValues[channel_number].PolePairs
           / SECONDS_PER_MINUTE;
    break;
  case HZ:
    temp =   MotorParameterValues[channel_number].SpeedChange.value;
    break;

  default:
    assert_param(0);
    break;
  }

  temp *= FIXPOINT_15;
  temp /= MotorControl[channel_number].HzMax;

  MotorControl[channel_number].PreCalculation.ChangeFrqUp_normed_to_HzMax
    = temp;

  /* Change down frequency normed to HzMax */
  temp *= VE_FOC_HYSTERESIS;
  temp /= 100;

  MotorControl[channel_number].PreCalculation.ChangeFrqDown_normed_to_HzMax
    = MotorControl[channel_number].PreCalculation.ChangeFrqUp_normed_to_HzMax-temp;

  /* HzMax normed to PWM Frequency */

  temp  = (uint64_t)FIXPOINT_16;
  temp *= MotorControl[channel_number].HzMax;
  temp /= SystemValues[channel_number].PwmFrequency;

  MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency
    = temp;

  switch (MotorParameterValues[channel_number].MaxAngAcc.unit)
  {
  case RAD_BY_S2:
    /* rad/sec 2 -> Hz/s : [rad]/2*pi=[Hz/s] : 125rad/s 2=20 1/s 2 */
    temp  = ((uint64_t)FIXPOINT_15)<<16;
    temp /= MotorControl[channel_number].HzMax;
    temp *= MotorParameterValues[channel_number].MaxAngAcc.value;
    temp *= 10;
    temp /= VE_CONTROL_LOOP_FREQUENCY;
    temp /= PAI2_VALUE;
    MotorControl[channel_number].PreCalculation.SpeedUpLimit = temp;
    break;

  case HZ_BY_S:
    temp  = ((uint64_t)FIXPOINT_15)<<16;
    temp /= MotorControl[channel_number].HzMax;
    temp *= MotorParameterValues[channel_number].MaxAngAcc.value;
    temp /= 1000;
    temp /= VE_CONTROL_LOOP_FREQUENCY;
    MotorControl[channel_number].PreCalculation.SpeedUpLimit = temp;
    break;

  case RPM_BY_S:
    temp  = ((uint64_t)FIXPOINT_15)<<16;
    temp /= MotorControl[channel_number].HzMax;
    temp *= MotorParameterValues[channel_number].MaxAngAcc.value;
    temp /= 1000;
    temp *= MotorParameterValues[channel_number].PolePairs;
    temp /= VE_CONTROL_LOOP_FREQUENCY;
    temp /= SECONDS_PER_MINUTE;
    MotorControl[channel_number].PreCalculation.SpeedUpLimit = temp;
    break;

  case RPM_BY_MIN:
    temp  = ((uint64_t)FIXPOINT_15)<<16;
    temp /= MotorControl[channel_number].HzMax;
    temp *= MotorParameterValues[channel_number].MaxAngAcc.value;
    temp /= 1000;
    temp *= MotorParameterValues[channel_number].PolePairs;
    temp /= VE_CONTROL_LOOP_FREQUENCY;
    temp /= SECONDS_PER_MINUTE;
    temp /= SECONDS_PER_MINUTE;
    MotorControl[channel_number].PreCalculation.SpeedUpLimit = temp;
    break;

  default:
    assert_param(0);
    break;
  }


  /* Pre calculations for estimator */
  /**********************************/

  /* Value */
  temp  = (uint64_t)FIXPOINT_15;
  temp *= MotorParameterValues[channel_number].Resistance.value;
  temp *= MotorControl[channel_number].a_max.value;
  temp /= MotorControl[channel_number].v_max.value;  
  temp /= pow10(  MotorControl[channel_number].v_max.unit
                - MotorControl[channel_number].a_max.unit
                - MotorParameterValues[channel_number].Resistance.unit
                );
  MotorControl[channel_number].PreCalculation.R_mult_amax_div_vmax
    =   temp;

  /* Value */
  temp  = (uint64_t)FIXPOINT_15;
  temp *= MotorParameterValues[channel_number].Inductance.value;
  temp *= PAI2_VALUE;
  temp /= pow10(-PAI2_UNIT);                                                    /* Scaling of PAI2 */
  temp /= MotorControl[channel_number].v_max.value;
  temp *= MotorControl[channel_number].a_max.value;
  temp *= MotorControl[channel_number].HzMax;
  temp /= pow10(- MotorControl[channel_number].a_max.unit
                - MotorParameterValues[channel_number].Inductance.unit
                + MotorControl[channel_number].v_max.unit
	        );

  MotorControl[channel_number].PreCalculation.InductanceDropoutFactor
    =   temp;

  /* Value */
  temp  = (uint64_t)FIXPOINT_15;
  temp *= PIControl[channel_number].PositionKi;
  temp *= MotorControl[channel_number].v_max.value;
  temp /= MotorControl[channel_number].HzMax;
  temp /= SystemValues[channel_number].PwmFrequency;
  temp /= pow10(-MotorControl[channel_number].v_max.unit);

  MotorControl[channel_number].PreCalculation.PositionKiPreCalculation
    = temp;

  /* Value */
  temp  = (uint64_t)FIXPOINT_15;
  temp *= PIControl[channel_number].PositionKp;
  temp *= MotorControl[channel_number].v_max.value;
  temp /= MotorControl[channel_number].HzMax;
  temp /= pow10(-MotorControl[channel_number].v_max.unit+3);

  MotorControl[channel_number].PreCalculation.PositionKpPreCalculation
    =   temp;

  temp = (uint64_t)FIXPOINT_16;
  temp *= PIControl[channel_number].SpeedKp;
  temp *= MotorControl[channel_number].HzMax;
  temp /= MotorControl[channel_number].a_max.value;
  temp /= pow10(-MotorControl[channel_number].a_max.unit-3);

  MotorControl[channel_number].PreCalculation.SpeedKpPreCalculation
    =   temp;

  temp = (uint64_t)FIXPOINT_16;
  temp *= PIControl[channel_number].SpeedKi;
  temp *= MotorControl[channel_number].HzMax;
  temp /= MotorControl[channel_number].a_max.value;
  temp /= SystemValues[channel_number].PwmFrequency;
  temp /= pow10(-MotorControl[channel_number].a_max.unit-3);

  MotorControl[channel_number].PreCalculation.SpeedKiPreCalculation
    =   temp;


  /* Current precalculations */
  /***************************/
  temp  = (uint64_t)FIXPOINT_15;
  temp *= MotorParameterValues[channel_number].IqLimit.value;
  if ((MotorParameterValues[channel_number].IqLimit.unit - MotorControl[channel_number].a_max.unit) < 0)
    temp /= pow10(-MotorParameterValues[channel_number].IqLimit.unit + MotorControl[channel_number].a_max.unit);
  else
    temp *= pow10( MotorParameterValues[channel_number].IqLimit.unit - MotorControl[channel_number].a_max.unit);
  temp /= MotorControl[channel_number].a_max.value;

  MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax
    = temp;

  temp  = (uint64_t)FIXPOINT_15;
  temp *= MotorParameterValues[channel_number].IdLimit.value;
  if ( (MotorParameterValues[channel_number].IdLimit.unit - MotorControl[channel_number].a_max.unit) < 0)
    temp /= pow10(-MotorParameterValues[channel_number].IdLimit.unit + MotorControl[channel_number].a_max.unit);
  else
    temp *= pow10( MotorParameterValues[channel_number].IdLimit.unit - MotorControl[channel_number].a_max.unit);
  temp /= MotorControl[channel_number].a_max.value;

  MotorControl[channel_number].PreCalculation.IdLimit_normed_to_amax
    = temp;

  temp  = (uint64_t)FIXPOINT_15;
  temp *= MotorParameterValues[channel_number].IdStart.value;
  if ((MotorParameterValues[channel_number].IdStart.unit - MotorControl[channel_number].a_max.unit) < 0)
    temp /= pow10(-MotorParameterValues[channel_number].IdStart.unit + MotorControl[channel_number].a_max.unit);
  else
    temp *= pow10( MotorParameterValues[channel_number].IdStart.unit - MotorControl[channel_number].a_max.unit);
  temp /= MotorControl[channel_number].a_max.value;

  MotorControl[channel_number].PreCalculation.IdCurrentForInitposition
    = temp;

  temp  = (uint64_t)FIXPOINT_15;
  temp *= MotorParameterValues[channel_number].IqStart.value;
  if ((MotorParameterValues[channel_number].IqStart.unit - MotorControl[channel_number].a_max.unit) < 0)
    temp /= pow10(-MotorParameterValues[channel_number].IqStart.unit + MotorControl[channel_number].a_max.unit);
  else
    temp *= pow10( MotorParameterValues[channel_number].IqStart.unit - MotorControl[channel_number].a_max.unit);
  temp /= MotorControl[channel_number].a_max.value;

  MotorControl[channel_number].PreCalculation.IqCurrentForInitposition
    = temp;

  temp   = (uint64_t)FIXPOINT_15;
  temp  *= SystemValues[channel_number].SwOvercurrent.value;
  if ((MotorControl[channel_number].a_max.unit-SystemValues[channel_number].SwOvercurrent.unit) < 0)
    temp *= pow10(-MotorControl[channel_number].a_max.unit+SystemValues[channel_number].SwOvercurrent.unit);
  else
    temp /= pow10( MotorControl[channel_number].a_max.unit-SystemValues[channel_number].SwOvercurrent.unit);

  temp  /= MotorControl[channel_number].a_max.value;

  MotorControl[channel_number].PreCalculation.SWOvercurrentSquare
    =  temp*temp;

}



/******************************************************************************/
/******************************************************************************/
/************************* Control loop stages ********************************/
/******************************************************************************/
/******************************************************************************/

/*! \brief  Emergency Stage
  *
  * Shut down the Motor as fast as possible
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_Emergency(uint8_t channel_number)
{
  if (channel_number==0)
    HAL_VE_PMD_Output(channel_number,PMD_OFF);
  MotorExtendedStateValues[channel_number].ActualControlledParameter  = 0;
  MotorExtendedStateValues[channel_number].Current                    = 0;
  MotorExtendedStateValues[channel_number].NonControlParameter        = 0;
  MotorExtendedStateValues[channel_number].TargetControlledParameter  = 0;

}

/*! \brief  Stop Stage
  *
  * Switch off the Motor
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_Stop(uint8_t channel_number)
{
  if (channel_number == 0)
  {
    HAL_VE_IRQ(channel_number,VE_IRQ_OFF);
    HAL_VE_PMD_Output(channel_number,PMD_OFF);
  }
  
#ifdef FIX_155_PMD_DELAY
  if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
  {
    gPmdRegisterSetting[channel_number][PMD_OutputOff_EmergencyDisable-1] = PMD_OutputOff_EmergencyDisable; /*0x01*/
  }
  else
  {
    HAL_PMD_OutputOff(channel_number);
    HAL_PMD_EmergencyDisable(channel_number);
  }
#else
  HAL_PMD_OutputOff(channel_number);
  HAL_PMD_EmergencyDisable(channel_number);
#endif

  MotorErrorField[channel_number].Error &= 0x1;                                 /* Reset all errors except overtemperature */

  switch (MotorExtendedStateValues[channel_number].ControlMethod)
  {
    case MOTORCONTROL_CONTROL_BY_TORQUE:
      TorqueController_Calculate_Id_Iq_ref(channel_number);
      break;

    case MOTORCONTROL_CONTROL_BY_SPEED:
      SpeedController_Calculate_Id_Iq_ref(channel_number);
      break;
  }

  MotorExtendedStateValues[channel_number].ActualControlledParameter  = 0;
  MotorExtendedStateValues[channel_number].Current                    = 0;
  MotorExtendedStateValues[channel_number].NonControlParameter        = 0;
  MotorExtendedStateValues[channel_number].TargetControlledParameter  = 0;

  MotorControl[channel_number].Omega.value         = 0;
  MotorControl[channel_number].Omega_command.value = 0;
  MotorControl[channel_number].StageCounter        = 0;                                   /* stage counter to limit time */

  if (channel_number == 0)
  {
    /* Reset some VE registers to 0 to prevent output on next start */
    TSB_VE0->ACTSCH   = 0;
    TSB_VE0->TMPREG0  = 0;
    TSB_VE0->TMPREG1  = 0;
    TSB_VE0->TMPREG2  = 0;
    TSB_VE0->TMPREG3  = 0;
    TSB_VE0->TMPREG4  = 0;
    TSB_VE0->TMPREG5  = 0;
  }

  if (SystemValues[channel_number].ShutdownMode != SHUTDOWN_GENTLE)
    MotorControl[channel_number].Theta.value = 0;
}

/*! \brief  Bootstrap Stage
  *
  * Bootstrap IGBTs
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_Bootstrap(uint8_t channel_number)
{
  switch (MotorExtendedStateValues[channel_number].ControlMethod)
  {
    case MOTORCONTROL_CONTROL_BY_TORQUE:
      TorqueController_Calculate_Id_Iq_ref(channel_number);
      break;

    case MOTORCONTROL_CONTROL_BY_SPEED:
      SpeedController_Calculate_Id_Iq_ref(channel_number);
      break;
  }

  MotorControl[channel_number].Omega.value         = 0;
  MotorControl[channel_number].Omega_command.value = 0;

#ifdef FIX_155_PMD_DELAY
  if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
  {
    gPmdRegisterSetting[channel_number][PMD_OutputPattern-1] = PMD_OutputPattern; /* 0x03 */
  }
  else
  {
    HAL_PMD_OutputPattern(channel_number,PMD_SHORTCUT_LOW_SIDE);
  }
#else
  HAL_PMD_OutputPattern(channel_number,PMD_SHORTCUT_LOW_SIDE);
#endif

  MotorControl[channel_number].StageCounter++;                                            /* stage counter to limit time */
                                                                                /* check timeout */
  if(MotorControl[channel_number].StageCounter >= MotorControl[channel_number].PreCalculation.WaitTime_Bootstrap)
  {
#ifdef FIX_155_PMD_DELAY
    if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
    {
      gPmdRegisterSetting[channel_number][PMD_NormalOutput-1] = PMD_NormalOutput; /* 0x04 */
    }
    else
    {
      HAL_PMD_NormalOutput(channel_number);
    }
#else
    HAL_PMD_NormalOutput(channel_number);
#endif
    MotorControl[channel_number].StageCounter     = 0;
    MotorControl[channel_number].ActualStage.main = Stage_ZeroCurrentMeasure;             /* startup */
    MotorControl[channel_number].ActualStage.sub  = Substage_Step0;
  }
}

/*! \brief  Brake Stage
  *
  * Shortbrake
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_Brake(uint8_t channel_number)
{

  MotorExtendedStateValues[channel_number].ActualControlledParameter  = 0;
  MotorExtendedStateValues[channel_number].Current                    = 0;
  MotorExtendedStateValues[channel_number].NonControlParameter        = 0;
  MotorExtendedStateValues[channel_number].TargetControlledParameter  = 0;

  MotorControl[channel_number].StageCounter++;                                            /* stage counter to limit time */
                                                                                /* check timeout */
  if(MotorControl[channel_number].StageCounter >= MotorControl[channel_number].PreCalculation.WaitTime_Brake)
  {
    MotorControl[channel_number].StageCounter     = 0;
    MotorControl[channel_number].ActualStage.main = Stage_Stop;
    MotorControl[channel_number].ActualStage.sub  = Substage_Step0;
  }
}

/*! \brief  Measure Zerocurrent
  *
  * Measure the current value when no current is running
  * Needed to determine the Zero Crossing Point
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_MeasureZeroCurrent(uint8_t channel_number)
{
  /* This part has been moved to the IRQ  for Channel 0 to minimize the time for the Zero current Measurement */

  if (channel_number > 0)
  {
    SW_FOC_GetZeroCurrent(channel_number);
    if (MotorControl[channel_number].Theta.value == 0)                                  /* first time of startup or nit shutdown gentle mode */
      MotorControl[channel_number].ActualStage.main = Stage_Initposition;               /* set next stage */
    else
      MotorControl[channel_number].ActualStage.main = Stage_Force;                      /* set next stage */

    MotorControl[channel_number].ActualStage.sub  = Substage_Step0;                     /* set next substage */
    MotorControl[channel_number].StageCounter     = 0;                                  /* stage counter to limit time */
  }
}

/*! \brief  Initposition Stage
  *
  * Put the Motor to a defined position by putting current to a polepair
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_Initposition(uint8_t channel_number)
{
  switch(MotorControl[channel_number].ActualStage.sub)
  {                                                                             /* initial substage */
  case Substage_Step0:
#ifdef USE_EMERGENCY_SIGNAL      
#ifdef FIX_155_PMD_DELAY
    if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
    {
      gPmdRegisterSetting[channel_number][PMD_EmergencyReset_Initposition-1] = PMD_EmergencyReset_Initposition; /* 0x05 */
    }
    else
    {
      HAL_PMD_EmergencyReset(channel_number);
    }
#else
    HAL_PMD_EmergencyReset(channel_number);
#endif
    if (channel_number==0)
      HAL_VE_PMD_EMGReturn(channel_number);
#endif /* USE_EMERGENCY_SIGNAL */
#ifdef USE_OVERVOLTAGE_SIGNAL
#ifdef FIX_155_PMD_DELAY
    if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
    {
      gPmdRegisterSetting[channel_number][PMD_OvervoltageReset_Initposition-1] = PMD_OvervoltageReset_Initposition; /* 0x06 */
    }
    else
    {
      HAL_PMD_OvervoltageReset(channel_number);
    }
#else
    HAL_PMD_OvervoltageReset(channel_number);
#endif
#endif /* USE_OVERVOLTAGE_SIGNAL */
    MotorControl[channel_number].ActualStage.sub= Substage_Step1;                         /* next step */
    /*  No break. Continue to the next statements. */
  case Substage_Step1:
    MotorControl[channel_number].StageCounter++;                                          /* stage counter to limit time */
                                                                                /* check timeout */
    if(MotorControl[channel_number].StageCounter >=MotorControl[channel_number].PreCalculation.WaitTime_Position)
    {
      MotorControl[channel_number].ActualStage.sub = Substage_StepEnd;                    /* set last substage */
      MotorControl[channel_number].StageCounter    = 0;                                   /* stage counter to limit time */
    }

    switch (MotorExtendedStateValues[channel_number].ControlMethod)
    {
      case MOTORCONTROL_CONTROL_BY_TORQUE:
        TorqueController_Omega_Theta(channel_number);
        TorqueController_Calculate_Id_Iq_ref(channel_number);
        break;

      case MOTORCONTROL_CONTROL_BY_SPEED:
        SpeedController_Omega_Theta(channel_number);
        SpeedController_Calculate_Id_Iq_ref(channel_number);
        break;
    }

    break;

  case Substage_StepEnd:                                                        /* final stage */
    MotorControl[channel_number].ActualStage.main  = Stage_Force;                         /* set next stage */
    MotorControl[channel_number].ActualStage.sub   = Substage_Step0;                      /* set next substage */
    break;

  default:
    assert_param(0);
    break;
  }
}

/*! \brief  Forced Commutation Stage
  *
  * Speed up the motor constantly with no regard to the torque
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_Force(uint8_t channel_number)
{
  switch(MotorControl[channel_number].ActualStage.sub)
  {
  case Substage_Step0:                                                          /* initial substage */
#ifdef USE_EMERGENCY_SIGNAL
#ifdef FIX_155_PMD_DELAY
    if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
    {
      gPmdRegisterSetting[channel_number][PMD_EmergencyReset_Force-1] = PMD_EmergencyReset_Force; /* 0x07 */
    }
    else
    {
      HAL_PMD_EmergencyReset(channel_number);
    }
#else
    HAL_PMD_EmergencyReset(channel_number);
#endif
    if (channel_number==0)
      HAL_VE_PMD_EMGReturn(channel_number);
#endif /* USE_EMERGENCY_SIGNAL */
#ifdef USE_OVERVOLTAGE_SIGNAL
#ifdef FIX_155_PMD_DELAY
    if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
    {
      gPmdRegisterSetting[channel_number][PMD_OvervoltageReset_Force-1] = PMD_OvervoltageReset_Force; /* 0x08 */
    }
    else
    {
      HAL_PMD_OvervoltageReset(channel_number);
    }
#else
    HAL_PMD_OvervoltageReset(channel_number);
#endif
#endif /* USE_OVERVOLTAGE_SIGNAL */

    /* If reach minimum frequency, it change to cStepEnd */
    if(abs(MotorControl[channel_number].Omega_command.part.reg) >= MotorControl[channel_number].PreCalculation.ChangeFrqUp_normed_to_HzMax)
      MotorControl[channel_number].ActualStage.sub = Substage_StepEnd;                    /* limit exceeded -> final stage active */

    /* Update drive frequency. for ever in this mode */
    MotorControl[channel_number].Omega_command.value = Limit_Omega(MotorControl[channel_number].Omega_command.value,
                                                        MotorControl[channel_number].Omega_Target<<16,
                                                        MotorControl[channel_number].PreCalculation.SpeedUpLimit);
    break;

  case Substage_StepEnd:
    MotorControl[channel_number].ActualStage.main = Stage_FOC;                            /* final set for FOC stage */
    MotorControl[channel_number].ActualStage.sub  = Substage_Step0;                       /* set next substage */
    break;

  default:
    assert_param(0);
    break;
  }
}

/*! \brief  FOC (Field Oriented Commutation) Stage
  *
  * Stage to reach for best performance
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
static void MotorControl_Stage_FOC(uint8_t channel_number)
{
  switch(MotorControl[channel_number].ActualStage.sub)
  {
  case Substage_Step0:                                                          /* continuous stage */
    /* Update drive frequency. */
    MotorControl[channel_number].Omega_command.value = Limit_Omega(MotorControl[channel_number].Omega_command.value,
                                                        MotorControl[channel_number].Omega_Target<<16,
                                                        MotorControl[channel_number].PreCalculation.SpeedUpLimit);

    /* check low speed rotation */
    if(abs(MotorControl[channel_number].Omega_command.part.reg) < MotorControl[channel_number].PreCalculation.ChangeFrqDown_normed_to_HzMax)
    {
      MotorControl[channel_number].ActualStage.main  = Stage_Force;                       /* terminate this stage */
      MotorControl[channel_number].ActualStage.sub   = Substage_Step0;
    }
    break;

  default:
    assert_param(0);
    break;
  }
}

/*! \brief  Functions that have to be called on regular basis
  *
  * This is the main worker part from the Vector Engine Task
  * put to an own function for better structure
  *
  * @param  channel_number: channel number to work with
  * @retval None
*/
static void MotorControl_Loop(uint8_t channel_number)
{
#if (defined USE_TEMPERATURE_CONTROL) && (!defined HV_COMM)
  static uint16_t temp_counter=0;
#endif /* (defined USE_TEMPERATURE_CONTROL) && (!defined HV_COMM) */
  
/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32CtrlLoopTimePrev[channel_number] = u32CtrlLoopTimeStart[channel_number]; 
    /* Get the new start time */
    u32CtrlLoopTimeStart[channel_number] = DWT_CYCCNT_VAL;
    /* Measure the control loop cycle time - delta between two successive starts */
    u32CtrlLoopOccuranceRate[channel_number] = u32CtrlLoopTimeStart[channel_number] - u32CtrlLoopTimePrev[channel_number];                             
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */
  
  calculate_new_target_speed(channel_number);
  collect_data_for_display(channel_number);

#ifdef  USE_TEMPERATURE_CONTROL
#ifdef HV_COMM
  TEMPERATURE_ConfigureADCforTemperature(channel_number);
#else
  TEMPERATURE_CheckOvertemp(channel_number);
  if (temp_counter++>=1000)
  {
    temp_counter=0;
    TEMPERATURE_CheckOvertemp(channel_number);
  }
#endif /* HV_COMM */
#endif /* USE_TEMPERATURE_CONTROL */

  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Stop:
    MotorControl_Stage_Stop(channel_number);                                              /* active in Stop mode */
    break;
  case Stage_Bootstrap:
    MotorControl_Stage_Bootstrap(channel_number);                                         /* active in Bootstrap mode */
    break;
  case Stage_Brake:
    MotorControl_Stage_Brake(channel_number);                                             /* active in Brake mode */
    break;
  case Stage_ZeroCurrentMeasure:
    MotorControl_Stage_MeasureZeroCurrent(channel_number);                                /* measure zero current */
    break;
  case Stage_Initposition:
    MotorControl_Stage_Initposition(channel_number);                                      /* Positioning mode */
    break;
  case Stage_Force:
    MotorControl_Stage_Force(channel_number);                                             /* Forced commutation mode */
    break;
  case Stage_FOC:
    MotorControl_Stage_FOC(channel_number);                                               /* Speed control by current mode */
    break;
  case Stage_Emergency:
    MotorControl_Stage_Emergency(channel_number);                                         /* Emergency mode */
    break;
  default:
    assert_param(0);
    break;
  }
  
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32CtrlLoopTimeStop[channel_number] = DWT_CYCCNT_VAL;
    /* Measure the control processing time - delta between the stat and end times */
    u32CtrlLoopProcessingTime[channel_number] = u32CtrlLoopTimeStop[channel_number] - u32CtrlLoopTimeStart[channel_number];
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */ 
}

/*! \brief  clear off the rest for stopping the Vector Engine
  *
  * Helper function to be able to perform a gentle shutdown
  *
  * @param  pvParameters: channel number
  * @retval None
*/
static void complete_shutdown(uint8_t channel_number)
{
  TSB_VE_TypeDef*     pVEx    = NULL;

  switch (channel_number)
  {
  case 0:
    pVEx    = TSB_VE0;
    break;
  default:
    break;
  }

  MotorControl[channel_number].ActualStage.main = Stage_Stop;                             /* set STOP stage */
  MotorControl[channel_number].ActualStage.sub  = Substage_Step0;
#ifdef USE_LED
  LED_SetState(LED_SIGNAL_VE_RUN_BASE+channel_number,LED_OFF);                  /* Clear FOC signal Led */
#endif
#ifdef USE_RGB_LED
  RGB_LED_RestoreValue();
#endif /* USE_RGB_LED */

  MotorExtendedStateValues[channel_number].ActualControlledParameter  = 0;
  MotorExtendedStateValues[channel_number].Current                    = 0;
  MotorExtendedStateValues[channel_number].NonControlParameter        = 0;
  MotorExtendedStateValues[channel_number].TargetControlledParameter  = 0;

  MotorControl[channel_number].xVETask=NULL;                                                 /* Delete Task Handle */
  if (channel_number == 0)
  {
    HAL_VE_IRQ(channel_number,VE_IRQ_OFF);

    pVEx->VDIH =0;                                                                /*!< Upper 32 bits of integral term (VDI ) of d-axis voltage */
    pVEx->VDILH=0;                                                                /*!< Lower 32 bits of integral term (VDI) of d-axis voltage */
    pVEx->VQIH =0;                                                                /*!< Upper 32 bits of integral term (VQI) of q-axis voltage */
    pVEx->VQILH=0;                                                                /*!< Lower 32 bits of integral term (VQI) of q-axis voltage */
    pVEx->IDREF=0;
    pVEx->IQREF=0;
    pVEx->VD=0;
    pVEx->VQ=0;
  }
  else
  {
    int32_t target_speed= MotorControl[channel_number].Omega_Target;
    memset (&MotorControl[channel_number],0,sizeof(MotorControlData));
    MotorControl[channel_number].Omega_Target = target_speed;
  }


  MotorControl[channel_number].Omega.value         = 0;
  MotorControl[channel_number].Omega_command.value = 0;
  MotorControl_Stage_Stop(channel_number);

  MotorControl[channel_number].PerformShutdown     = 0;                                   /* Shutdown has finihed */

  switch (channel_number)
  {
  case 0:
    HAL_Timer_Stop(0);
    break;
  case 1:
    HAL_Timer_Stop(1);
    break;
  case 2:
    HAL_Timer_Stop(2);
    break;
  default:
  assert_param(0);
    break;
  }

  vTaskDelete(MotorControl[channel_number].xShutdownHandle);                                 /* Kill the Task */
}
/*! \brief  Motor Control Worker Task
  *
  * @param  pvParameters: channel number
  * @retval None
*/
void MotorControlTask(void* pvParameters)
{
  long           channel_number = (long) pvParameters;
  static uint8_t state = 0;

  switch (channel_number)
  {
  case 0:
    HAL_Timer_Start(0);
    break;
  case 1:
    HAL_Timer_Start(1);
    break;
  case 2:
    HAL_Timer_Start(2);
    break;
  default:
  assert_param(0);
    break;
  }

  if (channel_number == 0)
    HAL_VE_IRQ(channel_number,VE_IRQ_ON);

  while(1)
  {
    vTaskDelay( 100 / portTICK_RATE_MS );

    if (state!=MotorControl[channel_number].ActualStage.main)
    {
      state = MotorControl[channel_number].ActualStage.main;
      if (MotorControl[channel_number].ActualStage.main == Stage_FOC)
      {
#ifdef USE_LED
        LED_SetState(LED_SIGNAL_VE_RUN_BASE+channel_number,LED_ON);
#endif /* USE_LED */
#ifdef USE_RGB_LED
        RGB_LED_SetValue(LED_RGB_WHITE);
#endif /* USE_RGB_LED */
      }
      else
      {
#ifdef USE_LED
        LED_SetState(LED_SIGNAL_VE_RUN_BASE+channel_number,LED_OFF);
#endif /* USE_LED */
#ifdef USE_RGB_LED
        RGB_LED_RestoreValue();
#endif /* USE_RGB_LED */
      }
    }

    if ((MotorErrorField[channel_number].Error & VE_OVERTEMPERATURE )!=0)
      complete_shutdown(channel_number);

    if (MotorControl[channel_number].PerformShutdown==1)
    {
      switch (SystemValues[channel_number].ShutdownMode)
      {
      case SHUTDOWN_SHORT_BRAKE:
        if (MotorControl[channel_number].ActualStage.main == Stage_Stop)
          complete_shutdown(channel_number);
        else
        {
          MotorControl[channel_number].ActualStage.main = Stage_Brake;                    /* short brake */
          MotorControl[channel_number].ActualStage.sub  = Substage_Step0;
        }
        break;
      case SHUTDOWN_DISABLE_OUTPUT:
        complete_shutdown(channel_number);
        break;
      case SHUTDOWN_GENTLE:
        MotorSetValues[channel_number].TargetSpeed=0;
        if (MotorControl[channel_number].Omega.value == 0)
          complete_shutdown(channel_number);
        break;
      default:
        assert_param(0);
        break;
      }
    }
  }
}

void MotorControl_Loop0 (void)
{
  MotorControl_Loop(0);
}

void MotorControl_Loop1 (void)
{
  MotorControl_Loop(1);
}

void MotorControl_Loop2 (void)
{
  MotorControl_Loop(2);
}

int32_t  cnt =0;
int32_t  d_theta_diff11;
int32_t  d_theta_diff22;
int32_t  d_theta_diff1;
int32_t  Calculated;

/*! \brief  Set motor rotation angle Theta
  *
  * Set theta register with updated value of angle
  *
  * @param  channel_number: channel to work on
  * @param  Theta: angle to be set to theta register
  * @retval None
*/
void MotorControl_SetTheta(unsigned char channel_number,uint16_t Theta)
{
  if (channel_number == 0)
  {
    MotorControl[channel_number].Omega.value=0;
    MotorControl[channel_number].Omega_command.value=0;
    MotorControl[channel_number].Theta.value = Theta << 16;
    TSB_VE0->THETA = Theta;
    MotorControl[channel_number].stop_calculate = 1;

    cnt++;


  int32_t  d_theta_diff1;
  int32_t  d_theta_diff2;
  int32_t  d_theta_diff;

  TurnStateValues[channel_number].ActualCalculatedEncoderValue = (EncoderData[channel_number].FullTurns - StartEncValue[channel_number].FullTurns ) *LINE_ENCODER_RESOLUTION;

  d_theta_diff1= (((int32_t)MotorControl[channel_number].Theta.part.reg) & 0x0000ffff) ;
  d_theta_diff2= (((int32_t)EncoderDataTest[0].Theta0.part.reg ) & 0x0000ffff);

  d_theta_diff = (d_theta_diff2 - d_theta_diff1 ) * LINE_ENCODER_RESOLUTION / FIXPOINT_16 ;

  if (d_theta_diff < 0)
    d_theta_diff += LINE_ENCODER_RESOLUTION;

  if ((d_theta_diff == 0) && (d_theta_diff1 < d_theta_diff2))
    d_theta_diff += LINE_ENCODER_RESOLUTION;

  TurnStateValues[channel_number].ActualCalculatedEncoderValue +=   d_theta_diff;


    if (cnt ==2)
    d_theta_diff11 =d_theta_diff1;


   d_theta_diff11 =d_theta_diff1;
   d_theta_diff22 =d_theta_diff2;
   d_theta_diff1 = d_theta_diff;
   Calculated =TurnStateValues[channel_number].ActualCalculatedEncoderValue;




  }
}

/*! \brief  Handle dynamic change of parameters
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void MotorControl_HandleParameterChange(unsigned char channel_number)
{
  if ((MotorControl[channel_number].ParameterChange & MOTORCONTROL_CHANGE_BOARD_PARAMS_VE) != 0)
  {
    Precalculate_ChannelValues(channel_number);
    Precalculate_MotorValues(channel_number);
    MotorControl[channel_number].ParameterChange &= ~MOTORCONTROL_CHANGE_BOARD_PARAMS_VE;
  }

  if ((MotorControl[channel_number].ParameterChange & MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE) != 0)
  {
    Precalculate_MotorValues(channel_number);
#ifdef HV_COMM
#ifdef USE_SW_OVER_UNDER_VOLTAGE_DETECTION
    HV_Communication_OverUndervoltageDetect(channel_number);
#endif /* USE_SW_OVER_UNDER_VOLTAGE_DETECTION */
#else
    HAL_ADC_OverUndervoltageDetect(channel_number);
#endif /* USE_HV_COMMUNICATION */
#ifdef USE_TEMPERATURE_CONTROL
    TEMPERATURE_ConfigureADCforTemperature(channel_number);
#endif
    MotorControl[channel_number].ParameterChange &= ~MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE;
  }

  if ((MotorControl[channel_number].ParameterChange & MOTORCONTROL_CHANGE_PI_PARAMS) != 0)
  {
    if (channel_number == 0)
        HAL_VE_InitPISettings(channel_number);
    
    MotorControl[channel_number].ParameterChange &= ~MOTORCONTROL_CHANGE_PI_PARAMS;
  }

  if ((MotorControl[channel_number].ParameterChange & MOTORCONTROL_CHANGE_MOTOR_PARAMS) != 0)
  {
    Precalculate_MotorValues(channel_number);
#ifdef USE_ENCODER
    ENC_Init(channel_number);
#endif
    switch (MotorExtendedStateValues[channel_number].ControlMethod)
    {
      case MOTORCONTROL_CONTROL_BY_TORQUE:
        TorqueController_Calculate_Id_Iq_ref(channel_number);
        break;

      case MOTORCONTROL_CONTROL_BY_SPEED:
        SpeedController_Calculate_Id_Iq_ref(channel_number);
        break;
    }

    MotorControl[channel_number].ParameterChange &= ~MOTORCONTROL_CHANGE_MOTOR_PARAMS;
  }
}

/*! \brief  Start the Vector Engine
  *
  * @param  channel_number: channel start
  * @retval Success
*/
int8_t MotorControl_Start(uint8_t channel_number)
{
  while (InitDone==INIT_NOT_DONE)
    vTaskDelay( 100 / portTICK_RATE_MS );
  
  if( CHANNEL_NOT_USED == ChannelUsageConfiguration[channel_number] )
  {
    return -1;
  }

  MotorErrorField[channel_number].Error &= ~(VE_SWOVERVOLTAGE|VE_SWUNDERVOLTAGE|VE_STALLDETECTED);

#ifdef HV_COMM
#ifdef USE_SW_OVER_UNDER_VOLTAGE_DETECTION
  HV_Communication_OverUndervoltageDetect(channel_number);
#endif /* USE_SW_OVER_UNDER_VOLTAGE_DETECTION */
#else
  HAL_ADC_OverUndervoltageDetect(channel_number);
#endif /* HV_COMM */

#ifdef USE_TEMPERATURE_CONTROL
#ifdef HV_COMM
  TEMPERATURE_ConfigureADCforTemperature(channel_number);
#else
  TEMPERATURE_CheckOvertemp(channel_number);
#endif /* HV_COMM */
#endif /* USE_TEMPERATURE_CONTROL */

  if (MotorControl[channel_number].PerformShutdown==1)
  {
    MotorControl[channel_number].PerformShutdown=0;
    MotorSetValues[channel_number].TargetSpeed=MotorControl[channel_number].PerformShutdownTargetSpeed;
    return 0;
  }

  if (MotorControl[channel_number].xVETask!=NULL)
    return 0;

  if ((MotorControl[channel_number].ParameterChange & MOTORCONTROL_CHANGE_BOARD_PARAMS_PMD) != 0)
  {
    HAL_ADC_Init(channel_number,(CURRENT_MEASUREMENT)ChannelValues[channel_number].MeasurementType);
    HAL_PMD_HandleParameterChangeBoard(channel_number);
    MotorControl[channel_number].ParameterChange &= ~MOTORCONTROL_CHANGE_BOARD_PARAMS_PMD;
  }

  if (MotorErrorField[channel_number].Error!=0)
    return -2;

  switch (channel_number)
  {
  case 0:
    HAL_Timer_RegisterIRQCallback(0,MotorControl_Loop0);
    HAL_Timer_Init(0,PERIPHERIAL_CLOCK/2/VE_CONTROL_LOOP_FREQUENCY/1000,PERIPHERIAL_CLOCK/2/VE_CONTROL_LOOP_FREQUENCY/1000);
    HAL_Timer_EnableIRQ(0);
    break;
  case 1:
    HAL_Timer_RegisterIRQCallback(1,MotorControl_Loop1);
    HAL_Timer_Init(1,PERIPHERIAL_CLOCK/2/VE_CONTROL_LOOP_FREQUENCY/1000,PERIPHERIAL_CLOCK/2/VE_CONTROL_LOOP_FREQUENCY/1000);
    HAL_Timer_EnableIRQ(1);
    break;
  case 2:
    HAL_Timer_RegisterIRQCallback(2,MotorControl_Loop2);
    HAL_Timer_Init(2,PERIPHERIAL_CLOCK/2/VE_CONTROL_LOOP_FREQUENCY/1000,PERIPHERIAL_CLOCK/2/VE_CONTROL_LOOP_FREQUENCY/1000);
    HAL_Timer_EnableIRQ(2);
    break;
  }

  Precalculate_ChannelValues(channel_number);                                 /* do pre-calculations */
  Precalculate_MotorValues(channel_number);                                   /* do pre-calculations */
  
#ifdef FIX_155_PMD_DELAY
  if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
  {
    memset(&gPmdRegisterSetting[0][0], 0, sizeof(gPmdRegisterSetting));
  }
#endif
  
  if (channel_number == 0)
    HAL_VE_Init(channel_number);                                                      /* Initialize the vector engine */

  HAL_PMD_HWInit(channel_number);
  MotorControl_Stage_Stop(channel_number);

#ifdef FIX_155_PMD_DELAY
  if(ChannelValues[channel_number].MeasurementType == CURRENT_SHUNT_3)
  {
    gPmdRegisterSetting[channel_number][PMD_NormalOutput_VE_PMD_Output-1] = PMD_NormalOutput_VE_PMD_Output; /* 0x02 */
  }
  else
  {
    HAL_PMD_NormalOutput(channel_number);
    if (channel_number == 0)
      HAL_VE_PMD_Output(channel_number,PMD_ON);
  }
#else
  HAL_PMD_NormalOutput(channel_number);
  if (channel_number == 0)
    HAL_VE_PMD_Output(channel_number,PMD_ON);
#endif

  MotorControl[channel_number].ActualStage.main = Stage_Bootstrap;                        /* startup */
  MotorControl[channel_number].ActualStage.sub  = Substage_Step0;

#ifdef USE_ENCODER
  ENC_Init(channel_number);                                                     /* initialize the Encoder 1 interface */
#endif

  if (xTaskCreate(MotorControlTask,
                  (char const*)"VE",
                  MOTORCONTROL_TASK_STACK_SIZE,
                  (void*)(int)channel_number,
                  MOTORCONTROL_TASK_PRIORITY,
                  &MotorControl[channel_number].xVETask
                  ) != pdPASS)
  {
    dprintf("Can't create MotorControl Task(%d)\n",channel_number);
    return -1;
  }
#ifdef USE_MOTION_CONTROL
  MotorControl[channel_number].IsStarted = 1;
#endif /* USE_MOTION_CONTROL */
  return 0;
}

/*! \brief  Stop the Vector Engine
  *
  * @param  channel_number: channel stop
  * @retval None
*/
int8_t MotorControl_Stop(uint8_t channel_number)
{
  if (MotorControl[channel_number].xVETask==NULL)
    return -1;

  if (MotorErrorField[channel_number].Error!=0)
  {
    MotorControl[channel_number].xShutdownHandle = MotorControl[channel_number].xVETask;
    complete_shutdown(channel_number);
    MotorControl_Stage_Stop(channel_number);
    return 0;
  }

  MotorControl[channel_number].PerformShutdown            = 1;
  MotorControl[channel_number].PerformShutdownTargetSpeed = MotorSetValues[channel_number].TargetSpeed;
  MotorControl[channel_number].xShutdownHandle               = MotorControl[channel_number].xVETask;
#ifdef USE_TURN_CONTROL
  MotorControl[channel_number].do_turn_control = 0;
#endif

#ifdef USE_MOTION_CONTROL
  MotorControl[channel_number].IsStarted = 0;
#endif /* USE_MOTION_CONTROL */
  return 0;
}

/*! \brief  GetControlMethodCommand
  *
  * @param  channel_number: channel
  * @retval currently used control method
*/
uint8_t GetControlMethodCommand(uint8_t channel_number)
{
  return (uint8_t)(MotorExtendedStateValues[channel_number].ControlMethod);
}

/*! \brief  SetControlMethodCommand
  *
  * @param  channel_number: channel
  * @param  control method: requested control method
  * @retval 0: on success
  * @retval 1: on error
*/
uint8_t SetControlMethodCommand(uint8_t channel_number, uint8_t control_method)
{
  uint8_t ret = 1;

  if( MAX_CHANNEL > channel_number )
  {
    if( (MOTORCONTROL_CONTROL_BY_TORQUE == control_method) || (MOTORCONTROL_CONTROL_BY_SPEED == control_method) )
    {
      MotorExtendedStateValues[channel_number].ControlMethod = control_method;
      ret = 0;
    }
  }

  return ret;
}
