/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011 All rights reserved
 */

#include <stdint.h>
#include <stdlib.h>
#include "motor_control.h"
#include "fwmathlib.h"
#include "hal_encoder.h"
#include "txz_driver_def.h"
#include "estimator.h"
#include "sw_foc.h"

/*! \brief  Calculate Id-Ref and Iq-Ref
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void SpeedController_Calculate_Id_Iq_ref(uint8_t channel_number)
{
  int16_t  Omega_Deviation;
  int64_t  Temp_P;
  int64_t  Temp_I;
  uint16_t increase_current;

#ifdef USE_LOAD_DEPENDANT_SPEED_REDUCTION
  int32_t  SpeedRedLimit;
  SpeedRedLimit = MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax * SystemValues[channel_number].SpeedReduction / 100;
#endif /* USE_LOAD_DEPENDANT_SPEED_REDUCTION */

  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Stop:
  case Stage_Bootstrap:
  case Stage_Brake:
  case Stage_ZeroCurrentMeasure:
  case Stage_Emergency:
    MotorControl[channel_number].Id_reference = 0;
    MotorControl[channel_number].Iq_reference = 0;
    break;

  case Stage_Initposition:
    MotorControl[channel_number].Id_reference = 0;

    /* set current "Iq" increasing as C-load */
    increase_current = (((abs(MotorControl[channel_number].PreCalculation.IqCurrentForInitposition)<<16))
                           / ((MotorControl[channel_number].PreCalculation.WaitTime_Position)/2))>>16;
    /* check minimized increase */
    if (increase_current==0)
      increase_current=1;

    if (MotorSetValues[channel_number].TargetSpeed >= 0)
      MotorControl[channel_number].Iq_reference += increase_current;
    else
      MotorControl[channel_number].Iq_reference -= increase_current;

    /* normalizing and limiting this values to maximum current */
    if(MotorControl[channel_number].Iq_reference   >= MotorControl[channel_number].PreCalculation.IqCurrentForInitposition)
      MotorControl[channel_number].Iq_reference  = (int16_t)(MotorControl[channel_number].PreCalculation.IqCurrentForInitposition);
    else if(MotorControl[channel_number].Iq_reference <= -MotorControl[channel_number].PreCalculation.IqCurrentForInitposition)
      MotorControl[channel_number].Iq_reference     = -(int16_t)(MotorControl[channel_number].PreCalculation.IqCurrentForInitposition);

    break;

  case Stage_Force:
    if (MotorControl[channel_number].Omega_command.part.reg >= 0)
      MotorControl[channel_number].Iq_reference = MotorControl[channel_number].PreCalculation.IqCurrentForInitposition;
    else
      MotorControl[channel_number].Iq_reference = -MotorControl[channel_number].PreCalculation.IqCurrentForInitposition;


    MotorControl[channel_number].Iq_reference_I.part.reg  = MotorControl[channel_number].Iq_reference;
    break;

  case Stage_FOC:
    /* calculate the speed deviation\n
    * Omega(Dev) = Omega(command) - Omega(calculated) */
    Omega_Deviation = (int16_t)((MotorControl[channel_number].Omega_command.value - MotorControl[channel_number].Omega.value)>>16);
    /* normalize the deviation to maximum speed and frequency based on current */
    Temp_I = (int64_t)Omega_Deviation * MotorControl[channel_number].PreCalculation.SpeedKiPreCalculation;

    MotorControl[channel_number].Iq_reference_I.value += Temp_I;                          /* integrate to Iq(reference) */

    /* keep this in range */
    if(MotorControl[channel_number].Iq_reference_I.value > (MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax<<16))
      MotorControl[channel_number].Iq_reference_I.value  = (MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax<<16);
    else if(MotorControl[channel_number].Iq_reference_I.value < -(MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax<<16))
      MotorControl[channel_number].Iq_reference_I.value       = -(MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax<<16);

    /* proportional part of the control algorithm normalized on base of current */
    Temp_P  = (int64_t)Omega_Deviation * MotorControl[channel_number].PreCalculation.SpeedKpPreCalculation;

    /* add the proportional part and and divide for correction */
    MotorControl[channel_number].Iq_reference = (int16_t)((MotorControl[channel_number].Iq_reference_I.value + Temp_P)>>16);
    MotorControl[channel_number].Id_reference = 0;

#ifdef USE_LOAD_DEPENDANT_SPEED_REDUCTION
    if( MotorControl[channel_number].Iq_reference > SpeedRedLimit)
    {
      MotorSetValues[channel_number].TargetSpeed = MotorExtendedStateValues[channel_number].ActualControlledParameter;
      MotorErrorField[channel_number].Error |= VE_SPEEDREDUCTION;
    }
    else if( MotorControl[channel_number].Iq_reference < -SpeedRedLimit)
    {
      MotorSetValues[channel_number].TargetSpeed = MotorExtendedStateValues[channel_number].ActualControlledParameter;
      MotorErrorField[channel_number].Error |= VE_SPEEDREDUCTION;
    }
    else
    {
      uint8_t turn_control = 0;
      uint8_t motion_control = 0;
#ifdef USE_TURN_CONTROL
      turn_control = (MotorControl[channel_number].do_turn_control == 1);
#endif
#ifdef USE_MOTION_CONTROL
      motion_control = (MotorControl[channel_number].do_pulses_control == 1);
#endif
          
      if ((turn_control != 1) && (motion_control != 1))
      {
        if ( MotorControl[channel_number].PerformShutdown != 1)
        {
            MotorSetValues[channel_number].TargetSpeed = MotorControl[channel_number].TargetSpeed_Backup;
            MotorErrorField[channel_number].Error &= ~VE_SPEEDREDUCTION;
        }
        else
          MotorSetValues[channel_number].TargetSpeed = 0;
      }
    }
#endif /* USE_LOAD_DEPENDANT_SPEED_REDUCTION */

    /* normalizing and limiting this values to maximum current */
    if(MotorControl[channel_number].Iq_reference   >= MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax)
      MotorControl[channel_number].Iq_reference  = (int16_t)(MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax);
    else if(MotorControl[channel_number].Iq_reference <= -MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax)
      MotorControl[channel_number].Iq_reference     = -(int16_t)(MotorControl[channel_number].PreCalculation.IqLimit_normed_to_amax);

    break;

  default:
    assert_param(0);
    break;
  }
}

/*! \brief  Calculate or set Omeag and Theta
  *
  * @param  channel_number: channel to work on
  * @retval None
*/
void SpeedController_Omega_Theta(uint8_t channel_number)
{
  uint8_t direction;
  static uint8_t initposition_angle_determin = 0;

  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Stop:
  case Stage_Bootstrap:
  case Stage_Brake:
    initposition_angle_determin = 0;
    break;
  case Stage_Initposition:
    if ( initposition_angle_determin == 0)
      initposition_angle_determin = 1;
    {

      direction = MotorControl[channel_number].Omega_Target >= 0 ;

#ifdef USE_TURN_CONTROL      
      if (MotorControl[channel_number].do_turn_control == 1)
      {
        //Direction check is removed to support Turn control in CCW,
        //may not be needed in Initposition stage
        {
          MotorControl[channel_number].LastDirection = direction;
          if (MotorControl[channel_number].LastDirection == 1)
          {
            MotorControl[channel_number].Theta.part.reg  = (int16_t)((int64_t)-90 * FIXPOINT_16 / 360);
            MotorControl[channel_number].Theta.part.frac = 0;        
          }
          else
          {
            MotorControl[channel_number].Theta.part.reg  = (uint16_t)(((uint64_t)90 * FIXPOINT_16 / 360));
            MotorControl[channel_number].Theta.part.frac = 0;  
          }
        }
      }
      else
#endif
      {
        if (MotorControl[channel_number].LastDirection != direction)
        {
          MotorControl[channel_number].LastDirection = direction;
          if (MotorControl[channel_number].LastDirection == 1)
          {
            MotorControl[channel_number].Theta.part.reg  = (int16_t)((int64_t)-90 * FIXPOINT_16 / 360);
            MotorControl[channel_number].Theta.part.frac = 0;
#ifdef USE_ENCODER          
            if (MotorParameterValues[channel_number].Encoder != ENCODER_NO_ENCODER)
              MotorControl[channel_number].Theta.part.reg = ((ENC_GetTheta(channel_number)) * FIXPOINT_16 / 360);
#endif /* USE_ENCODER */        
          }
          else
          {
            MotorControl[channel_number].Theta.part.reg  = (uint16_t)(((uint64_t)90 * FIXPOINT_16 / 360));
            MotorControl[channel_number].Theta.part.frac = 0;
#ifdef USE_ENCODER          
            if (MotorParameterValues[channel_number].Encoder != ENCODER_NO_ENCODER)
              MotorControl[channel_number].Theta.part.reg = ((ENC_GetTheta(channel_number)) * FIXPOINT_16 / 360);
#endif /* USE_ENCODER */  
          }
        }
      }
     
      EncoderData[channel_number].Theta0.part.reg = MotorControl[channel_number].Theta.part.reg;
    }
    break;

  case Stage_Force:
    initposition_angle_determin = 0;
    if (MotorControl[channel_number].Omega.part.reg !=0)
    {
      direction = MotorControl[channel_number].Omega_command.part.reg > 0;
      if  (MotorControl[channel_number].LastDirection!=direction)
      {
        MotorControl[channel_number].LastDirection=direction;
        MotorControl[channel_number].Theta.part.reg +=  FIXPOINT_16 / 2;
        if (channel_number == 0)
        {
          long long temp;
          temp = ( ((long long)TSB_VE0->VDIH) << 32) | TSB_VE0->VDILH;
          temp *= -1;
          TSB_VE0->VDIH = (temp >> 32);
          TSB_VE0->VDILH = temp & 0x00000000ffffffffL;
          temp = ( ((long long)TSB_VE0->VQIH) << 32) | TSB_VE0->VQILH;
          temp *= -1;
          TSB_VE0->VQIH = (temp >> 32);
          TSB_VE0->VQILH = temp & 0x00000000ffffffffL;
        }
        else
        {
          SW_FOC_Invert_Current_Intergral_Value(channel_number);
        }
      }
    }
    Estimator_Calculate_Omega_BLDC(channel_number);
    MotorControl[channel_number].Omega.value  = MotorControl[channel_number].Omega_command.value;
    MotorControl[channel_number].Theta.value += (int32_t) ( MotorControl[channel_number].Omega.part.reg
                                             *( MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency<<1));
    break;

  case Stage_FOC:
    initposition_angle_determin = 0;
#ifdef USE_ENCODER
    if ((MotorParameterValues[channel_number].Encoder != ENCODER_NO_ENCODER) && (MotorParameterValues[channel_number].EncoderUsage == ENCODER_USAGE_SPEED))
      ENC_Determine_Omega_Theta(channel_number);
    else
#endif /* USE_ENCODER */
    {
    Estimator_Calculate_Omega_BLDC(channel_number);
    MotorControl[channel_number].Theta.value += (int32_t) ( MotorControl[channel_number].Omega.part.reg
                                                 *( MotorControl[channel_number].PreCalculation.HzMax_normed_to_PwmFrequency<<1));
    }
    break;

  default:
    assert_param(0);
    break;
  }
}
