/* THE SOURCE CODE AND ITS
* RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
* WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
* AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
* INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
* DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
* INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
* MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
* TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
* LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
* FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
* LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
* OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
* ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
* ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
* FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
* OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2019 All rights reserved
*/

#include <stdint.h>
#include <cmsis_iccarm.h>
#include <stdlib.h>
#include "fwmathlib.h"
#include "api.h"
#include "motor_control.h"
#include "hal_pmd.h"
#include "txz_driver_def.h"
#include "TMPM4Ky_adc.h"
#include "speed_controller.h"
#include "torque_controller.h"
#include "performance_measurement.h"

/***********************************************************************************************
* @brief		SW_FOC_SetPMDData
*
* @param                uint8_t channel_number                                  channel of control
* @param                uint16_t duty_u                                         U-phase PWM duty ratio
* @param                uint16_t duty_v                                         V-phase PWM duty ratio
* @param                uint16_t duty_w                                         W-phase PWM duty ratio
* @param                int16_t  adtrg0                                         AD trigger timing 0 position
* @param                int16_t  adtrg1                                         AD trigger timing 1 position
*
* @return		none
**********************************************************************************************/
static void SW_FOC_SetPMDData(uint8_t channel_number, uint16_t duty_u, uint16_t duty_v, uint16_t duty_w, int16_t adtrg0, int16_t adtrg1)
{
  int32_t trgtmp0, trgtmp1;
  TSB_PMD_TypeDef* PMDx;

  switch (channel_number)
  {
  case 0:
    PMDx = TSB_PMD0;
    break;
  case 1:
    PMDx = TSB_PMD1;
    break;
  case 2:
    PMDx = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  /* TRGCMPx = 0 to 0x7FFF */
  trgtmp0 = ((int32_t)adtrg0 + 0x8000) / 2;
  if (trgtmp0 >= 0x8000) trgtmp0 = 0x7fff;
  trgtmp1 = ((int32_t)adtrg1 + 0x8000) / 2;
  if (trgtmp1 >= 0x8000) trgtmp1 = 0x7fff;

  PMDx->TRGCMP0 = trgtmp0;
  PMDx->TRGCMP1 = trgtmp1;

  /* When <CMPx> >= 0x8000, the duty ratio is 100%.*/
  PMDx->CMPU = duty_u;
  PMDx->CMPV = duty_v;
  PMDx->CMPW = duty_w;
}

/**
 *********************************************************************************************
 * @brief		Initialize deadtime control parameter
 *
 * @param		deadtime_t* const	_deadtime			The structure address for deadtime control.
 * @param		Deadtime_InitTypeDef* const	_initdata	Initial data
 *
 * @return		none
 *********************************************************************************************
 */
void init_deadtime(uint8_t channel_number, deadtime_t* const _deadtime, Deadtime_InitTypeDef* const _initdata)
{
	MotorControl[channel_number].Deadtime.filter		= _initdata->filter;
	MotorControl[channel_number].Deadtime.max_value	= _initdata->max_value;
	MotorControl[channel_number].Deadtime.current		= _initdata->current;
}


/**
 *********************************************************************************************
 * @brief		Deadtime control
 *
 * @param		deadtime_t* const	_deadtime	The structure address for deadtime control.
 * @param		int16_t _ia						U-Phase current
 * @param		int16_t _ib						V-Phase current
 * @param		int16_t _ic						W-Phase current
 * @param		uint8_t _idet_err				Current detection error determination result
 *
 * @return		none
 *********************************************************************************************
 */
void SW_FOC_Deadtime_Control(uint8_t channel_number, int16_t _ia, int16_t _ib, int16_t _ic, uint8_t _idet_err)
{
	/* average current calculation */
	MotorControl[channel_number].Deadtime.ia_ave.value += (((_ia << 16) - MotorControl[channel_number].Deadtime.ia_ave.value) >> MotorControl[channel_number].Deadtime.filter);
	MotorControl[channel_number].Deadtime.ib_ave.value += (((_ib << 16) - MotorControl[channel_number].Deadtime.ib_ave.value) >> MotorControl[channel_number].Deadtime.filter);
	MotorControl[channel_number].Deadtime.ic_ave.value += (((_ic << 16) - MotorControl[channel_number].Deadtime.ic_ave.value) >> MotorControl[channel_number].Deadtime.filter);

	/* check detect current error */
	if (_idet_err == 0)
	{
		MotorControl[channel_number].Deadtime.iu_dt = MotorControl[channel_number].Deadtime.ia_ave.part.reg;
		MotorControl[channel_number].Deadtime.iv_dt = MotorControl[channel_number].Deadtime.ib_ave.part.reg;
		MotorControl[channel_number].Deadtime.iw_dt = MotorControl[channel_number].Deadtime.ic_ave.part.reg;
	}

	/* U-phase compensation operation */
	MotorControl[channel_number].Deadtime.u_duty = (MotorControl[channel_number].Deadtime.iu_dt * MotorControl[channel_number].Deadtime.max_value) / (int32_t)MotorControl[channel_number].Deadtime.current;
	if (MotorControl[channel_number].Deadtime.u_duty >= MotorControl[channel_number].Deadtime.max_value)
	{
		MotorControl[channel_number].Deadtime.u_duty = MotorControl[channel_number].Deadtime.max_value;
	}
	else if (MotorControl[channel_number].Deadtime.u_duty <= -MotorControl[channel_number].Deadtime.max_value)
	{
		MotorControl[channel_number].Deadtime.u_duty = -MotorControl[channel_number].Deadtime.max_value;
	}

	/* V-phase compensation operation */
	MotorControl[channel_number].Deadtime.v_duty = (MotorControl[channel_number].Deadtime.iv_dt * MotorControl[channel_number].Deadtime.max_value) / (int32_t)MotorControl[channel_number].Deadtime.current;
	if (MotorControl[channel_number].Deadtime.v_duty >= MotorControl[channel_number].Deadtime.max_value)
	{
		MotorControl[channel_number].Deadtime.v_duty = MotorControl[channel_number].Deadtime.max_value;
	}
	else if (MotorControl[channel_number].Deadtime.v_duty <= -MotorControl[channel_number].Deadtime.max_value)
	{
		MotorControl[channel_number].Deadtime.v_duty = -MotorControl[channel_number].Deadtime.max_value;
	}

	/* W-phase compensation operation */
	MotorControl[channel_number].Deadtime.w_duty = (MotorControl[channel_number].Deadtime.iw_dt * MotorControl[channel_number].Deadtime.max_value) / (int32_t)MotorControl[channel_number].Deadtime.current;
	if (MotorControl[channel_number].Deadtime.w_duty >= MotorControl[channel_number].Deadtime.max_value)
	{
		MotorControl[channel_number].Deadtime.w_duty = MotorControl[channel_number].Deadtime.max_value;
	}
	else if (MotorControl[channel_number].Deadtime.w_duty <= -MotorControl[channel_number].Deadtime.max_value)
	{
		MotorControl[channel_number].Deadtime.w_duty = -MotorControl[channel_number].Deadtime.max_value;
	}
}

/**********************************************************************************************
* @brief                Set Trigger select Register.
*
* @param                uint8_t channel_number                                  channel of control
* @param                uint32_t _trgsel                                        Trigger Select
*
* @return		none
*********************************************************************************************/
static void SW_FOC_SetPMDTriggerSelect(uint8_t channel_number, uint32_t _trgsel)
{
  TSB_PMD_TypeDef* PMDx;

  switch (channel_number)
  {
  case 0:
    PMDx = TSB_PMD0;
    break;
  case 1:
    PMDx = TSB_PMD1;
    break;
  case 2:
    PMDx = TSB_PMD2;
    break;
  default:
    assert_param(0);
    break;
  }

  PMDx->TRGSEL = _trgsel;
}

/***********************************************************************************************
* @brief		SW_FOC_GetZeroCurrent
*
* @param		channel_number
*
* @return		none
**********************************************************************************************/
void SW_FOC_GetZeroCurrent(uint8_t channel_number)
{
  TSB_AD_TypeDef* ADx;

  switch (channel_number)
  {
  case 0:
    ADx= TSB_ADA;
    break;
  case 1:
    ADx= TSB_ADB;
    break;
  case 2:
    ADx= TSB_ADC;
    break;
  default:
    assert_param(0);
    break;
  }

  switch (ChannelValues[channel_number].MeasurementType)
  {
  case CURRENT_SHUNT_1:
    assert_param(0);
    break;

  case CURRENT_SHUNT_3:
    {
      ADC_Result result;

      switch (channel_number)
      {
      case 0:
        ADC_SetSWTrg(ADx, ADC_REG0, TRG_ENABLE(AIN0_3PHASE_U));
        ADC_SetSWTrg(ADx, ADC_REG1, TRG_ENABLE(AIN0_3PHASE_V));
        ADC_SetSWTrg(ADx, ADC_REG2, TRG_ENABLE(AIN0_3PHASE_W));
        ADC_Start(ADx, ADC_TRG_SINGLE);
        break;
      case 1:
        ADC_SetSWTrg(ADx, ADC_REG0, TRG_ENABLE(AIN1_3PHASE_U));
        ADC_SetSWTrg(ADx, ADC_REG1, TRG_ENABLE(AIN1_3PHASE_V));
        ADC_SetSWTrg(ADx, ADC_REG2, TRG_ENABLE(AIN1_3PHASE_W));
        ADC_Start(ADx, ADC_TRG_SINGLE);
        break;
      case 2:
        ADC_SetSWTrg(ADx, ADC_REG0, TRG_ENABLE(AIN2_3PHASE_U));
        ADC_SetSWTrg(ADx, ADC_REG1, TRG_ENABLE(AIN2_3PHASE_V));
        ADC_SetSWTrg(ADx, ADC_REG2, TRG_ENABLE(AIN2_3PHASE_W));
        ADC_Start(ADx, ADC_TRG_SINGLE);
        break;
      default:
        assert_param(0);
        break;
      }

      while (ADC_GetConvertState(ADx,ADC_TRG_SINGLE)==BUSY);

      result = ADC_GetConvertResult(ADx, ADC_REG0);
      MotorControl[channel_number].Ia0 = result.Bit.ADResult<<4;
      result = ADC_GetConvertResult(ADx, ADC_REG1);
      MotorControl[channel_number].Ib0 = result.Bit.ADResult<<4;
      result = ADC_GetConvertResult(ADx, ADC_REG2);
      MotorControl[channel_number].Ic0 = result.Bit.ADResult<<4;
    }
    break;

  case CURRENT_SENSOR_2:
    assert_param(0); /* not implemented */
    break;
  }
}

/**
*********************************************************************************************
* @brief		SW_FOC_InputProcessing
*
* @param		channel_number
*
* @return		none
*********************************************************************************************
*/
static void SW_FOC_InputProcessing(uint8_t channel_number)
{
  TSB_AD_TypeDef* ADx;

  ADC_Result  ADxREG0,ADxREG1,ADxREG2,ADxREG3;

  switch (channel_number)
  {
  case 0:
    ADx= TSB_ADA;
    break;
  case 1:
    ADx= TSB_ADB;
    break;
  case 2:
    ADx= TSB_ADC;
    break;
  default:
    assert_param(0);
    break;
  }

  ADxREG0 = ADC_GetConvertResult(ADx, ADC_REG0);
  ADxREG1 = ADC_GetConvertResult(ADx, ADC_REG1);
  ADxREG2 = ADC_GetConvertResult(ADx, ADC_REG2);
  ADxREG3 = ADC_GetConvertResult(ADx, ADC_REG3);

  MotorControl[channel_number].Vdc = ADxREG3.Bit.ADResult<<3;

#ifdef BOARD_PWR_HEADER_FILE_0
#include BOARD_PWR_HEADER_FILE_0
#endif
#ifdef BOARD_VDC_CHANNEL_0_VALUE                                                /* If there is no measurement channel for VDC, we have to give a constant value */
  if (channel_number==0)
  {
    int8_t unit = MotorControl[channel_number].v_max.unit-BOARD_VDC_CHANNEL_0_UNIT;
    if (unit >= 0)                                                              /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      MotorControl[channel_number].Vdc =(0xfff<<3)*BOARD_VDC_CHANNEL_0_VALUE/MotorControl[channel_number].v_max.value/pow10(unit);
    else                                                                        /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      MotorControl[channel_number].Vdc =(0xfff<<3)*BOARD_VDC_CHANNEL_0_VALUE/MotorControl[channel_number].v_max.value*pow10(-unit);
  }
#endif
#include "pwr_undefine.h"

#ifdef BOARD_PWR_HEADER_FILE_1
#include BOARD_PWR_HEADER_FILE_1
#endif /* BOARD_PWR_HEADER_FILE_1 */
#ifdef BOARD_VDC_CHANNEL_1_VALUE                                                /* If there is no measurement channel for VDC, we have to give a constant value */
  if (channel_number==1)
  {
    int8_t unit = MotorControl[channel_number].v_max.unit-BOARD_VDC_CHANNEL_1_UNIT;
    if (unit >= 0)                                                              /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      MotorControl[channel_number].Vdc =(0xfff<<3)*BOARD_VDC_CHANNEL_1_VALUE/MotorControl[channel_number].v_max.value/pow10(unit);
    else                                                                        /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      MotorControl[channel_number].Vdc =(0xfff<<3)*BOARD_VDC_CHANNEL_1_VALUE/MotorControl[channel_number].v_max.value*pow10(-unit);
  }
#endif
#include "pwr_undefine.h"

#ifdef BOARD_PWR_HEADER_FILE_2
#include BOARD_PWR_HEADER_FILE_2
#endif /* BOARD_PWR_HEADER_FILE_2 */
#ifdef BOARD_VDC_CHANNEL_2_VALUE                                                /* If there is no measurement channel for VDC, we have to give a constant value */
  if (channel_number==2)
  {
    int8_t unit = MotorControl[channel_number].v_max.unit-BOARD_VDC_CHANNEL_2_UNIT;
    if (unit >= 0)                                                              /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      MotorControl[channel_number].Vdc =(0xfff<<3)*BOARD_VDC_CHANNEL_2_VALUE/MotorControl[channel_number].v_max.value/pow10(unit);
    else                                                                        /* ADC valus is 12 bit - in bits 15-3 - max is 0xfff - blown up to right position */
      MotorControl[channel_number].Vdc =(0xfff<<3)*BOARD_VDC_CHANNEL_2_VALUE/MotorControl[channel_number].v_max.value*pow10(-unit);
  }
#endif
#include "pwr_undefine.h"

  switch (ChannelValues[channel_number].MeasurementType)
  {
  case CURRENT_SHUNT_1:
    assert_param(0);
    break;

  case CURRENT_SHUNT_3:
    /* Get phase current */
    switch(MotorControl[channel_number].Sector1)
    {
    case 0:
    case 5:
      /* Sector 0,5 */
      if (ChannelValues[channel_number].MeasurementDirection==CURRENT_MEASUREMENT_INVERTED)
      {
        MotorControl[channel_number].Ia_adc = -(int16_t)(ADxREG2.Bit.ADResult<<4);
        MotorControl[channel_number].Ib_adc = -(int16_t)(ADxREG0.Bit.ADResult<<4);
        MotorControl[channel_number].Ic_adc = -(int16_t)(ADxREG1.Bit.ADResult<<4);
      }
      else
      {
        MotorControl[channel_number].Ia_adc = (int16_t)(ADxREG2.Bit.ADResult<<4);
        MotorControl[channel_number].Ib_adc = (int16_t)(ADxREG0.Bit.ADResult<<4);
        MotorControl[channel_number].Ic_adc = (int16_t)(ADxREG1.Bit.ADResult<<4);
      }
      MotorControl[channel_number].Ib_raw = (int16_t)(MotorControl[channel_number].Ib0 - MotorControl[channel_number].Ib_adc);
      MotorControl[channel_number].Ic_raw = (int16_t)(MotorControl[channel_number].Ic0 - MotorControl[channel_number].Ic_adc);
      MotorControl[channel_number].Ia_raw = (int16_t)(-MotorControl[channel_number].Ib_raw - MotorControl[channel_number].Ic_raw);
      break;
    case 1:
    case 2:
      /* Sector1,2 */
      if (ChannelValues[channel_number].MeasurementDirection==CURRENT_MEASUREMENT_INVERTED)
      {
        MotorControl[channel_number].Ia_adc = -(int16_t)(ADxREG1.Bit.ADResult<<4);
        MotorControl[channel_number].Ib_adc = -(int16_t)(ADxREG2.Bit.ADResult<<4);
        MotorControl[channel_number].Ic_adc = -(int16_t)(ADxREG0.Bit.ADResult<<4);
      }
      else
      {
        MotorControl[channel_number].Ia_adc = (int16_t)(ADxREG1.Bit.ADResult<<4);
        MotorControl[channel_number].Ib_adc = (int16_t)(ADxREG2.Bit.ADResult<<4);
        MotorControl[channel_number].Ic_adc = (int16_t)(ADxREG0.Bit.ADResult<<4);
      }
      MotorControl[channel_number].Ia_raw = (int16_t)(MotorControl[channel_number].Ia0 - MotorControl[channel_number].Ia_adc);
      MotorControl[channel_number].Ic_raw = (int16_t)(MotorControl[channel_number].Ic0 - MotorControl[channel_number].Ic_adc);
      MotorControl[channel_number].Ib_raw = (int16_t)(-MotorControl[channel_number].Ic_raw - MotorControl[channel_number].Ia_raw);
    break;
    case 3:
    case 4:
      /* Sector3,4 */
      if (ChannelValues[channel_number].MeasurementDirection==CURRENT_MEASUREMENT_INVERTED)
      {
        MotorControl[channel_number].Ia_adc = -(int16_t)(ADxREG0.Bit.ADResult<<4);
        MotorControl[channel_number].Ib_adc = -(int16_t)(ADxREG1.Bit.ADResult<<4);
        MotorControl[channel_number].Ic_adc = -(int16_t)(ADxREG2.Bit.ADResult<<4);
      }
      else
      {
        MotorControl[channel_number].Ia_adc = (int16_t)(ADxREG0.Bit.ADResult<<4);
        MotorControl[channel_number].Ib_adc = (int16_t)(ADxREG1.Bit.ADResult<<4);
        MotorControl[channel_number].Ic_adc = (int16_t)(ADxREG2.Bit.ADResult<<4);
      }
      MotorControl[channel_number].Ia_raw = (int16_t)(MotorControl[channel_number].Ia0 - MotorControl[channel_number].Ia_adc);
      MotorControl[channel_number].Ib_raw = (int16_t)(MotorControl[channel_number].Ib0 - MotorControl[channel_number].Ib_adc);
      MotorControl[channel_number].Ic_raw = (int16_t)(-MotorControl[channel_number].Ia_raw - MotorControl[channel_number].Ib_raw);
      break;
    default:
      break;
    }
  break;
  default:
    assert_param(0);
    break;
  }

  if ((MotorControl[channel_number].ActualStage.main == Stage_Stop) || (MotorControl[channel_number].idetect_error == 0))
  {
    MotorControl[channel_number].Ia = MotorControl[channel_number].Ia_raw;
    MotorControl[channel_number].Ib = MotorControl[channel_number].Ib_raw;
    MotorControl[channel_number].Ic = MotorControl[channel_number].Ic_raw;
  }
}

/**
*********************************************************************************************
* @brief		SW_FOC_InputCurrentConversion
*
* @param		vector_t* const	_motor			The structure address for motor vector control.
*
* @return		none
*********************************************************************************************
*/
static void SW_FOC_InputCurrentConversion(uint8_t channel_number)
{
  if (MotorControl[channel_number].ActualStage.main >= Stage_Initposition)
  {
    /* When the current detection can not be performed normally, conversion is not performed and Id and Iq are not updated. */
    if (MotorControl[channel_number].idetect_error == 0)
    {
      /* abc/alpha_beta conversion (Clarke.t) */
      ClarkeTransformation(MotorControl[channel_number].Ia, MotorControl[channel_number].Ib, MotorControl[channel_number].Ic, &MotorControl[channel_number].Ialpha, &MotorControl[channel_number].Ibeta);
      ParkTransformation  (MotorControl[channel_number].Ialpha, MotorControl[channel_number].Ibeta, MotorControl[channel_number].Theta.part.reg, &MotorControl[channel_number].Id, &MotorControl[channel_number].Iq);
    }
  }
  else
  {
    MotorControl[channel_number].Id = 0;
    MotorControl[channel_number].Iq = 0;
  }
}

/***********************************************************************************************
* @brief		SW_FOC_CurrentPIControl
*
* @param		vector_t* const	_motor			The structure address for motor vector control.
*
* @return		none
**********************************************************************************************/
static void SW_FOC_CurrentPIControl(uint8_t channel_number,int32_t measured_id, int32_t measured_iq, int32_t* vd , int32_t* vq)
{
  int64_t deltaId,deltaIq;

  deltaId = MotorControl[channel_number].Id_reference - measured_id;                                      // Difference between current reference value and current feedback
  MotorControl[channel_number].Vdi = PIControl[channel_number].PiIdKi * deltaId + MotorControl[channel_number].Vdi;       // Integral term computation
  *vd = PIControl[channel_number].PiIdKp * deltaId + MotorControl[channel_number].Vdi;                       // Voltage calculation using proportional term

  deltaIq = MotorControl[channel_number].Iq_reference - measured_iq;                                                           // Difference between current reference value and
  MotorControl[channel_number].Vqi = PIControl[channel_number].PiIqKi * deltaIq + MotorControl[channel_number].Vqi;       // Integral term computation
  *vq = PIControl[channel_number].PiIqKp * deltaIq + MotorControl[channel_number].Vqi;                       // Voltage calculation using proportional term
}

void SW_FOC_Invert_Current_Intergral_Value(uint8_t channel_number)
{
  MotorControl[channel_number].Vqi *=-1;
}


/***********************************************************************************************
* @brief		FOC_Control_Current
*
* @param		vector_t* const	_motor			The structure address for motor vector control.
*
* @return		none
**********************************************************************************************/
static void SW_FOC_Control_Current(uint8_t channel_number)
{

  switch (MotorControl[channel_number].ActualStage.main)
  {
  case Stage_Emergency:
  case Stage_ZeroCurrentMeasure:
  case Stage_Bootstrap:
  case Stage_Brake:
  case Stage_Stop:
    break;
  case Stage_Initposition:
  case Stage_Force:
  case Stage_FOC:
    SW_FOC_CurrentPIControl(channel_number,MotorControl[channel_number].Id, MotorControl[channel_number].Iq, &MotorControl[channel_number].Vd.value , &MotorControl[channel_number].Vq.value);
    break;
  default:
    assert_param(0);
    break;
  }

}

/***********************************************************************************************
* @brief		SW_FOC_CalSector
*
* @param		int32_t 		_valpha		alpha-axis Valpha
* @param		int32_t 		_vbeta		beta-axis Vbeta
*
* @return		sector
**********************************************************************************************/
static uint8_t SW_FOC_CalSector(int32_t _valpha, int32_t _vbeta)
{
#define	cCAL_1_DIV_SQRT3	(int32_t)(FIXPOINT_15 * 1 / 1.73205f)	/* Q15 1/sqrt3 */

  uint8_t sector = 0;
  int32_t temp_vbeta = ((((int64_t)abs(_vbeta)) * cCAL_1_DIV_SQRT3) >> 15);

  if ((_valpha >= 0) && (_vbeta >= 0))
    sector = (_valpha >= temp_vbeta)? 0: 1;
  else if ((_valpha < 0) && (_vbeta >= 0))
    sector = (abs(_valpha) < temp_vbeta)? 1: 2;
  else if ((_valpha < 0) && (_vbeta < 0))
    sector = (abs(_valpha) < temp_vbeta)? 4: 3;
  else if ((_valpha >= 0) && (_vbeta < 0))
    sector = (_valpha < temp_vbeta)? 4: 5;
    
  return(sector);
}


/***********************************************************************************************
* @brief		FOC_OutputTransformation
*
* @param		vector_t* const	_motor			The structure address for motor vector control.
*
* @return		none
**********************************************************************************************/
static void SW_FOC_OutputTransformation(uint8_t channel_number)
{
  {
    /*
    It converts from dq-axis voltage (Vd, Vq) to alpha_beta-axis voltage (Valpha, Vbeta) by reverse park conversion.
    */
    InvParkTransformation(MotorControl[channel_number].Vd.value, MotorControl[channel_number].Vq.value, MotorControl[channel_number].Theta.part.reg, &MotorControl[channel_number].Valpha.value, &MotorControl[channel_number].Vbeta.value);
    /*
    Calculate sectors from alpha_beta axis voltage (Valpha, Vbeta).
    Sectors before calculation are saved in the last sector value and used for motor current acquisition.
    */
    MotorControl[channel_number].Sector1 = MotorControl[channel_number].Sector;
    MotorControl[channel_number].Sector = SW_FOC_CalSector(MotorControl[channel_number].Valpha.value, MotorControl[channel_number].Vbeta.value);

    /*
    Convert from alpha_beta axis voltage (Valpha, Vbeta) to 3-phase duty (DutyV, DutyW, DutyW) by space vector modulation or reverse clark conversion.
    */
    SpaceVectorModulation(MotorControl[channel_number].Valpha.value, MotorControl[channel_number].Vbeta.value, MotorControl[channel_number].Vdc, MotorControl[channel_number].Sector, 0 /* Modulation - 0-> 3 Phase */,
                          (uint16_t *)&MotorControl[channel_number].DutyU, (uint16_t *)&MotorControl[channel_number].DutyV, (uint16_t *)&MotorControl[channel_number].DutyW);

  }
}

/***********************************************************************************************
* @brief		FOC_CalTrgTiming
*
* @param		vector_t* const	_motor			The structure address for motor vector control.
*
* @return		none
**********************************************************************************************/
static void SW_FOC_TriggerGeneration(uint8_t channel_number)
{
  uint32_t  dmin, dmid; //dmax;

  switch (ChannelValues[channel_number].MeasurementType)
  {
  case CURRENT_SHUNT_1:
    dmid = MediumOf3(MotorControl[channel_number].DutyU,
                     MotorControl[channel_number].DutyV,
                     MotorControl[channel_number].DutyW);
    dmin = MinimumOf3(MotorControl[channel_number].DutyU,
                      MotorControl[channel_number].DutyV,
                      MotorControl[channel_number].DutyW);

    if (MotorControl[channel_number].modulation == 0)
    {
      /* 3phase modulation */
      MotorControl[channel_number].AdTrg0 = (-dmid) + MotorControl[channel_number].TrgComp;
      MotorControl[channel_number].AdTrg1 = (-dmin) + MotorControl[channel_number].TrgComp;
    }
    else
    {
      /* 2phase modulation */
      MotorControl[channel_number].AdTrg0 = (-dmid) + MotorControl[channel_number].TrgComp;
      MotorControl[channel_number].AdTrg1 = dmid + MotorControl[channel_number].TrgComp;
    }
    break;

  case CURRENT_SHUNT_3:
    {
      int32_t	trgtmp;

      if (MotorControl[channel_number].TrgComp > 0)
      {
        trgtmp = (1L<<15) + MotorControl[channel_number].TrgComp;
        if (trgtmp > (1L<<15))	trgtmp -= (1L<<16);
      }
      else
      {
        trgtmp = (1L<<15);
      }

      MotorControl[channel_number].AdTrg0 = trgtmp;
      MotorControl[channel_number].AdTrg1 = trgtmp;
    }
    break;

  default:
    break;
  }

}

static void INT_VectorControlBySoft(uint8_t channel_number)
{
/* Measure the occurance rate of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME  
    /* Store the previous start time */ 
    u32IRQLoopTimePrev[channel_number] = u32IRQLoopTimeStart[channel_number]; 
    /* Get the new start time */
    u32IRQLoopTimeStart[channel_number] = DWT_CYCCNT_VAL;
    /* Measure the interrupt loop cycle time - delta between two successive starts */
    u32IRQLoopOccuranceRate[channel_number] = u32IRQLoopTimeStart[channel_number] - u32IRQLoopTimePrev[channel_number];                             
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */
    
  if  ((MotorControl[channel_number].ActualStage.main == Stage_Initposition)
       ||(MotorControl[channel_number].ActualStage.main == Stage_Force)
         ||(MotorControl[channel_number].ActualStage.main == Stage_FOC))
  {    
    /***** Get data from ADC register *****/
    SW_FOC_InputProcessing(channel_number);

    if (MotorErrorField[channel_number].Error == 0)                             /* No error? */
    {
      /***** Input phase transformation *****/
      SW_FOC_InputCurrentConversion(channel_number);


      switch (MotorExtendedStateValues[channel_number].ControlMethod)
      {
      case MOTORCONTROL_CONTROL_BY_TORQUE:
        TorqueController_Omega_Theta(channel_number);
        TorqueController_Calculate_Id_Iq_ref(channel_number);
        break;

      case MOTORCONTROL_CONTROL_BY_SPEED:
        /***** Detect Rotor Position *****/
        SpeedController_Omega_Theta(channel_number);
        SpeedController_Calculate_Id_Iq_ref(channel_number);
        break;
      }

      /***** Current control  *****/
      SW_FOC_Control_Current(channel_number);

      /***** Output coordinate axis transformation  *****/
      SW_FOC_OutputTransformation(channel_number);

      /***** Trigger generation  *****/
      SW_FOC_TriggerGeneration(channel_number);
    }

    /***** Output control  *****/
    SW_FOC_SetPMDTriggerSelect(channel_number, MotorControl[channel_number].Sector1);
    SW_FOC_SetPMDData(channel_number, (uint16_t)MotorControl[channel_number].DutyU, (uint16_t)MotorControl[channel_number].DutyV, (uint16_t)MotorControl[channel_number].DutyW, (int16_t)MotorControl[channel_number].AdTrg0, (int16_t)MotorControl[channel_number].AdTrg1);       
  }
/* Measure the processing time of the routine */  
#ifdef MEASURE_MOTOR_CONTROL_PROCESS_TIME               
    /* Get the end processing time */
    u32IRQLoopTimeStop[channel_number] = DWT_CYCCNT_VAL;
    /* Measure the interrupt processing time - delta between the stat and end times */
    u32IRQLoopProcessingTime[channel_number] = u32IRQLoopTimeStop[channel_number] - u32IRQLoopTimeStart[channel_number];
#endif /* MEASURE_MOTOR_CONTROL_PROCESS_TIME */ 
}

void INTADBPDB_IRQHandler(void)
{
  if (MotorControl[1].ParameterChange!=0)
    MotorControl_HandleParameterChange(1);

  INT_VectorControlBySoft(1);
  __DSB();                                                                      /* for ARM processor errata 838469 */
}


void INTADCPDB_IRQHandler(void)
{
  if (MotorControl[2].ParameterChange!=0)
    MotorControl_HandleParameterChange(2);

  INT_VectorControlBySoft(2);
  __DSB();                                                                      /* for ARM processor errata 838469 */
}

