/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2012 All rights reserved
 */

#include <stdint.h>
#include <stdlib.h>

#include "config.h"
#ifdef USE_TURN_CONTROL

#include "FreeRTOS.h"
#include "task.h"

#include "motor_control.h"
#include "hal_encoder.h"
#include TMPM_HEADER_FILE
#include "fwmathlib.h"
#include "turn_control.h"

/* Time calcualtion */
#define CALC_IN_MILLISECONDS                1000
#define FULL_TURN_IN_TENTHS_Of_DEGREES      3600

/* Approach calculation - keep it low to reduces the overshoots at transitions */
#define MIN_APROACH_SPEED                   1
#define MAX_APROACH_SPEED                   40
#define APROACH_SPEED_FACTOR                15
#define DEFAULT_APPROACH_RPM                (MAX_APROACH_SPEED/APROACH_SPEED_FACTOR)

/* Low speed border conditions */
#define LOW_SPEED_TURNS_FACTOR              8
#define DEFAULT_LOW_SPEED_TURNS             1

/* Smooth acceleration */
#define SMOOTH_ACC_UP_FACTOR                (35/10)
#define MIN_SMOOTH_ACC_UP                   50
#define DEFAULT_SMOOTH_ACC_UP               80

/* Debug output */
//#define ADVANCED_TUNR_CTRL_DEBUG

#if defined __TMPM_4KN__
#include "txz_driver_def.h"
#endif


/* Forward declartaions - VE state variables */


/* Local state variables */
static int32_t MaxSpeed[MAX_CHANNEL];
static int32_t WaitValueForApproach[MAX_CHANNEL];       //Declared as signed int to support both CW and CCW
static int16_t ApproachSpeed[MAX_CHANNEL]= {DEFAULT_APPROACH_RPM, DEFAULT_APPROACH_RPM, DEFAULT_APPROACH_RPM};
static uint16_t MinLowSpeedTurns = DEFAULT_LOW_SPEED_TURNS;
static uint16_t SpeedUpTime;
static uint16_t TurnsWhileSpeedUp;
static uint16_t SpeedDownTime;
static uint16_t TurnsWhileSpeedDown;
static uint32_t u32MaxAngAcc[MAX_CHANNEL];
static TargetPosition TargetEncValue[MAX_CHANNEL];

/*! \brief  Set a new zero postion
*
* Sets the curent absolute position as new zero position and clears the encoder value
*
* @param  channel_number: channel number
* @retval None
*/
void SetAbsoluteZeroPosition(long channel_number)
{
  /* clear encoder value */
  ENC_ClearEncCounter (channel_number);
  /* sets the current position as new absolute zero position */
  MotorControl[channel_number].CurrentAbsPos = 0;
}

/*! \brief  Calculate Ramp
  *
  * Calculate the time and turns needed for the ramp slopes
  *
  * @param  channel_number: channel number to use
  * @param  max_speed: maximum rpm to reach while turning
  * @retval None
*/
void vCalcualteRampTimesAndTurns(uint8_t channel_number, uint16_t max_speed)
{
    switch( MotorParameterValues[channel_number].MaxAngAcc.unit )
    {
        case RAD_BY_S2:
            /*
                Calculate the time needed to reach target rpm in ms
                Alpha = DeltaOmega / DeltaTime => DeltaTime (T-T0) = DeltaOmega (W - W0)/ Delta Alpha,
                T0 = 0, W0 = 0, W = 2*Pi*F, F = RPM/60
                DeltaTime = (2*Pi*MaxSpeed/60) / Alpha
            */
            SpeedUpTime = (uint16_t)  ( (long long)PAI2_VALUE                           /* 2* PI */
                                                 * CALC_IN_MILLISECONDS
                                                 * max_speed
                                                 * MotorParameterValues[channel_number].PolePairs
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);

            /* Calculate the time for stopping from forced commutation stage */
            SpeedDownTime = (uint16_t)( (long long)PAI2_VALUE                           /* 2* PI */
                                                 * CALC_IN_MILLISECONDS
                                                 * abs(max_speed - ApproachSpeed[channel_number])
                                                 * MotorParameterValues[channel_number].PolePairs
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);
            break;

        case HZ_BY_S:
            SpeedUpTime = (uint16_t)  ( (long long)CALC_IN_MILLISECONDS
                                                 * max_speed
                                                 * MotorParameterValues[channel_number].PolePairs
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);

            /* Calculate the time for stopping from forced commutation stage */
            SpeedDownTime = (uint16_t)( (long long)CALC_IN_MILLISECONDS
                                                 * abs(max_speed - ApproachSpeed[channel_number])
                                                 * MotorParameterValues[channel_number].PolePairs
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);
            break;

        case RPM_BY_S:
            SpeedUpTime = (uint16_t)  ( (long long)CALC_IN_MILLISECONDS
                                                 * max_speed
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE);

            /* Calculate the time for stopping from forced commutation stage */
            SpeedDownTime = (uint16_t)( (long long)CALC_IN_MILLISECONDS
                                                 * abs(max_speed - ApproachSpeed[channel_number])
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value
                                                 / SECONDS_PER_MINUTE/10);
            break;

        case RPM_BY_MIN:
            SpeedUpTime = (uint16_t)  ( (long long)CALC_IN_MILLISECONDS
                                                 * max_speed
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value);

            /* Calculate the time for stopping from forced commutation stage */
            SpeedDownTime = (uint16_t)( (long long)CALC_IN_MILLISECONDS
                                                 * abs(max_speed - ApproachSpeed[channel_number])
                                                 / pow10(-PAI2_UNIT)                    /* normalize PI value */
                                                 / MotorParameterValues[channel_number].MaxAngAcc.value);
            break;

        default:
            assert_param(0);
            break;
    }

    /* Calculate the number of tuns during speed up - spinning the whole time with half the maximum speed */
    TurnsWhileSpeedUp   = max_speed
                          / 2
                          / SECONDS_PER_MINUTE
                          * SpeedUpTime
                          / CALC_IN_MILLISECONDS;

    /* Calculate the number of tuns during speed down - spinning the whole time with half the maximum speed */
    TurnsWhileSpeedDown = max_speed
                          / 2
                          / SECONDS_PER_MINUTE
                          * SpeedDownTime
                          / CALC_IN_MILLISECONDS;
}


/*! \brief  Calculate Turn Ramp
  *
  * Calculate the different values for the Turn Control Drive Profile
  *
  * @param  channel_number: channel number to use
  * @param  nr_turns: number of turns
  * @param  max_speed: maximum rpm to reach while turning
  * @retval None
*/
void CalculateTurnRampAndTurn(uint8_t channel_number, int16_t nr_turns, int16_t additional_angle_tenthOfDegree, uint16_t max_speed)
{
    uint16_t actualAngleInTenthOfDegree = (uint16_t)(((uint16_t)EncoderData[channel_number].Theta0.part.reg+5) * (uint64_t)3600 / FIXPOINT_16);
    int32_t newAngleInTenthOfDegree = actualAngleInTenthOfDegree - additional_angle_tenthOfDegree;
    uint16_t Turns;
      
    nr_turns *= MotorParameterValues[channel_number].PolePairs;
    Turns = abs(nr_turns);

#ifdef ADVANCED_TUNR_CTRL_DEBUG
    dprintf("Target: Speed = %d, Turns = %d,  Angle = %d\n", max_speed, nr_turns, additional_angle_tenthOfDegree);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

    /* Re-initialization of the trigger for the intial 90o compensation */

    MotorControl[channel_number].trigger_cw  = 0;
    MotorControl[channel_number].trigger_ccw = 0;

    /* Reduce the initial acceleration, backup the original parameter for restoring later on */
    u32MaxAngAcc[channel_number] = MotorParameterValues[channel_number].MaxAngAcc.value;
    if( MotorParameterValues[channel_number].MaxAngAcc.value > MIN_SMOOTH_ACC_UP )
    {
        /* ~35% reduction for smoothing the drive */
        MotorParameterValues[channel_number].MaxAngAcc.value /= SMOOTH_ACC_UP_FACTOR;
        if( MotorParameterValues[channel_number].MaxAngAcc.value < MIN_SMOOTH_ACC_UP )
        {
            /* We shall drive with ceratin minimal acceleration */
            MotorParameterValues[channel_number].MaxAngAcc.value = MIN_SMOOTH_ACC_UP;
        }
    }

    /* Backup the start angle Theta0 */
    EncoderDataTest[0].Theta0.part.reg = EncoderData[channel_number].Theta0.part.reg;

    /* Start conditions */
    StartEncValue[channel_number].ThetaDest = EncoderData[channel_number].Theta0.part.reg;
    StartEncValue[channel_number].FullTurns = EncoderData[channel_number].FullTurns;
    /* The start sector is the old end sector */
    StartEncValue[channel_number].SectorNr = EncoderData[channel_number].sector_nr;
    /* Event is used in linear motion control. Clean-up required */
    StartEncValue[channel_number].EventNr = EncoderData[channel_number].event_nr;

#ifdef ADVANCED_TUNR_CTRL_DEBUG
  dprintf("Start: Theta0 = %d, Turns = %d, Sector = %d\n", StartEncValue[channel_number].ThetaDest, StartEncValue[channel_number].FullTurns, StartEncValue[channel_number].SectorNr);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

    /* Normalize the new target angle - previous angle has to be taken into account and eventually the turn number will change */
    if( newAngleInTenthOfDegree >= FULL_TURN_IN_TENTHS_Of_DEGREES )
    {
        newAngleInTenthOfDegree -= FULL_TURN_IN_TENTHS_Of_DEGREES;
    }
    if( newAngleInTenthOfDegree < 0 )
    {
        newAngleInTenthOfDegree += FULL_TURN_IN_TENTHS_Of_DEGREES;
    }

    /* Calculate the new end position */
    TargetEncValue[channel_number].ThetaDest = (uint16_t)((uint64_t) newAngleInTenthOfDegree * FIXPOINT_16 / 3600 / MotorParameterValues[channel_number].PolePairs);
    /*
        MAX_SECTOR/2 is exactly 1/2 of 360o. Compensation for potential direction change as considered in the VE
        Only full sectors are counted, the last partial sector is the remining non-full turn fraction
    */
    TargetEncValue[channel_number].SectorNr = (int16_t)( MAX_SECTOR/2 + (int32_t) newAngleInTenthOfDegree * MotorParameterValues[channel_number].PolePairs * MAX_SECTOR / 3600 )-2;
    TargetEncValue[channel_number].FullTurns += nr_turns;

    /* Nomralize the sector number on overflow */
    if( TargetEncValue[channel_number].SectorNr >= MAX_SECTOR )
    {
        TargetEncValue[channel_number].SectorNr -= MAX_SECTOR;
    }

#ifdef ADVANCED_TUNR_CTRL_DEBUG
    dprintf("Target: Theta = %d, Physical Turns = %d, Sector = %d\n", TargetEncValue[channel_number].ThetaDest, (TargetEncValue[channel_number].FullTurns /MotorParameterValues[channel_number].PolePairs), TargetEncValue[channel_number].SectorNr);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

    /* Calculate until we reduce the maximal speed to the minimal supported - "1" */
    do
    {
        /* Remember the requested maximum speed */
        MaxSpeed[channel_number] = max_speed;

        /*
           Calcualte the approach speed as percentage of the requested maximal
           not exceeding the pre-defined maximal allowed approach speed
        */
        ApproachSpeed[channel_number] = max_speed/APROACH_SPEED_FACTOR;
        if( MAX_APROACH_SPEED < ApproachSpeed[channel_number] )
        {
            ApproachSpeed[channel_number] = MAX_APROACH_SPEED;
        }
        /* Approach speed is higher than the maximal, set it to the minimal possible */
        if( 0 == ApproachSpeed[channel_number] )
        {
            ApproachSpeed[channel_number] = 1;
        }

        /* Calcualte the times/turns to speed-up and speed-down */
        vCalcualteRampTimesAndTurns(channel_number, max_speed);

        /*
           Low speed border is the safety margin in terms of ~LOW_SPEED_FACTOR% of
           the total turns, but at least two full turns are to be made at all
        */
        MinLowSpeedTurns = nr_turns/LOW_SPEED_TURNS_FACTOR;
        if( (0 == MinLowSpeedTurns) && (1 < nr_turns) )
        {
            MinLowSpeedTurns = 1;
        }
        /* The safety margin shall not exceed the time needed to slow down */
        if( MinLowSpeedTurns > TurnsWhileSpeedDown )
        {
            MinLowSpeedTurns = TurnsWhileSpeedDown;
        }

        /* Add the safety margin */
        if( TurnsWhileSpeedDown > 0 ) TurnsWhileSpeedDown += MinLowSpeedTurns;

        /* Check if the target is achievable as calculated */
        if( (TurnsWhileSpeedUp + TurnsWhileSpeedDown + MinLowSpeedTurns) >= Turns )
        {
            /* Reduce the speed and re-check */
            max_speed /= 2;
        }
        else
        {
            /* Calculation done */
            break;
        }
    }
    while( max_speed > 1 );

    /* Calcualte the time to wait prior to stopping the motor */
    WaitValueForApproach[channel_number] = 0;

    /* Drive with maximum achievable speed only if more than one turn is to be done */
    if( 1 < Turns )
    {
        /* Number of turns is sufficient to drive fast and break gently */
        if( Turns > (TurnsWhileSpeedUp + TurnsWhileSpeedDown + MinLowSpeedTurns) )
        {
            /* Adaptation is dependent on the revolution direction */
            if( 0 > nr_turns )
            {
                /* CCW */
                WaitValueForApproach[channel_number] = TargetEncValue[channel_number].FullTurns + TurnsWhileSpeedDown;
            }
            else
            {
                /* CW */
                WaitValueForApproach[channel_number] = TargetEncValue[channel_number].FullTurns - TurnsWhileSpeedDown;
            }
        }
        else
        {
            /* We drive slow, thus reducing at the end of the wait will be sufficient to match the target position */
            WaitValueForApproach[channel_number] = TargetEncValue[channel_number].FullTurns;
        }
    }
   
    if( nr_turns < 0 )
    {
        /* Inverse the speed */
        MaxSpeed[channel_number] *= -1;
    }   

    /* Activate the task */
    MotorControl[channel_number].do_turn_control = 1;
    
    dprintf("TurnCtrl %d\n", channel_number);

    /* Re-enable the position calcualtion in the Vector Engine */
    MotorControl[channel_number].stop_calculate = 0;
}


/*! \brief  UpdateApproachSpeed
  *
  * Control upadte apporoach speed for a channel
  *
  * @param  channel_number: motor channel number
  * @param  speed: spproach speed in rpm
  * @retval None
*/
void UpdateApproachSpeed(uint8_t channel_number, uint16_t speed)
{
    ApproachSpeed[channel_number] = speed;
}
/*! \brief  Turn Task
  *
  * Control Task for reaching an absolute or relative turn position
  *
  * @param  pvParameters: None
  * @retval None
*/
void TurnTask(void* pvParameters)
{
  long channel = (long) pvParameters;
  
#ifdef ADVANCED_TUNR_CTRL_DEBUG
  int32_t encCurHalVal;
  int32_t encCurEncVal;
  int32_t encSetHalVal;
  int32_t encSetEncVal;
  int32_t deltaEncoder;
  int32_t deltaHall;
#endif

#ifdef ADVANCED_TUNR_CTRL_DEBUG
  /* We need to know the start values for the proper calculation */
  static int32_t encStartHalVal;
  static int32_t encStartEncVal;
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

  while( 1 )
  {
      if( 1 == MotorControl[channel].do_turn_control )
      {
#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("Maximal Speed = %d, Turns with 0-Max Speed = %d\n", MaxSpeed[channel], WaitValueForApproach[channel]);
          dprintf("Final Approach Speed = %d, Final Turns = %d, EventNr: %d, Theta:%d\n", ApproachSpeed[channel], TargetEncValue[channel].FullTurns, TargetEncValue[channel].EventNr, TargetEncValue[channel].ThetaDest);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */
                    
          /* Start with the requested (calculated as possible) maximal speed */
          MotorSetValues[channel].TargetSpeed = MaxSpeed[channel];

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("##Maximal Speed##\n");
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

          /* Wait to reach shutdown to forced commutation value */
           if (MotorSetValues[channel].TargetSpeed >= 0)
           {
              while( (EncoderData[channel].FullTurns < WaitValueForApproach[channel]) && (MotorControl[channel].do_turn_control==1) )
              {
                  vTaskDelay( 1 / portTICK_RATE_MS );
              }
           }
           else
           {
              while( ((EncoderData[channel].FullTurns) > (WaitValueForApproach[channel])) && (MotorControl[channel].do_turn_control==1) )
              {
                  vTaskDelay( 1 / portTICK_RATE_MS );
              }
          }

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("##Maximal Speed End##\n");
          encCurHalVal = -ENC_GetEncCounter(0);
          encCurEncVal = -ENC_GetEncCounter(2);
          encSetHalVal = (WaitValueForApproach[channel] * MotorParameterValues[0].EncoderResolution);
          encSetEncVal = (WaitValueForApproach[channel] * MotorParameterValues[2].EncoderResolution / MotorParameterValues[2].PolePairs);
          deltaEncoder = (encCurEncVal + encStartEncVal) - encSetEncVal;
          deltaHall = (encCurHalVal + encStartHalVal) - encSetHalVal;
          dprintf("Set Hall: %d, Current Hall: %d, Delta Hall: %d\n", encSetHalVal, (encCurHalVal + encStartHalVal), deltaHall);
          dprintf("Set Encoder: %d, Current Encoder: %d, Delta Encoder: %d\n", encSetEncVal, (encCurEncVal + encStartEncVal), deltaEncoder);
          dprintf("APPROACH_SPEED: %d\n",ApproachSpeed[channel]);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

          /* Limit speed to approach level */
          if( 1 == EncoderData[channel].CW )
          {
              MotorSetValues[channel].TargetSpeed = ApproachSpeed[channel];
          }
          else
          {
              MotorSetValues[channel].TargetSpeed = -ApproachSpeed[channel];
          }

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("##TURN TASK - Approach Speed##\n");
          dprintf("Turns: %d, Current Turn: %d\n", TargetEncValue[channel].FullTurns, EncoderData[channel].FullTurns);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

          /* Wait for reaching destination */
          if (EncoderData[channel].CW == E_ENC_ROTATE_CLOCKWISE)
          {
              while( (EncoderData[channel].FullTurns < TargetEncValue[channel].FullTurns-1) && (MotorControl[channel].do_turn_control==1) )
              {
                vTaskDelay( 1 / portTICK_RATE_MS );
              }
          }
          else
          {
              while( ((EncoderData[channel].FullTurns) > (TargetEncValue[channel].FullTurns-1)) && (MotorControl[channel].do_turn_control==1) )
              {
                vTaskDelay( 1 / portTICK_RATE_MS );
              }              
          }

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("##TURN TASK - Approach Speed End##\n");
          encCurHalVal = - ENC_GetEncCounter(0);
          encCurEncVal = - ENC_GetEncCounter(2);
          encSetHalVal = (TargetEncValue[channel].FullTurns * MotorParameterValues[0].EncoderResolution);
          encSetEncVal = (TargetEncValue[channel].FullTurns * MotorParameterValues[2].EncoderResolution / MotorParameterValues[2].PolePairs);
          deltaEncoder = (encCurEncVal + encStartEncVal) - encSetEncVal;
          deltaHall = (encCurHalVal + encStartHalVal) - encSetHalVal;
          dprintf("Set Hall: %d, Current Hall: %d, Delta Hall: %d\n", encSetHalVal, (encCurHalVal + encStartHalVal), deltaHall);
          dprintf("Set Encoder: %d, Current Encoder: %d, Delta Encoder: %d\n", encSetEncVal, (encCurEncVal + encStartEncVal), deltaEncoder);
          dprintf("##TURN TASK - Final Angle##\n");
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

          /* Wait for reaching destination */
           if (EncoderData[channel].CW == E_ENC_ROTATE_CLOCKWISE)
           {
              while( (EncoderData[channel].sector_nr != TargetEncValue[channel].SectorNr) && (MotorControl[channel].do_turn_control==1) 
                     && (EncoderData[channel].FullTurns < TargetEncValue[channel].FullTurns+1) )
              {
                  vTaskDelay( 1 / portTICK_RATE_MS );
              }
           }
           else
           {
              while( (EncoderData[channel].sector_nr != TargetEncValue[channel].SectorNr) && (MotorControl[channel].do_turn_control==1) 
                     && (EncoderData[channel].FullTurns > TargetEncValue[channel].FullTurns+1) )
              {
                  vTaskDelay( 1 / portTICK_RATE_MS );
              }
           }

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("##TURN TASK - Final Angle End##\n");
          encCurHalVal = - ENC_GetEncCounter(0);
          encCurEncVal = - ENC_GetEncCounter(2);
          dprintf("Current Hall: %d\n", encCurHalVal);
          dprintf("Current Encoder: %d\n", encCurEncVal);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

          /* Reset the target speed to 0 - if someone has interrupted the turn control by stopping the motor */
          MotorSetValues[channel].TargetSpeed = 0;
          while( 0 != MotorControl[channel].Omega.part.reg )
          {
              vTaskDelay(5 / portTICK_RATE_MS );
          }

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          dprintf("##TURN TASK - Speed 0##\n");
          encCurHalVal = -ENC_GetEncCounter(0);
          encCurEncVal = -ENC_GetEncCounter(2);
          dprintf("Current Hall: %d\n", encCurHalVal);
          dprintf("Current Encoder: %d\n", encCurEncVal);
#endif /* ADVANCED_TUNR_CTRL_DEBUG */

#ifdef ADVANCED_TUNR_CTRL_DEBUG
          /* Needed for the debug output */
          encStartEncVal += encCurEncVal;
          encStartHalVal += encCurHalVal;
#endif /* ADVANCED_TUNR_CTRL_DEBUG */          

          MotorControl_SetTheta(channel, TargetEncValue[channel].ThetaDest - FIXPOINT_16/2);
          vTaskDelay( 1500 / portTICK_RATE_MS );

          /* Task completed. Revert to wait for antoher activation */
          MotorControl[channel].do_turn_control = 0;

          dprintf("FinalValue: %d\n",-ENC_GetEncCounter(2));
          vTaskDelay( 1500 / portTICK_RATE_MS );

          /* Restore the original acceleration */
          MotorParameterValues[channel].MaxAngAcc.value = u32MaxAngAcc[channel];
        }

        vTaskDelay( 50 / portTICK_RATE_MS );
    }
}
#endif /* USE_TURN_CONTROL */
