/* THE SOURCE CODE AND ITS
 * RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA CORPORATION MAKES NO OTHER
 * WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR, STATUTORY AND DISCLAIMS ANY
 * AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY, NON
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. THE SOURCE CODE AND
 * DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION RESERVES THE RIGHT TO
 * INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER REVISIONS OF IT, AND TO
 * MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR THE PRODUCTS OR
 * TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME. TOSHIBA CORPORATION SHALL NOT BE
 * LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGE OR LIABILITY ARISING
 * FROM YOUR USE OF THE SOURCE CODE OR ANY DOCUMENTATION, INCLUDING BUT NOT
 * LIMITED TO, LOST REVENUES, DATA OR PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL
 * OR CONSEQUENTIAL NATURE, PUNITIVE DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS
 * ARISING OUT OF OR IN CONNECTION WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF
 * ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM
 * FOR SUCH DAMAGE IS BASED UPON WARRANTY, CONTRACT, TORT, NEGLIGENCE OR
 * OTHERWISE. (C)Copyright TOSHIBA CORPORATION 2011-2020 All rights reserved
 */

#ifndef _MOTOR_CONTROL_DEFINE_H_
#define _MOTOR_CONTROL_DEFINE_H_

#include "config.h"
#include "api.h"
#include "FreeRTOS.h"
#include "task.h"
#include "hal_encoder.h"

/* The refernce resolution - currently Line Encoder connected on channel two - shall be replaced by MotorParameterValues[channel].EncoderResolution  */
#define LINE_ENCODER_RESOLUTION     2000
/* Hugo shall use "-1" for direction normalization, due to the different encoding */
#define LINE_ENCODER_DIR_ADAPTATION 1

/*! \brief Channel usage
 *
 *   Individual channels may be configured for usage or kept unused
 */
typedef enum ChannelUsageSettingType
{
  CHANNEL_NOT_USED = 0,
  CHANNEL_IS_USED = 1
} ChannelUsageSetting;

/*! \brief Vector Engine Main stages
 *
 *  The different stages the Vector Engine passes till reaching target speed
 */
enum _mainstage
{
  Stage_Stop,                                                                   /* control loop is off, no actions pending */
  Stage_Bootstrap,                                                              /* bootstrap IGBTs */
  Stage_Brake,                                                                  /* shortbrake Motor */
  Stage_ZeroCurrentMeasure,                                                     /* measure zero current adc value */
  Stage_Initposition,                                                           /* control loop is off, first signals out and in */
  Stage_Force,                                                                  /* forced commutation */
  Stage_FOC,                                                                    /* Field Oriented Commutation */
  Stage_Emergency                                                               /* emergency state */
};

/*! \brief Substages
 *
 *  The different substages inside the mainstages
 */
enum _substage {
  Substage_Step0,
  Substage_Step1,
  Substage_Step2,
  Substage_StepEnd
};

/*! \brief Vector Engine Actual State
 *
 *  Major and minor actual stage
 */
typedef struct
{
  enum _mainstage main;
  enum _substage  sub;
} MotorControl_Stage;


/*! \brief Avaliable vector engine channels.
 *
 *  This enumeration is for selecting the available channels for the board.
 */
typedef enum MOTORCONTROL_CHANNELS{
  MOTORCONTROL_CHANNEL_0  = (1<<0),                                             /*!< Channel 0 is available */
  MOTORCONTROL_CHANNEL_1  = (1<<1),                                             /*!< Channel 1 is available */
  MOTORCONTROL_CHANNEL_2  = (1<<2),                                             /*!< Channel 2 is available */
  MOTORCONTROL_CHANNELS_END = (1<<3),                                           /*!< Mark end of Channel entries */
} MOTORCONTROL_CHANNELS;

/*! \brief Control methods
 *
 *  This enumeration is for selecting the current control method.
 */
typedef enum tag_motorcontrol_method
{
  MOTORCONTROL_CONTROL_BY_SPEED  = (1<<0),                                      /*!< The motor is controlled by speed */
  MOTORCONTROL_CONTROL_BY_TORQUE  = (1<<1),                                     /*!< The motor is controlled by torque */
} motorcontrol_method;

extern uint8_t GetControlMethodCommand(uint8_t channel_number);
extern uint8_t SetControlMethodCommand(uint8_t channel_number, uint8_t control_method);

/*! \brief Parameter Changed Flags.
 *
 *  This enumeration is for the different uploaded parameter sets
 */
typedef enum {
  MOTORCONTROL_CHANGE_MOTOR_PARAMS      = (1<<0),                               /*!< New motor parameter set */
  MOTORCONTROL_CHANGE_PI_PARAMS         = (1<<1),                               /*!< New PI control parameter set */
  MOTORCONTROL_CHANGE_SYSTEM_PARAMS_VE  = (1<<2),                               /*!< New system values parameter set */
  MOTORCONTROL_CHANGE_SYSTEM_PARAMS_PMD = (1<<3),                               /*!< New system values parameter set */
  MOTORCONTROL_CHANGE_BOARD_PARAMS_VE   = (1<<4),                               /*!< New system values parameter set */
  MOTORCONTROL_CHANGE_BOARD_PARAMS_PMD  = (1<<5),                               /*!< New system values parameter set */
} MOTORCONTROL_CHANGED_VALUE;

#ifdef FIX_155_PMD_DELAY

typedef enum
{
  PMD_OutputOff_EmergencyDisable  = 0x01,
  PMD_NormalOutput_VE_PMD_Output  = 0x02,
  PMD_OutputPattern               = 0x03,
  PMD_NormalOutput                = 0x04,
  PMD_EmergencyReset_Initposition = 0x05,
  PMD_OvervoltageReset_Initposition = 0x06,
  PMD_EmergencyReset_Force        = 0x07,
  PMD_OvervoltageReset_Force      = 0x08,
}PMD_Delay_Enum;

extern uint8_t gPmdRegisterSetting[3][8];
#endif

int8_t MotorControl_Start(uint8_t channel_number);
int8_t MotorControl_Stop (uint8_t channel_number);

/* global data */

/*! \brief Precalculated Values
 *
 *  Struct holding some precalculated values due to run time optimisation
 */
typedef struct
{
  uint16_t   WaitTime_Position;                                                 /* ms */
  uint16_t   WaitTime_Bootstrap;                                                /* ms */
  uint16_t   WaitTime_Brake;                                                    /* ms */
  uint32_t   R_mult_amax_div_vmax;                                              /* fixpoint15 calculation R*I/U */
  uint32_t   InductanceDropoutFactor;                                           /* fixpoint15 calculation 2*PI*I*Speed*1000/U */
  uint32_t   PositionKiPreCalculation;                                          /* fixpoint15 calculation Ki*Tt*U/f */
  uint32_t   PositionKpPreCalculation;                                          /* fixpoint15 calculation Kp*U/f */
  uint32_t   SpeedKiPreCalculation;
  uint32_t   SpeedKpPreCalculation;
  uint32_t   HzMax_normed_to_PwmFrequency;                                      /* fixpoint   calcualtion HzMax normalized to PwmFrequency */
  uint32_t   SpeedUpLimit;                                                      /* speed up limit of the motor */
  uint16_t   ChangeFrqUp_normed_to_HzMax;                                       /* fixpoint15 calculation HzChange normalized to HzMax */
  uint16_t   ChangeFrqDown_normed_to_HzMax;                                     /* fixpoint15 calculation HzChange normalized to HzMax */
  uint16_t   IqLimit_normed_to_amax;                                            /* fixpoint15 calculation IqLimit normalized to a_max */
  uint16_t   IdLimit_normed_to_amax;                                            /* fixpoint31 calculation IdLimit normalized to a_max */
  uint16_t   IdCurrentForInitposition;                                          /* Id current for reaching the initposition */
  uint16_t   IqCurrentForInitposition;                                          /* Iq current for reaching the initposition */
  uint32_t   SWOvercurrentSquare;                                               /* precalculation value for sw overcurrent detection */
} PreCalc;                                                                      /* set of pre-calculations for control loop */

/*! \brief Motor Control values
 *
 *
 */
typedef struct
{
  int16_t               Id;                                                     /* [A/maxA] d-axis current */
  int16_t               Iq;                                                     /* [A/maxA] q-axis current */
  int16_t               Id_reference;                                           /* [A/maxA] Reference of d-axis c */
  int16_t               Iq_reference;                                           /* [A/maxA] Reference of q-axis c */
  FRACTIONAL            Omega;                                                  /* [Hz/maxHz] Omega(speed): Elect */
  FRACTIONAL            Theta;
  FRACTIONAL            Omega_command;
  PreCalc               PreCalculation;
  uint16_t              HzMax;
  int16_t               OmegaCalc;
  uint8_t               PerformShutdown;
  UNIT_VALUE            a_max;
  UNIT_VALUE            v_max;
  MotorControl_Stage    ActualStage;                                            /*< Actual stage */
  uint8_t               LastDirection;
  uint16_t	        StageCounter;                                           /*< Stage counter (for reaching the above timings) */
  FRACTIONAL            Vd;						        /* [V/maxV] d-axis voltage */
  FRACTIONAL            Iq_reference_I;                                         /*< reference command current */
  int16_t               Omega_Target;                                           /*< [Hz/maxHz] OMEGA Target */
  xTaskHandle           xVETask;                                                /*< VE Task Handle */
  xTaskHandle           xShutdownHandle;                                        /*< Local storage of VE Task Handle needed for gentle shutdown */
  uint8_t               ParameterChange;
  uint32_t              PerformShutdownTargetSpeed;                             /*< Remember Target Speed before Shutdown has initiated for recover */
//#ifdef USE_MOTION_CONTROL
  uint8_t               IsStarted;
//#endif /* USE_MOTION_CONTROL */
#ifdef USE_LOAD_DEPENDANT_SPEED_REDUCTION
  int32_t               TargetSpeed_Backup;
#endif /* USE_LOAD_DEPENDANT_SPEED_REDUCTION */
#ifdef USE_MOTOR_DISCONNECT_DETECTION
  uint16_t              disconnect_counter;
#endif /* USE_MOTOR_DISCONNECT_DETECTION */
  uint8_t               do_turn_control;
  uint8_t               do_pulses_control;
  uint32_t              CurrentAbsPos;                                            /* Absolute position information per channel */

  /* variables needed for SW FOC */
  int32_t       Ia0;                                                            /* [A/maxA] a phase average of zero current */
  int32_t       Ib0;                                                            /* [A/maxA] b phase average of zero current */
  int32_t       Ic0;                                                            /* [A/maxA] c phase average of zero current */
  int32_t       Ia;                                                             /* [A/maxA] a phase current */
  int32_t       Ib;                                                             /* [A/maxA] b phase current */
  int32_t       Ic;                                                             /* [A/maxA] c phase current */
  int32_t       Ia_adc;                                                         /* [A/maxA] U phase current ADC result */
  int32_t       Ib_adc;                                                         /* [A/maxA] V phase current ADC result */
  int32_t       Ic_adc;                                                         /* [A/maxA] W phase current ADC result */
  int32_t       Ia_raw;                                                         /* [A/maxA] a phase current Raw data */
  int32_t       Ib_raw;                                                         /* [A/maxA] b phase current Raw data */
  int32_t       Ic_raw;                                                         /* [A/maxA] c phase current Raw data */
  int16_t       Ialpha;                                                         /* [A/maxA] alpha-axis current */
  int16_t       Ibeta;                                                          /* [A/maxA] beta-axis current */
  int16_t       Vdc;                                                            /* [V/maxV] DC voltage ADC value */
  FRACTIONAL    Vq;						                /* [V/maxV] q-axis voltage */
  FRACTIONAL    Valpha;                                                         /* [V/maxV] alpha-axis voltage	*/
  FRACTIONAL    Vbeta;                                                          /* [V/maxV] beta-axis voltage	*/
  FRACTIONAL    Va;
  FRACTIONAL    Vb;
  FRACTIONAL    Vc;
  int64_t       Vdi;
  int64_t       Vqi;
  uint16_t      DutyU;                                                          /* U-phase PWM duty ratio */
  uint16_t      DutyV;                                                          /* V-phase PWM duty ratio */
  uint16_t      DutyW;                                                          /* W-phase PWM duty ratio */
  uint8_t       Sector;                                                         /* Sector */
  uint8_t       Sector1;                                                        /* One previous sector */
  int16_t       AdTrg0;                                                         /* AD trigger timing 0 */
  int16_t       AdTrg1;                                                         /* AD trigger timing 1 */

  uint8_t       modulation;
  int16_t       TrgComp;
  uint8_t       idetect_error;                                                  /* current detect error state */

  uint8_t       Initposition_Done;
  uint8_t       trigger_cw;
  uint8_t       trigger_ccw;
  uint8_t       stop_calculate;

  deadtime_t    Deadtime;
} MotorControlData;

typedef struct TargetPositionTag
{
  int32_t FullTurns;
  int16_t SectorNr;
  int16_t EventNr;
  uint16_t ThetaDest;
} TargetPosition;


/* GLOBAL Variables */
extern MotorSetSettings              MotorSetValues[MAX_CHANNEL];
extern MotorExtendedStateSettings    MotorExtendedStateValues[MAX_CHANNEL];
extern MotorParameters               MotorParameterValues[MAX_CHANNEL];
extern MotorError                    MotorErrorField[MAX_CHANNEL];
extern PIControlSettings             PIControl[MAX_CHANNEL];
extern ChannelDependandValues        ChannelValues[MAX_CHANNEL];
extern SystemDependandValues         SystemValues[MAX_CHANNEL];
extern MotorControlData              MotorControl[MAX_CHANNEL];
extern TurnStateSettings             TurnStateValues[MAX_CHANNEL];
extern TargetPosition                StartEncValue[MAX_CHANNEL];
extern EncoderValues                 EncoderDataTest[MAX_CHANNEL];                                 /*<! generic encoder result data */
extern ChannelUsageSetting           ChannelUsageConfiguration[MAX_CHANNEL];



void MotorControl_HandleParameterChange(unsigned char channel_number);
void MotorControl_SetTheta(unsigned char channel_number,uint16_t Theta);
#endif // _MOTORCONTROL_DEFINE_H_
